#!/usr/bin/perl
#
# Cobalt::UI
# 	useful ui utility functions to make substitutions in templates
#	and get the Locale based information
#	I use them in a couple of ui modules.
#
# Copyright 2000 Cobalt Networks Inc
#

package Cobalt::UI;

require Exporter;
use vars qw( @ISA @EXPORT_OK );

@ISA = qw(Exporter);
@EXPORT_OK = qw( get_style_info output_page );

use strict;
use Cobalt::Locale;
use Locale::gettext;

# sets up the LANGUAGE environment variable and returns some style info
# for the current locale
# in list context:
# it returns a hash reference to style info for the current
# locale (currently only button, header, and table) and a reference to the
# locale object used to get the information
#
# in scalar context:
# it returns just the hash reference to the style info
sub get_style_info
{
	my $style;
	my $locale = Cobalt::Locale->getByHTTPHeader( 
					$ENV{ 'HTTP_ACCEPT_LANGUAGE' } );

	if( $locale ) { 
		$ENV{ 'LANGUAGE' } = $locale->getLocale();
                $style->{button} = $locale->getProperty("buttonStyle");
                $style->{header} = $locale->getProperty("headerStyle");
                $style->{table} = $locale->getProperty("tableStyle"); 
	}

	if (wantarray) {
		return ($style, $locale);
	}
	
	return $style;
}

sub output_page
{
	# filename of template
	my $template = shift;  

	# hash reference, keys are SUB_ME, values are the replacement
	my $subs = shift;  

	# get the header to use if specified
	my $header = shift;

	# get our style info (maybe this should be passed in)
	my ($style, $locale) = get_style_info();

	open(TMPL, $template) || die "Could not read template $template: $!";

	if ($header) {
		print $header;
	} else {
	        my $charset = $locale->getProperty("http_Charset");
		my $charsetText;
		if ($charset) {
		    $charsetText = "; charset=$charset";
		}
		print "Content-type: text/html$charsetText \nCache-Control: no-cache\n\n";
		undef($charset, $charsetText);
	}

	while(<TMPL>) {
		# need to loop over the line read because there may be
		# tags introduced by substituting for another tag
		while (/\[([^]]+)\]/o) {
			my $match = $1;
			my $line = $_;

			if (exists $subs->{$match}) { s/\[$match\]/$subs->{$match}/; }
			# do the default substitutions that happen for every page
			s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
			s/\[FONTH\]/$style->{header}/g;
			s/\[FONTT\]/$style->{table}/g;
			s/\[FONTB\]/$style->{button}/g;
		
			# check if anything has changed
			if ($line eq $_) {
				# unknown tag just go to the next line
				last;
			}
		}

		print;
	}
	close TMPL;
}

# return a button object
# how UIFC like
sub button
{
	my $info = shift;

	my $button = <<BUTTON;

<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0">
<TR>
	<TD><A HREF="$info->{action}" onMouseOver="status='$info->{help}'; return true;"><IMG SRC="/libImage/buttonLeftDot.gif" BORDER=0></A></TD>
	<TD NOWRAP STYLE="[FONTB]"><A HREF="$info->{action}" onMouseOver="status='$info->{help}'; return true;"><FONT STYLE="[FONTB]">$info->{text}</FONT></A></TD>
	<TD><A HREF="$info->{action}" onMouseOver="status='$info->{help}'; return true;"><IMG SRC="/libImage/buttonRight.gif" BORDER=0></A></TD>
</TR>
</TABLE>
BUTTON

	return $button;
}

1;
