#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 3.0
#
# Version 2.0 uses Monitor.pm instead of AlertDB.pm
#
# date: 6-28-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::alert;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::swatch;
use Cobalt::RAID;
use Cobalt::User;

use Locale::gettext;

my ( $q, $locale, $page, $button_style, $header_style, $table_style);
my $fans_enabled = 0;			# Are we on >= Monterey hardware?
my $alertScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/alert';

sub main {

# global variables
$q=shift;

# the page that run this script
$page=$q->param('page');

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); 
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthoried access by $user, not an admin.\n";
}

# get ball colors
my (%balls,$defcon,$obj,@items,$comp,$service,$item);

# cpu
$balls{"cpu"} = $defcon_images{ swatch_get_defcon("cpu") };
# memory
$balls{"memory"} = $defcon_images{ swatch_get_defcon("memory") };
# network
$balls{"network"} = $defcon_images{ 1 };
# disk usage
$balls{"disk"} = $defcon_images{ swatch_get_defcon_disk() };
# fan health
#if /proc/faninfo contains "monitoring not supported" we're on old hardware
open(FANINFO, "/proc/faninfo") or die "Cannot open /proc/faninfo: $!\n";
while (<FANINFO>) {
	if ($_ =~ m/monitoring not supported/) {
		$fans_enabled = 0
	} else {
		$fans_enabled = 1;
		$balls{"fans"} = $defcon_images{ swatch_get_defcon("fans") };
	}
}

# RAID "disk integrity"
my $raid_enabled;
open(MOUNTS, "/bin/mount |");
while(<MOUNTS>) {
    if(/md1 on \/ /) {
        $raid_enabled = 1;
        last;
    }
}
close(MOUNTS);

my $raid_line;
if ($raid_enabled) {
    my $raid_state = (&raid_get_state)[0];
    $raid_state ||= 3;
    $raid_state = 3 if ($raid_state == 4); # generalize error state
    $raid_state = 3 if ($raid_state == 5); # ditto
    $balls{"raid"} = $defcon_images{ $raid_state };

    $raid_line = <<EOF;
        <TR>
                <TD WIDTH="25"><P ALIGN="CENTER">[RAID]</TD>
                <TD WIDTH="375"><FONT STYLE="[FONTT]">&nbsp;&nbsp;[str.diskraid]</FONT></TD>
		            <TD WIDTH="50"><P ALIGN="CENTER"><A HREF="/cgi-bin/.cobalt/turbo_ui/dispatch/raidUsage?alert=t"><IMG ALT="[str.diskraid]" BORDER="0" SRC="/libImage/detail.gif"></A></TD>
        </TR>
EOF
} 

# now do services
my @services = ('http', 'ftp', 'snmp', 'dns', 'casp', 'telnet');
foreach $service (@services) {
    $balls{ $service } = $defcon_images{ swatch_get_defcon("service", $service) };
}
# don't forget email
$balls{"email"} = $defcon_images{ swatch_get_defcon_email() };

# Read in the template
open( TEMPLATE, "$Template_dir/alert.tmpl" )
    || die "Cannot read $Template_dir/alert.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


while ( <TEMPLATE> ) {
    s/\[RAIDLINE\]/$raid_line/;

    s/\[CPU\]/$balls{'cpu'}/;
    s/\[MEMORY\]/$balls{'memory'}/;
    s/\[DISK\]/$balls{'disk'}/;
    s/\[RAID\]/$balls{'raid'}/;
    s/\[NETWORK\]/$balls{'network'}/;

    if ($fans_enabled) {
	s/\[HIDEFANSSTART\]//o;
	s/\[HIDEFANSEND\]//o;
	s/\[FANS\]/$balls{'fans'}/;
    } else {
	s/\[HIDEFANSSTART\]/<!-- /o;
	s/\[HIDEFANSEND\]/ -->/o;
    }

    s/\[FTP\]/$balls{'ftp'}/;
    s/\[EMAIL\]/$balls{'email'}/;
    s/\[TELNET\]/$balls{'telnet'}/;
    s/\[HTTP\]/$balls{'http'}/;
    s/\[SNMP\]/$balls{'snmp'}/;
    s/\[DNS\]/$balls{'dns'}/;
    s/\[CASP\]/$balls{'casp'}/;

    s/\[DEFCON0\]/$defcon_images{ 0 }/o;
    s/\[DEFCON1\]/$defcon_images{ 1 }/o;
    s/\[DEFCON2\]/$defcon_images{ 2 }/o;
    s/\[DEFCON3\]/$defcon_images{ 3 }/o;

    s/\[FONTH\]/$header_style/g;
    s/\[FONTT\]/$table_style/g;
    s/\[FONTB\]/$button_style/g;

    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

    print;
}

close TEMPLATE;

}

1;

