#!/usr/bin/perl
#
# Adrian Sun <asun@cobaltnet.com>
# Copyright (c) 1999 Cobalt Networks <http://www.cobaltnet.com>
#
# inspired by siteUserAdd.cgi

package Cobalt::UI::diskAdd;

use Cobalt::Product;
use Cobalt::DiskUtils;
use Cobalt::Util;
use Cobalt::Locale;
use Cobalt::User;

use Locale::gettext;
Locale::gettext::textdomain("ui");

my ($q, $locale, $page);
my $MainDisk = "/dev/hda";
my $KBperGB = 1024*1024;
my $diskListPageURL="/cgi-bin/.cobalt/turbo_ui/dispatch/diskList";
my $diskAddScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/diskAdd";
my $error_noadd;
my $error_noent;
my $error_badname;
my $error_exist;
my $error_locked;
my $error_nodisks;
my $error_mountpoint;

my $note_part_exists;

my $ide_msg;
my $scsi_msg;
my $md_msg;
my $unknown_msg;
my ($table_style, $header_style, $button_style);

sub main {

Locale::gettext::textdomain("ui");
$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthoried access by $user, not an admin.\n";
}

$error_noadd = MSG_get_ui("volErrNoAdd") . "', 'error";
$error_noent = MSG_get_ui("volErrNoEnt") . "', 'error";
$error_badname = MSG_get_ui("volErrBadName") . "', 'error";
$error_exist = MSG_get_ui("volErrExist") . "', 'error";
$error_locked = MSG_get_ui("volErrStorageBusy") . "', 'error";
$error_nodisks = MSG_get_ui("volErrNoDisks") . "', 'error";
$error_mountpoint = MSG_get_ui("volErrMountPoint") . "', 'error";

$note_part_exists = MSG_get_ui("volPartitionExist");

$ide_msg = MSG_get_ui("IDE");
$scsi_msg = MSG_get_ui("SCSI");
$md_msg = MSG_get_ui("RAID");
$unknown_msg = MSG_get_ui("volErrUnknown");

my $info;

# figure out where we're from
$q=shift;
$page = $q->param('page');

if( $ENV{ PARAM_INFO } ) {
    $info=$ENV{ PARAM_INFO };
} else {
    $info=$q->param( 'info' );
}

if (disk_checklock() or (($page eq 'diskAdd') and not 
		       ($info = handleDiskAddPage($q)))) {
    listpage($error_locked);
} else {
    printDiskAddPage( \*STDOUT, $info );
}

}


# return to diskListPage with a message.
sub listpage
{
    my $info = shift;
    print "location: $diskListPageURL\?info=$info\n\n";
}

#
# event handler for add user page. this needs to loop through
# all of the available disks. it needs to check a few things first
# and then send everything into the background.
#
sub handleDiskAddPage 
{
    my $q = shift;
    
    return "No CGI specified." unless defined($q);
    
    my %to_format;
    my @disks = split(",", $q->param('devices'));
    my $disk;

    # okay. loop through all of the options and make sure we have
    # something useful to do. if not, release the lock and spit
    # out an appropriate message. 
    # note: some of this is already checked by the javascript stuff.
    foreach $disk (@disks) {
	if ($q->param("add:$disk")) { # add this disk
	    my $name = $q->param("name:$disk");

	    # error if no name
	    return $error_noent unless (defined($name) and $name ne "");
	    # error if illegal name. in the future, only the / 
	    # character should be illegal.
	    return $error_badname unless $name =~ /^[\w\d\-\.]+$/;
	    # error if name is the same as /home
	    return $error_exist if $name eq $Vol_home; 
	    # error if directory already exists
	    return $error_exist if -d "$Vol_rootdir/$name";

	    # add to list
	    $to_format{$disk}{Name} = $name;
	    $to_format{$disk}{Check} = $q->param("format:$disk");
	} 
    }

    # also checked by javascript stuff.
    return $error_noadd unless %to_format;

    # lock things up
    return $error_locked if disk_lock;

    # make sure that we can use the necessary directories
    unless (disk_mountok($Vol_diskdir) and disk_mountok($Vol_rootdir)) {
	disk_unlock;
	return $error_mountpoint;
    }

    # now fork and format everything. 
    my $pid;
    FORK: {
	if ($pid = fork) {
	    listpage($error_locked);
	    exit 0;
	} elsif (defined $pid) {
	    last;
	} elsif ($! =~ /No more process/) {
	    sleep 5;
	    redo FORK;
	} else {
	    disk_unlock;
	    return "Can't fork: $!\n";
	}
    }

    # child process
    close(\*STDOUT);
    # close(\*STDERR);
    foreach $disk (keys %to_format) {
	# try to partition
	next if disk_partition($disk);
	my $label = $to_format{$disk}{Name};
	$disk .= "1";
	next if disk_format($disk, $label, $to_format{$disk}{Check});
	disk_mount($disk, "nocheck");
    }
    disk_unlock;
    exit 0; # the parent process handled the connection.
}

sub parsediskinfo
{
    my ($dev, $size, $warning) = @_;

    return {
	Size => sprintf("%.3f GB",$size/$KBperGB),
	Interface => ($dev =~ /^\/dev\/h/) ? $ide_msg : 
	    (($dev =~ /^\/dev\/s/) ? $scsi_msg : 
	     (($dev =~ /^\dev\/m/) ? $raid_msg : $unknown_msg)),
	Warning => "$warning"
    };
}

sub printDiskAddPage {
    my ( $handle, $info )=@_;
    my %umounts = disk_getumounts;
    my %unpartitioned = disk_getunpartitioned;
    my %partitions = disk_getpartitions;
    my %raidparts = disk_getraidparts;
    my (%disks, %parts);
    my ($key, $keys);

    # include unpartitioned media 
    foreach $key (keys %unpartitioned) {
	$disks{$key} = parsediskinfo($key, $partitions{$key});
    }

    # figure out how many partitions per disk exist for unmounted
    # partitions.
    $keys = join(",", keys %umounts);
    foreach $key (keys %partitions) {
	next unless $keys =~ /$key/;
	next if $key =~ /^$MainDisk/o;
	$key =~ s/\d+$//;
	$parts{$key} += 1;
    }
    
    # include the disks that have multiple partitions.
    foreach $key (keys %parts) {
	$disks{$key} = parsediskinfo($key, $partitions{$key}, 
				     $note_part_exists)
	    if ($parts{$key} > 2);
    }
    
    # try to mount the rest. to speed things along, this just quickly
    # tries to mount/unmount a partition without checking.
    $keys = join(",", keys %disks);
    foreach $key (keys %umounts) {
	my ($mountpoint, $warning);
	my $disk = $key;

	# skip ones that we've already done. also skip anything
	# on our primary disk.
	$disk =~ s/\d+$//;
	next if ($keys =~ /$disk/ or $disk =~ /^$MainDisk/o);
	next if ($raidparts{$disk});
	
        # maybe it's just temporarily unmounted. deal with that.
	# note: this gets handled by diskList.cgi as well.
	if ($mountpoint = 
	  disk_mount($key, "ro,nocheck")) {
	    disk_umount($mountpoint);
	    next;
	}

	# see if it's an okay partition. we'll tack on an extra
	# warning message if that's true.
	if ($mountpoint = 
	  disk_mount($key, "ro,nocheck,noenable")) {
	    disk_umount($mountpoint);
	    $warning = $note_part_exists;
        }
	
	$disks{$disk} = parsediskinfo($disk, $partitions{$disk}, $warning);
    }
    
    # now spit everything back out.

    my @keys = sort keys %disks;
    if (not @keys) {
	listpage($error_nodisks);
	return;
    }

    $keys = join(',', @keys);

    # the list of devices
    my $disklist =<<END;
<INPUT TYPE="HIDDEN" NAME="devices" SIZE="-1" VALUE="$keys">
END

    foreach $key (@keys) {
	$disklist .=<<END;
<TR ALIGN=CENTER>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><INPUT TYPE=CHECKBOX NAME="add:$key" CHECKED></TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><INPUT TYPE=CHECKBOX NAME="format:$key" SIZE="25"></TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><INPUT TYPE=TEXT NAME="name:$key" SIZE="25"></TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$disks{$key}{Interface}</TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$disks{$key}{Size}</TD>
<TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$disks{$key}{Warning}</TD>
</TR>
END
    }

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    open(TEMPLATE, "$Template_dir/diskAdd.tmpl") or 
	die "Can't read $Template_dir/diskAdd.tmpl: $!\n";

    while (<TEMPLATE>) {
	s/\[DISK_LIST\]/$disklist/;
	s/\[DISK_INFO\]/$info/;
	s/\[DISKLISTPAGEURL\]/$diskListPageURL/o;
	s/\[DISKADDSCRIPTURL\]/$diskAddScriptURL/o;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/ge;
	print $handle $_;
    }
    close(TEMPLATE);
}
1;
