#!/usr/bin/perl
#
# Adrian Sun <asun@cobaltnet.com>
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com/
#
# inspired by siteUserList.cgi

package Cobalt::UI::diskList;

use Cobalt::Product;
use Cobalt::Meta::vsite;
use Cobalt::Vsite;
use Cobalt::DiskUtils;
use Cobalt::Util;
use Cobalt::Locale;
use Cobalt::User;

use Locale::gettext;
Locale::gettext::textdomain("ui");

my ($q, $page, $locale, $info, $button_style, $header_style, $table_style);
my $KBperGB = 1024*1024;
my $diskListPageURL ="/cgi-bin/.cobalt/turbo_ui/dispatch/diskList";
my $diskAddPageURL ="/cgi-bin/.cobalt/turbo_ui/dispatch/diskAdd";
my $disabled_msg = '<IMG SRC=/.cobalt/images/suspend_icon_small ALT="no">';
my $enabled_msg = '<IMG SRC=/.cobalt/images/yes ALT="yes">';


my $str_help;
my $str_volModify;
my $str_volDelete;
my $ide_msg;
my $scsi_msg;
my $raid_msg;
my $unknown_msg;
my $diskadd_msg;
my $error_voldelete; 

sub main {

$q=shift; 
$page = $q->param('page');

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";

}

if( $ENV{ PARAM_INFO } ) {
    $info=$ENV{ PARAM_INFO };
} else {
    $info=$q->param( 'info' );
}

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

$error_voldelete = MSG_get_ui('volErrDeleteBusy');
$diskadd_msg = MSG_get_ui('volDiskAdd');

$str_help = MSG_get_ui('help');
$str_volModify = MSG_get_ui('volModifyHelp');
$str_volDelete = MSG_get_ui('volDeleteHelp');
$ide_msg = MSG_get_ui('IDE');
$scsi_msg = MSG_get_ui('SCSI');
$raid_msg = MSG_get_ui('RAID');
$unknown_msg = MSG_get_ui('volErrUnknown');

if ($page eq 'diskList') {
    $info = handleDiskListPage($q);
}
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);
printDiskList( \*STDOUT, $info );
}


sub dev_to_name
{
    my $dev = shift;
    
    if ($dev =~ /^\/dev\/h/) {
	return $ide_msg;
    }
    if ($dev =~ /^\/dev\/s/) {
	return $scsi_msg;
    }
    if ($dev =~ /^\/dev\/m/) {
	return $raid_msg;
    }

    return $unknown_msg;
}

# this just handles disk deletion
sub handleDiskListPage
{
    my $q = shift;
    my $label = $q->param('label');
    my $name = "$Vol_rootdir/$label";

    return undef unless defined($label) and -e $name;
    if (-f $name) {
	unlink($name);
	return undef;
    } else {
	my $link;

	opendir(DIR, $Htgroup_dir);
	while ($_ = readdir(DIR)) {
	    next unless $_ =~ /^site/; 
	    next unless (-l "$Htgroup_dir/$_");
	    $link = readlink("$Htgroup_dir/$_");
	    next unless $link =~ /^$name\/sites/;

	    my $site = new Cobalt::Meta::vsite;
	    $site->retrieve($_) || next;
            my ($ret, $info) = $site->del();
            undef ($ret, $info); # We ignore meta's return value and test on the following pass
	}
	closedir(DIR);

	# loop through again to see if something wrong happened.
	opendir(DIR, $Htgroup_dir);
	while ($_ = readdir(DIR)) {
	    next unless $_ =~ /^site/; 
	    next unless (-l "$Htgroup_dir/$_");
	    $link = readlink("$Htgroup_dir/$_");
	    next unless $link =~ /^$name\/sites/;
	    closedir(DIR);
	    return $error_voldelete;
	}
	closedir(DIR);

	# just wipe out the disk label
	if (disk_getmounts($name)) {
	    unlink("$name/.cobaltsig");
	    return $error_voldelete if disk_umount($name);
	}
	unlink("$name/.cobaltsig");  # just in case of a screwup
	return $error_voldelete if (-d $name and not rmdir($name));
    }
    return undef;
}


# construct a form for doing stuff to a disk
sub constructForm
{
    my ($device, $label, $state) = @_;

    my $text = <<END;
<A HREF="javascript: modifyDisk(\'$device\', \'$label\', \'$state\');" onMouseOver="status='$str_help'; top.showInfo(modifyHelp); return true;"><IMG SRC="/libImage/modify" ALT="$str_volModify" BORDER="0"></A>
END

    if($state eq 'disabled') {
        $text .= '<IMG SRC="/libImage/removeGray" BORDER="0">';
    } else {
        $text .= <<END;
<A HREF="javascript: deleteDisk(\'$label\');" onMouseOver="status='$str_help'; top.showInfo(trashHelp); return true;"><IMG SRC="/libImage/remove" ALT="$str_volDelete" BORDER="0"></A>
END
    }

    return $text;
}

sub printDiskList
# description: print out a list of all the disks in use.
#
{
    my ($handle, $info) = @_;
    my (%mounts, %hash);
    my ($disklist, $key);
    my ($interface, $size, $label, $misc);
    my ($hidestart, $hideend);
    my $directories = "/.//../";

    # do /home first.
    %mounts = disk_getmounts("/$Vol_home");
    foreach $key (sort keys %mounts) {
	$label = $Vol_home;
	$interface = dev_to_name($key);
	$size = sprintf("%.3f GB", $mounts{$key}{Size}/$KBperGB); 
	$disklist .= <<END;
<TR>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$label</TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$interface</TD>
<TD NOWRAP ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$size</TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$enabled_msg</TD>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><IMG SRC="/libImage/modifyGray" BORDER="0"><FONT COLOR="#FFFFFF">&nbsp;</FONT><IMG SRC="/libImage/removeGray" BORDER="0"></TD>
END
    }
	
    # do /vol next. 
    %mounts = disk_getmounts($Vol_rootdir);
    foreach $key (keys %mounts) {
	$label = disk_getlabel($mounts{$key}{MountPoint});
	if ($label) {
	    $hash{$label}{Interface} = dev_to_name($key);
	    $hash{$label}{Size} = 
		sprintf("%.3f GB", $mounts{$key}{Size}/$KBperGB);
	    $hash{$label}{Misc} = constructForm($key, $label, "enabled"); 
	    $directories .= "/$label/";
	}
    }

    foreach $key (sort keys %hash) {
	$disklist .= <<END;
<TR>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$key</TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$hash{$key}{Interface}</TD>
<TD NOWRAP ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$hash{$key}{Size}</TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$enabled_msg</TD>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$hash{$key}{Misc}</TD>
</TR>
END
    }

    # get all of the disabled disks. usually, they're disabled because
    # the enabled option is set to no, but somebody might have come
    # around and just unmounted a disk as well. 
    # NOTE: we need to disable this bit when a disk update is in progress.
    if (&Cobalt::DiskUtils::disk_checklock()) {
	$hidestart = '';
	$hideend = '';
	$disklist .= <<END;
<TR>
<TD NOWRAP COLSPAN=5 BGCOLOR="#FFFFFF" ALIGN=CENTER><FONT COLOR=#FFFFFF>&nbsp;</FONT>$diskadd_msg</TD>
</TR>
END
    } else {
	$hidestart = '<!-- ';
	$hideend = ' -->';
	%mounts = disk_getumounts;
	%hash = ();
	foreach $key (keys %mounts) {
	    my $mountpoint;
	    
	    if ($mountpoint = disk_mount($key, "ro,nocheck")) {
		my ($id, $enabled);

		($label, $id, $enabled) = disk_getsignature($mountpoint);
		next unless $label;
		$hash{$label}{Interface} = dev_to_name($key);
		$hash{$label}{Size} = 
		    sprintf("%.3f GB", $mounts{$key}/$KBperGB);
		$hash{$label}{Misc} = 
		    constructForm($key, $label, "disabled");
		$directories .= "/$label/";
		disk_umount($mountpoint);
	    }
	}

	foreach $key (sort keys %hash) {
	    $disklist .= <<END;
<TR>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$key</FONT></TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$hash{$key}{Interface}</FONT></TD>
<TD NOWRAP ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$hash{$key}{Size}</FONT></TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$disabled_msg</FONT></TD>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#555555>$hash{$key}{Misc}</FONT></TD>
</TR>
END
        }

  
	# now get all of the directories without disks
	if (opendir(ROOTDIR, $Vol_rootdir)) {
	    while ($key = readdir(ROOTDIR)) {
		if ($directories !~ /\/$key\//) {
		    $disklist .= <<END;
<TR>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$key</FONT></TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$unknown_msg</FONT></TD>
<TD NOWRAP ALIGN=RIGHT BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$unknown_msg</FONT></TD>
<TD NOWRAP ALIGN=CENTER BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><FONT COLOR=#AAAAAA>$disabled_msg</FONT></TD>
<TD NOWRAP BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT><IMG SRC="/libImage/modifyGray" BORDER="0"><A HREF="javascript: deleteDisk(\'$key\');" onMouseOver="status='$str_help'; top.showInfo(trashHelp); return true;"><IMG SRC="/libImage/remove" ALT="$str_volDelete" BORDER="0"></A></TD>
</TR>
END
                }
	    }
	    closedir(ROOTDIR);
	}
    }

    open(TEMPLATE, "$Template_dir/diskList.tmpl") or 
	die "Can't read $Template_dir/diskList.tmpl: $!\n";

    while (<TEMPLATE>) {
	s/\[DISK_LIST\]/$disklist/;
	s/\[DISK_INFO\]/$info/;
	s/\[HIDESTART\]/$hidestart/;
	s/\[HIDEEND\]/$hideend/;
	s/\[DISKLISTPAGEURL\]/$diskListPageURL/;
	s/\[DISKADDPAGEURL\]/$diskAddPageURL/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/ge;
	print $handle $_;
    }
    close(TEMPLATE);

}

1;
