#!/usr/bin/perl
#
# Adrian Sun <asun@cobaltnet.com>
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com/
#
# inspired by siteUserList.cgi

package Cobalt::UI::diskMod;

use Cobalt::Product;
use Cobalt::DiskUtils;
use Cobalt::Util;
use Cobalt::Locale;
use Cobalt::User;

use Locale::gettext;
Locale::gettext::textdomain("ui");

my ($q, $locale, $page);
my $diskListPageURL ="/cgi-bin/.cobalt/turbo_ui/dispatch/diskList";
my $diskModPageURL ="/cgi-bin/.cobalt/turbo_ui/dispatch/diskMod";

my $error_noent;
my $error_badname;
my $error_exist;
my $error_busy;
my $error_mountpoint;


sub main {

  my $info;

  $q = shift;
  $page = $q->param('page');

  my $forbiddenURL="/.cobalt/error/forbidden.html";
  my $user = $ENV{REMOTE_USER};
  unless (user_authok($user, "admin")) {
      print "location: $forbiddenURL\n\n";
      die "Unauthorized access by $user, not an admin.\n";
  } 

  if( $ENV{ PARAM_INFO } ) {
      $info=$ENV{ PARAM_INFO };
  } else {
      $info=$q->param( 'info' );
  }

  $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
  if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

  $error_noent = MSG_get_ui("volErrNoEnt");
  $error_badname = MSG_get_ui("volErrBadName");
  $error_exist = MSG_get_ui("volErrExist");
  $error_busy = MSG_get_ui("volErrBusy");
  $error_mountpoint = MSG_get_ui("volErrMountPoint");


  if ($page eq 'diskMod' and not ($info = handleDiskModify($q))) {
      print "location: $diskListPageURL\n\n";
  } else {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    printDiskModifyPage( \*STDOUT, $q, $info);
  }
}

sub handleDiskModify
{
    my $q = shift;

    my $partition = $q->param('partition');
    my $oldenabled = $q->param('state');
    my $oldname = $q->param('label');
    my $name = $q->param('volname');
    my $enabled = $q->param('enabled') ? "enabled" : "disabled";
    
    return $error_noent unless (defined($name) and $name ne "");
    return $error_badname unless $name =~ /^[\w\d\-\.]+$/;
    return $error_exist if $name eq $Vol_home;

    if ($name ne $oldname) {
	my $newdev = "$Vol_rootdir/$name";
	my $flags = "override,nocheck";

	$flags .= ",disable" if $enabled eq "disabled";

	return $error_exist if -d $newdev;

	return $error_busy if 
	    ($oldenabled eq "enabled") and disk_umount($partition);

	# fix up the disk and unmount it if it's disabled.
	disk_mount($partition, $flags, $name);
	disk_umount($newdev) if $enabled eq "disabled";

	# cleanup
	rmdir("$Vol_rootdir/$oldname");

	# retarget all of the sites that use this disk. 
	opendir(DIR, $Htgroup_dir);
	while ($_ = readdir(DIR)) {
	    my ($sitename, $link);

	    next unless /^site/;
	    $sitename = "$Htgroup_dir/$_";
	    next unless (-l $sitename);

	    $link = readlink($sitename);
	    next unless $link =~ /^$Vol_rootdir\/$oldname/o;
	    unlink($sitename);
	    symlink("$newdev/sites/$_", $sitename);
	}
	closedir(DIR);
	return undef;
    }
    
    if ($enabled ne $oldenabled) {
	if ($enabled eq "enabled") {
	    return $error_mountpoint unless 
		(disk_mountok($Vol_diskdir) and disk_mountok($Vol_rootdir));
	    return $error_mountpoint unless disk_mount($partition, "enable");
	} else {
	    return $error_busy if disk_umount($partition, "disable");
	}
    }
    
    return undef;
}


# things that can be modified
sub printDiskModifyPage
{

    my ($handle, $q, $info) = @_;
    my $partition = $q->param('partition');
    my $label = $q->param('label');
    my $state = $q->param('state');

    my $enabled = "CHECKED" if ($state eq "enabled");

    open(TEMPLATE, "$Template_dir/diskMod.tmpl") or 
	die "Can't read $Template_dir/diskMod.tmpl: $!\n";

    while (<TEMPLATE>) {
	s/\[PARTITION\]/$partition/;
	s/\[VOLNAME\]/$label/;
	s/\[STATE\]/$state/;
	s/\[ENABLE_STATE\]/$enabled/;
	s/\[DISK_INFO\]/$info/;
	s/\[DISKMODSCRIPTURL\]/$diskModPageURL/o;
	s/\[DISKLISTSCRIPTURL\]/$diskListPageURL/o;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/ge;
	print $handle $_;
    }
    close(TEMPLATE);

}
1;
