#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Microserver
# version: 4.0
# date: 7-6-99
#
# version 1.0 supports users and groups
# version 2.0 supports sites
# Version 3.0 uses Monitor.pm instead of AlertDB.pm
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::diskUsage;

use Cobalt::Product;
use Cobalt::Group;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Quota;
use Cobalt::User;
use Cobalt::DiskUtils;
use Cobalt::Vsite;
use Cobalt::swatch;
use Cobalt::Util;

use Locale::gettext;


# global variables
my ($q, $locale, $page, $alert, $show, $site );
my ($button_style, $header_style, $table_style);
my ( @groups, @siteNames, @groupSeverityLogos, @groupUsages, @groupFrees, @groupQuotaStrs, @groupUsageBars, @groupGreeBars, @groupUsagePercents, @vsiteInfo, %groupToNum );
my $alertScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/alert';
my $forbiddenURL='/.cobalt/error/forbidden.html';
my $diskUsageScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/diskUsage';


sub main {

undef @groups;
$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); 
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

$alert=$q->param( 'alert' );

# get which page to show
$show=$q->param( 'show' );
$site=$q->param( 'site' );

my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin") or
	(($show eq 'oneSite') and (user_authok($user, "site-adm", $site)))) {
    print "location: $forbiddenURL\n\n";
    if ($show eq 'oneSite') {
	die "Unauthorized access by $user, not a member of $site.\n"; 
    } else {
        die "Unauthorized access by $user, not $AdminUser.\n";
    }
}

# check to see if everything is set up ok. if we aren't set up
# yet, just spit out a message.
my $defcon = swatch_get_defcon_disk();
unless ($defcon) {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    open( TEMPLATE, "$Template_dir/diskUsageNoReport.tmpl" )
	|| die "Cannot read $Template_dir/diskUsageNoReport.tmpl: $!\n";
    while (<TEMPLATE>) {
	s/\[SEVERITY_LOGO\]/$defcon_images{$defcon}/o;
	s/\[MESSAGE\]/MSG_get_ui("swatch_body_defcon_$defcon")/eo;
        s/\[ALERTSCRIPTURL\]/$alertScriptURL/o;
	s/\[FONTH\]/$header_style/go;
	s/\[FONTT\]/$table_style/go;
	s/\[FONTB\]/$button_style/go;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;
	print;
    }
    return 0;
}

# add virtual sites
@vsiteInfo=vsite_list();
my $i;
for( $i=0; $i<=$#vsiteInfo; $i++ ) {
    my $groupName=$vsiteInfo[ $i ][ 2 ];

    push @siteNames, $vsiteInfo[ $i ][ 1 ];
    push @groups, $groupName;
    push @groupSeverityLogos, $defcon_images{ swatch_get_defcon_quota_member($groupName) };
    push @siteSeverityLogos, $defcon_images{ swatch_get_defcon_quota_vsite($groupName) };

    $groupToNum{ $groupName }=$i;
}

my %mounts = disk_getmounts();
my ($label,%totalFrees);
foreach (keys %mounts) {
    $label = disk_getlabel($mounts{$_}{MountPoint});
    if ($label) {
	$totalFrees{$label} = $mounts{$_}{Free} * $BytesPerBlock/1048576;
    }
}

my $allSitesUsage=0;
my $group;
my $site_home;
foreach $group ( @groups ) {
    my ( $groupUsage, $groupQuota )=( group_get_quota( $group ) );

    # block to MB
    $groupQuota*=$BytesPerBlock/1048576;
    $groupQuota=sprintf("%.1f",$groupQuota);
    $groupUsage*=$BytesPerBlock/1048576;
    $groupUsage=sprintf("%.1f", $groupUsage);

    $site_home = disk_getlabel("$Htgroup_dir/$group");
			  
    # Note: disk usage can be larger than quota because it is a soft quota
    my $groupFree;
    if( $groupQuota==0 ) { $groupFree=$totalFrees{$site_home}; }
    else {
	$groupFree=( $groupQuota>$groupUsage )?$groupQuota-$groupUsage:0;
	# quota can be larger than physical size
	if( $groupFree>$totalFrees{$site_home} ) { $groupFree=$totalFrees{$site_home}; }
    }
    $groupFree=sprintf("%.1f", $groupFree);

    push @groupUsages, $groupUsage;
    push @groupQuotas, $groupQuota;
    push @groupFrees, $groupFree;

    $allSitesUsage+=$groupUsage;

    # Note: disk usage can be larger than quota because it is a soft quota
    my $groupUsagePercent;
    my $groupFreePercent;

    $groupUsagePercent=sprintf("%1.f", 100*$groupUsage/( $groupUsage+$groupFree ) );
    $groupFreePercent=100-$groupUsagePercent;

    my $groupQuotaStr;
    if( $groupQuota==0 ) { $groupQuotaStr='<I>Unlimited</I>'; }
    else { $groupQuotaStr=$groupQuota; }

    my $groupUsageBar;
    if( $groupUsagePercent>0 ) { $groupUsageBar="<IMG SRC=\"/libImage/bar1.jpg\" WIDTH=$groupUsagePercent HEIGHT=20 BORDER=0>"; }
    my $groupFreeBar;
    if( $groupFree>0 ) { $groupFreeBar="<IMG SRC=\"/libImage/bar2.jpg\" WIDTH=$groupFreePercent HEIGHT=20 BORDER=0>"; }

    push @groupUsagePercents, $groupUsagePercent;
    push @groupFreePercents, $groupFreePercent;
    push @groupQuotaStrs, $groupQuotaStr;
    push @groupUsageBars, $groupUsageBar;
    push @groupFreeBars, $groupFreeBar;
}
# block to MB
$allSitesUsage=sprintf("%.1f", $allSitesUsage);


# get which page to show
if ( $show eq 'allSites' ) {
    genAllSitesPage();
} elsif ( $show eq 'oneSite' ) {
    my $back=$q->param( 'back' );
    genSitePage( $site, $back, 
		 $totalFrees{disk_getlabel("$Htgroup_dir/$site")});
} else {
    # show the summary page
    genSummaryPage();
}

undef ($q, $locale, $page, $alert, $show, $site);
undef ( @groups, @siteNames, @groupSeverityLogos, @groupUsages, @groupFrees, @groupQuotaStrs, @groupUsageBars, @groupGreeBars, @groupUsagePercents, @vsiteInfo, %groupToNum );

}

sub genSummaryPage {
    my $back='';
    if($alert) {
	$back="<TABLE BORDER=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\"><TR>
  			<TD><A HREF=\"$alertScriptURL\" onMouseOver=\"status='[str.back]'; return true;\"><IMG SRC=\"/libImage/buttonLeftDot.gif\" BORDER=0></A></TD>
				<TD NOWRAP STYLE=\"[FONTB]\" ><A HREF=\"$alertScriptURL\" onMouseOver=\"status='[str.back]'; return true;\"><FONT STYLE=\"[FONTB]\">[str.back]</FONT></A></TD>
				<TD><A HREF=\"$alertScriptURL\" onMouseOver=\"status='[str.back]'; return true;\"><IMG SRC=\"/libImage/buttonRight.gif\" BORDER=0></A></TD>
			</TR></TABLE>";
    }

    my $fmt = "%.1f";

    my $sysObj = swatch_get("quota", "sys");
    my $sysTotal = sprintf $fmt, $sysObj->get("total");
    my $sysFree = sprintf $fmt, $sysObj->get("free");
    my $sysUsed = sprintf $fmt, $sysObj->get("used");
    my $sysPercent = sprintf $fmt, $sysObj->get("percent");
    my $sysDefcon = swatch_get_defcon("quota", "sys");

# Bug here, reporting small...    
    my $siteObj = swatch_get("quota", "all");
    print STDERR "Site quota for all is: ".$siteObj->get("total")."\n" if ($DEBUG);

    my $siteTotal = sprintf $fmt, $siteObj->get("total") - $sysTotal;
    my $siteFree = sprintf $fmt, $siteObj->get("free") - $sysFree;
    my $siteUsed = sprintf $fmt, $siteObj->get("used") - $sysUsed;
    my $sitePercent = ($siteTotal > 0.01) ? 
	sprintf $fmt, $siteUsed / $siteTotal * 100 : '?';

    # The values are given in bytes, divide by 1048576 to get megs
    my $siteLight = $defcon_images{ swatch_get_defcon_sites() };
    my $sysLight = $defcon_images{ $sysDefcon };

    my $mb = MSG_get_ui("megabyte");
    my $dhtml;
    my $homedisk = swatch_get("quota", "/$Vol_home");
    my $name = $Vol_home;
    my $light = $defcon_images{ swatch_get_defcon("quota", "/$Vol_home") };
    my $total = sprintf $fmt, $homedisk->get("total");
    my $free = sprintf $fmt, $homedisk->get("free");
    my $percent = sprintf $fmt, $homedisk->get("percent");
    
    my $dhtml = <<END;
<TR>
    <TD WIDTH="26">
	<CENTER><A HREF="javascript: top.showInfo( volUsageHelp );" onMouseOver="status='[str.help]'; top.showInfo( volUsageHelp ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A></CENTER>
    </TD>
    <TD WIDTH="26">
	<CENTER>$light</CENTER>
    </TD>
    <TD ALIGN="LEFT" WIDTH="260">
    <FONT COLOR="#FFFFFF">&nbsp;</FONT><B>$name</B>
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
		$total $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
		$free $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="73">
		$percent
    </TD>
</TR>
END
    ;

    my @disks = Cobalt::Meta::query("type"  => "quota",
				    "where" => ["name", "~", "$Vol_rootdir"]);

    foreach (@disks) {
	$name = $_->getname;
	$light = $defcon_images{ swatch_get_defcon("quota", $name) };
	$name =~ s|/vol/(.*)|$1|;
	$total = sprintf $fmt, $_->get("total");
	$free = sprintf $fmt, $_->get("free");
	$percent = sprintf $fmt, $_->get("percent");

	$dhtml .= <<END;
<TR>
    <TD WIDTH="26">
	<CENTER><A HREF="javascript: top.showInfo( volUsageHelp );" onMouseOver="status='[str.help]'; top.showInfo( volUsageHelp ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A></CENTER>
    </TD>
    <TD WIDTH="26">
	<CENTER>$light</CENTER>
    </TD>
    <TD ALIGN="LEFT" WIDTH="260">
	<FONT COLOR="#FFFFFF">&nbsp;</FONT>
	<B>$name</B>
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
	$total $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
	$free $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="73">
	$percent
    </TD>
</TR>
END
    ;
    }


    # Read in the template
    open( TEMPLATE, "$Template_dir/diskUsageSum.tmpl" )
	|| die "Cannot read $Template_dir/diskUsageSum.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    while ( <TEMPLATE> ) {
	s/\[SYSLIGHT\]/$sysLight/;
	s/\[SYSTOTAL\]/$sysTotal/;
	s/\[SYSFREE\]/$sysFree/;
	s/\[SYSPERCENT\]/$sysPercent/;

	s/\[SITELIGHT\]/$siteLight/;
	s/\[SITETOTAL\]/$siteTotal/;
	s/\[SITEFREE\]/$siteFree/;
	s/\[SITEPERCENT\]/$sitePercent/;

	s/\[SITEUSAGEURL\]/$diskUsageScriptURL?show=allSites&alert=$alert/;
	s/\[BACK\]/$back/;

	s/\[DISKINFO\]/$dhtml/;
  s/\[FONTH\]/$header_style/g;
  s/\[FONTT\]/$table_style/g;
  s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print;
    }

    close( TEMPLATE );
}


sub genAllSitesPage {
    my $usageRow;
    my $i;
    for( $i=0; $i<=$#groups; $i++ ) {
	$usageRow.=<<END;
<TR ALIGN="CENTER">
  <TD><P ALIGN="CENTER">$groupSeverityLogos[ $i ]</TD>
  <TD><P ALIGN="CENTER"><A HREF="$diskUsageScriptURL?show=oneSite&site=$groups[ $i ]&back=t&alert=$alert">$siteNames[ $i ]</A></TD>
  <TD><P ALIGN="CENTER">$groupUsages[ $i ]</TD>
  <TD><P ALIGN="CENTER">$groupFrees[ $i ]</TD>
  <TD><P ALIGN="CENTER">$groupQuotaStrs[ $i ]</TD>
  <TD NOWRAP><P ALIGN="LEFT"><IMG SRC="/libImage/bar0.jpg" BORDER="0" HEIGHT="20">$groupUsageBars[ $i ]$groupFreeBars[ $i ]<IMG SRC="/libImage/bar3.jpg" BORDER="0" HEIGHT="20"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$groupUsagePercents[ $i ]%</TD>
</TR>
END
    }

    # Read in the template
    open( TEMPLATE, "$Template_dir/diskUsageSites.tmpl" )
        || die "Cannot read $Template_dir/diskUsageSites.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
	s/\[USAGE_ROW\]/$usageRow/;
        s/\[SUMUSAGEURL\]/$diskUsageScriptURL?alert=$alert/;
  s/\[FONTH\]/$header_style/g;
  s/\[FONTT\]/$table_style/g;
  s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

        print;
    }

    close( TEMPLATE );
}


# argument 0 is the group name of the site
# argument 1 is true if a back button is needed
sub genSitePage {
    my ( $group, $back, $siteFree )=@_;

    my $groupNo=$groupToNum{ $group };
    my $groupFree = $groupFrees[ $groupNo ];

    my $siteName=$siteNames[ $groupNo ];

    my @users=vsite_get_users($group);
    if ($users[0] =~ /^[134]\S*\s/) {
	# substr( $users[0], 5 ).'<BR>';
	@users = ();
    }

    my $usageRow;
    my $user;
    foreach $user ( @users ) {
	my $usage;
	my $free;

	my $userObj = Cobalt::Meta->new("type" => "users");
	$userObj->retrieve($user);
	my $fullName = $userObj->get("fullname");
	my $userQuota = sprintf("%1.f", $userObj->get("quota"));
	my $userUsage = (user_show($user))[3];

	# block to MB
	$userUsage*=$BytesPerBlock/1024/1024;
	$userUsage=sprintf("%.1f", $userUsage);

	# Note: disk usage can be larger than quota because it is a soft quota
	my $userFree;
	if( $userQuota==0 ) {      # 0 quota means unlimited
	    $userFree = $groupFree;
	    $usage=sprintf("%1.f", 100*$userUsage/($userUsage + $userFree));
	}
	else {
	    $userFree=( $userQuota>$userUsage )?$userQuota-$userUsage:0;
	    # quota can be larger than physical size
	    if( $userFree>$siteFree ) { $userFree=$siteFree; }

	    $usage=sprintf("%1.f", 100*$userUsage/( $userUsage+$userFree ) );
	}
	$userFree=sprintf("%.1f", $userFree);

	$free=100-$usage;

	my $severityLogo=$defcon_images{ swatch_get_defcon_quota_user($user) };

	my $quota;
	if( $userQuota==0 ) { $quota='<I>Unlimited</I>'; }
	else { $quota=$userQuota; }

	my $usageBar;
	if( $usage>0 ) { $usageBar="<IMG SRC=\"/libImage/bar1.jpg\" WIDTH=$usage HEIGHT=20 BORDER=0>"; }
	my $freeBar;
	if( $free>0 ) {  $freeBar="<IMG SRC=\"/libImage/bar2.jpg\" WIDTH=$free HEIGHT=20 BORDER=0>"; }

	$usageRow.=<<END;
<TR ALIGN="CENTER">
  <TD><P ALIGN="CENTER">$severityLogo</TD>
  <TD><P ALIGN="CENTER">$user</TD>
  <TD><P ALIGN="CENTER">$fullName</TD>
  <TD><P ALIGN="CENTER">$userUsage</TD>
  <TD><P ALIGN="CENTER">$userFree</TD>
  <TD><P ALIGN="CENTER">$quota</TD>
  <TD NOWRAP><P ALIGN="LEFT"><IMG SRC="/libImage/bar0.jpg" BORDER="0" HEIGHT="20">$usageBar$freeBar<IMG SRC="/libImage/bar3.jpg" BORDER="0" HEIGHT="20"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$usage%</TD></TR>
END
    }

    if( $#users<0 ) {
	$usageRow="<TR><TD COLSPAN=7><FONT STYLE=\"[FONTT]\">&nbsp;<I>".eval("\"".Locale::gettext::dgettext("ui", "noUsers")."\"")."</I></FONT></TD></TR>";
    }

    my $hideBackStart='<!-- ';
    my $hideBackEnd=' -->';
    if( $back ) {
	$hideBackStart='';
	$hideBackEnd='';
    }

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    # Read in the template
    open( TEMPLATE, "$Template_dir/diskUsageUsers.tmpl" )
	|| die "Cannot read $Template_dir/diskUsageUsers.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[HIDEBACKEND\]/$hideBackEnd/;
	s/\[HIDEBACKSTART\]/$hideBackStart/;
	s/\[SITEFREE\]/$groupFrees[ $groupNo ]/;
	s/\[SITEFREEBAR\]/$groupFreeBars[ $groupNo ]/;
	s/\[SITEQUOTA\]/$groupQuotaStrs[ $groupNo ]/;
	s/\[SITENAME\]/$siteName/;
	s/\[SITEUSAGEURL\]/$diskUsageScriptURL?show=allSites&alert=$alert/;
	s/\[SITESEVERITYLOGO\]/$siteSeverityLogos[ $groupNo ]/;
	s/\[SITEUSAGE\]/$groupUsages[ $groupNo ]/;
	s/\[SITEUSAGEBAR\]/$groupUsageBars[ $groupNo ]/;
	s/\[SITEUSAGEPERCENT\]/$groupUsagePercents[ $groupNo ]/;
	s/\[USAGE_ROW\]/$usageRow/;
	s/\[FONTH\]/$header_style/g;
	s/\[FONTT\]/$table_style/g;
	s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print;
    }

    close( TEMPLATE );
}

1;
