#!/usr/bin/perl
#
# Allows users to download raw log files.
#
# Patrick Bose <pbose@cobalt.com> 
#   adapted from webLog.cgi 
#
# Copyright 2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::downloadLog;

use Cobalt::Product;
use Cobalt::Group;
use Cobalt::Vsite;
use Cobalt::User;

# global variables
my ($q, $group, $type, $logFile);
my $forbiddenURL="/.cobalt/error/forbidden.html";
my $Debug = 1;

sub main {

	$q=shift;
	$group=$q->param( 'group' );
	$type=$q->param( 'type' );

	if ( $group && $group ne "server" ) {
		$logFile="$Htgroup_dir/$group/$Logdir/$type.log";
	} else {
		# For the whole RaQ.
		if ( $type eq 'mail' ) {
			$logFile=$Mail_log;
		} elsif ( $type eq 'ftp' ) {
			$logFile=$FTP_log;
		} elsif ( $type eq 'web' ) {
			$logFile=$Access_log;
		} elsif ( $type eq 'net' ) {
			$logFile=$Network_log;
		}
	}

	# Add some remote hint of security here 
	my $user = $ENV{REMOTE_USER};
	unless (user_authok($user, "site-adm", $group)) {
		print "location: $forbiddenURL\n\n";    
		die "Unauthorised access by $user, not a member of $group or not a site admin.\n";
	}

	# find site hostname
	my($host)=(vsite_get_bygroup($group))[1];
	$host ||= 'raqwide';
	print STDERR "Host: $host Group: $group\n" if $Debug;

	# IE 4 tries to display standard binary mime types.
	# print "Content-type: application/octet-stream\n";
	print "Content-type: application/UnknownWebLog\n";
	print "Content-disposition: filename=$host.$type.log\n\n";

	# First read the fragmented log file, then parse the to-the-minute entries 
	# from the consolidated access log if this is a web log. Otherwise don't
	# bother. Note that we miss the compressed logs, if people care enough they
	# can get them via ftp.

	my($foundLog)=0;

	# Just for web we grab up to date entires from the access file.
	# Skip this bit for tha main group as we grab all of this anyways.
	if($type eq 'web' && $group && $group ne 'server' ) {
		open(AXL,"$Access_log");
		while(<AXL>) {
			if ($_ =~ /^(\S+) (.*)/) {
				if ($1 eq $host) {
					print $2."\n";
					$foundLog=1;
				}
			}
		}
		close(AXL);
	}

	my $i;
	my $current_logFile = $logFile;

	while (-r $current_logFile) {
		print STDERR "Now grabbing $current_logFile\n" if $Debug;
		open(LGZ,$current_logFile);
		while (<LGZ>) {
			print;
			# Moved here to make sure that we don't think we've got
			# data when we just have an empty file.
			$foundLog=1;
		}

	close(LGZ);
	$i++;
	# Now we grab all the .<number> files that logrotate cataches as
	# well.
	$current_logFile = $logFile . ".$i";
	}

	# Null files are bad
	print "0\n" unless ($foundLog);

}

1;

