#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 1.0
# date: 12-24-97
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::email;

use Cobalt::Product;
use Cobalt::Admserv;
use Cobalt::Group;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::User;
use Cobalt::Vacation;
use Cobalt::Meta;

use Locale::gettext;


my ($q,$locale,$page, $button_style, $header_style, $table_style);
my $emailScriptDir="$AdminCGIs_dir/email/";
my $emailScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/email';

sub main {

# global variables
$q=shift;

$page=$q->param( 'page' );

my $userListPageURL='/.cobalt/userList/';

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); 
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

if( $page eq 'email' ) {
    if( needLogin( $emailScriptDir ) ) {
	# ask user to enter user name
	my $userName=$q->param( 'username' );
	my $forwardAddr=$q->param( 'forwardaddr' );
	my $vacationMode=$q->param( 'vacationmode' );
	my $vacationMsg=$q->param( 'vacationmsg' );

	print "location: $emailScriptURL?page=$page&username=$userName&forwardaddr=$forwardAddr&vacationmode=$vacationMode&vacationmsg=$vacationMsg\n\n";
    }
    else { handleEmailPage(); }
}
else {
    my $userName=$q->param( 'username' );

    # force login
    if( needLogin( $emailScriptDir ) ) {
	# ask user to enter user name
	print "location: $emailScriptURL?page=$page&username=$userName\n\n";
    }
    else {
	my $isSysManage=( $q->param( 'page' ) eq 'userlist' );
	my $loginName=$ENV{ REMOTE_USER };
	if( !$loginName ) {
	    genEmailPage( $userName, $isSysManage );
	}
	elsif( $loginName eq $Adminuser ) {
	    if( $userName ) {
		genEmailPage( $userName, $isSysManage );
	    }
	    else {
		genEmailPage( $Adminuser, $isSysManage );
	    }
	}
	else { genEmailPage( $loginName, $isSysManage ); }
    }
}

}

#
# event handler for email page
#
sub handleEmailPage {
    # read form input
    my $userName=$q->param( 'username' );
    my $forwardAddr=$q->param( 'forwardaddr' );
    my $vacationMode=$q->param( 'vacationmode' );
    my $vacationMsg=$q->param( 'vacationmsg' );

    # enforce access security
#    my $loginName=$ENV{ REMOTE_USER };
#    if( ( $loginName ne $Adminuser ) && ( $loginName ne $userName ) ) { return; }

    my $ret;
    my $err=0;
    my $info='';
    my @allforwards=();
    my $forward='';
    @allforwards = split(/,/, $forwardAddr);
    map(s/^\s*//g, @allforwards); #trim left whitespace
    map(s/\s*$//g, @allforwards); #trim right whitespace

    # error check
    if( $forwardAddr ) {
	foreach $forward (@allforwards) {
	    if (!netutil_test_emailaddr( $forward ) ) {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "forwardAddrNotValid")."\"");
		$err=1;
		last;
	    }
	}
    }

    # everything is fine
    if (!$err) {
	my $meta = new Cobalt::Meta("type" => "users");
	$meta->retrieve($userName);

	# set forward to
	if( $forwardAddr ) {
	    # build list
            # $forwardAddr could actually be a comma separated list of aliases
	    $ret=alias_set( $userName, @allforwards );
	    $meta->put("forward", $forwardAddr);
	}
	else {
	    # delete list
	    $ret='2';
	    if( alias_exist( $userName ) ) { $ret=alias_delete( $userName ); }
	    $meta->put("forward", "");
	}
        if( $ret!~/^2/o ) {
            $info.=substr( $ret, 4 ).'<BR>';
            $err=1;
        }

	my $domainName;
#-IF vmachine
	$domainName=( user_list_site( $userName ) )[ 0 ];
#-ENDIF
	# set vacation mode
	$ret=vacation_set_on( $userName, $vacationMode, $domainName );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
	$meta->put("vacation" => (($vacationMode) ? "on" : "off"));

	# set vacation message
	$ret=vacation_set_message( $userName, $vacationMsg );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	$meta->save;
    }
    
    if ($info) {
	$info = "'$info', 'error'";
    } else {
	$info = "'', 'status'";
    }

    if( $err ) { genEmailPage( $userName, ( $q->param( 'sysmanage' ) eq 't' ), $info ); }
    else {
	if( $q->param( 'sysmanage' ) ) { print "location: $userListPageURL\n\n"; }
	else { print "location: $emailScriptURL\n\n"; }
    }
}

#
# argument 0 is the user name
# argument 1 is true if the page is for system management, personal profile otherwise
# argument 2 is the infomation shown to user
#
sub genEmailPage {
    my ( $userName, $isSysManage, $info )=@_;

    my ($forwardAddr, @forwardList);
    @forwardList = Cobalt::List::alias_get_vacationless( $userName );
    if ($forwardList[0] =~ /^[134]\S*\s/) {
	# substr( $forwardList[0], 5 ).'<BR>';
	@forwardList = ();
    }
    $forwardAddr=join ',', @forwardList;

    my $vacationMode=( vacation_get_on( $userName ) )?'CHECKED':'';
    my $vacationMsg=vacation_get_message( $userName );

    my $cancel='';
    if( $isSysManage ) {
	$cancel="<A HREF=\"$userListPageURL\" onMouseOver=\"status=".eval("\"".Locale::gettext::dgettext("ui", "backToUserList")."\"")."; return true;\"><IMG SRC=\"/.cobalt/images/back_to_user_but\" BORDER=0></A><INPUT TYPE=HIDDEN NAME=\"sysmanage\" VALUE=\"t\">";
    }

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    # Read in the template
    open( TEMPLATE, "$Template_dir/email.tmpl" )
        || die "Cannot read $Template_dir/email.tmpl: $!\n";

    if (not $info) {
	$info = "'', 'status'";
    }

    while ( <TEMPLATE> ) {
	 s/\[CANCEL\]/$cancel/;
         s/\[EMAILSCRIPTURL\]/$emailScriptURL/o;
         s/\[FORWARDADDR\]/$forwardAddr/;
         s/\[USER_INFO\]/$info/;
         s/\[USERNAME\]/$userName/;
         s/\[VACATIONMODE\]/$vacationMode/;
         s/\[VACATIONMESSAGE\]/$vacationMsg/;
         s/\[FONTH\]/$header_style/g;
         s/\[FONTT\]/$table_style/g;
         s/\[FONTB\]/$button_style/g;

	 s/\[str.([^\]]+)\]/eval("\"".Locale::gettext::dgettext("ui", "$1")."\"")/ge;

         print;
    }

    close TEMPLATE;

}


#
# argument 0 is the directory of script in question
# return true ( 1 ) or false ( 0 )
#
sub needLogin {
    if( admserv_get_access( $_[ 0 ] ) ne 'none' ) {
	if( !$ENV{ REMOTE_USER } ) { return 1; }
	else { return 0; }
    }
    else { return 0; }
}
1;
