#! /usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 2.0
# date: 7-01-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::info;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::DiskUtils;
use Cobalt::User;

use Locale::gettext;

my ($locale, $button_style, $header_style, $table_style);

sub main {
# global variable
$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); 
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "site-adm")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

my $speed;
open(CPU,"/proc/cpuinfo")
    || die "Cannot read /proc/cpuinfo: $!\n";
while (<CPU>)
{
    if (/^cpu MHz\s+:\s([\d\.]+)/o)
    {
        $speed = $1;
        last;
    }
}
close CPU;

my $memory;
open(MEM,"/proc/meminfo")
    || die "Cannot read /proc/meminfo: $!\n";
while (<MEM>)
{
    if (/^MemTotal:\s+(\d+)/o)
    {
	# the total is always less the real amount because kernel use up some memory
	# this algorithm is only good if the kernel don't take up >16MB and
	# memory is blocks of 16MB
	$memory = int (($1+16384)/16384)*16;
#        $memory = int (($1+1024)/1000 ); 
    }
}
close MEM;


my $diskspace;
my %mounts = disk_getmounts;
my $part;
foreach $part (keys %mounts) {
    $diskspace += $mounts{$part}{Size};
}

$diskspace = int($diskspace / 100000) /10;


my $rel;
open(REL,"/etc/cobalt-release")
  || die "Cannot read /etc/cobalt-release: $!\n";
while (<REL>)
{
   if (/release ([\d\.]+)+/o)
   {
       $rel = $1;
   }
}
close REL;

my $registerLink = <<END;
	<TR>
		<TD COLSPAN="2">
			<UL><FONT STYLE="[FONTT]"><IMG SRC="/.cobalt/images/bluedot1" ALIGN="BOTTOM" BORDER="0"><A HREF="/cgi-bin/.cobalt/turbo_ui/dispatch/register">[str.productRegistration]</A></FONT></UL>
		</TD>
	</TR>
END

# Read in the template
open( TEMPLATE, "$Template_dir/info.tmpl" )
    || die "Cannot read $Template_dir/info.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


while ( <TEMPLATE> ) {
    s/\[RAM\]/$memory/;
    s/\[SPEED\]/$speed/;
    s/\[HD\]/$diskspace/;
    s/\[REL\]/$rel/;
    s/\[PRODUCTREGISTRATION\]/$registerLink/;
    s/\[FONTH\]/$header_style/g;
    s/\[FONTT\]/$table_style/g;
    s/\[FONTB\]/$button_style/g;

    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

    print;
}

close TEMPLATE;

# print "Speed $speed RAM $memory disk $diskspace GB Release $rel\n";
}
1;
