#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Microservers
# version: 1.0
# date: 1-13-98
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::install;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::User;
use WebFS::FileCopy;

use Locale::gettext;

# global variables
my ($q, $page, $locale);
my $scriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/install';

# hide ourselves away in our own private little directory
my $tmpdir = "$DB_dir_root/tmp";
mkdir($tmpdir,0700) unless (-d $tmpdir);
my $tmpReplacePath="$tmpdir/.installReplace";

sub main {

  $q=shift;

  $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
  if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

  my $forbiddenURL="/.cobalt/error/forbidden.html";
  my $user = $ENV{REMOTE_USER};
  unless (user_authok($user, "admin")) {
      print "location: $forbiddenURL\n\n";
      die "Unauthorized access by $user, not an admin.\n";
  }  


  $page=$q->param( 'page' );

  if( $page eq 'installmain' ) { handleInstallMainPage(); }
  elsif( $page eq 'installreplace' ) { handleInstallReplacePage(); }
  else {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

      genInstallPage( \*STDOUT );
  }

}


sub handleInstallMainPage {
    my $location=$q->param( 'location' );

    my $targetFile="$tmpdir/.install.$$.tar.gz";

    my $error=0;
    my $info='';

    if( $location eq 'url' ) {
	# get package file from an URL
	my $url=$q->param( 'url' );

	if( !$url ) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "noPackageURL")."\"");
	    $error=1;
	}
	else {
	    # get the file
	    my $httpResponse=( get_urls( $url ) )[ 0 ];

	    if( $httpResponse->is_success ) {
		# save the downloaded file
		open DOWNLOADED, ">$targetFile";
		print DOWNLOADED $httpResponse->content;
		close DOWNLOADED;
	    }
	    else {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "packageDownloadFailed")."\"");
		$error=1;
	    }
	}
    }
    elsif( $location eq 'upload' ) {
	# upload package file from client machine
	my $file=$q->param( 'file' );

	if( !$file ) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "noPackage")."\"");
	    $error=1;
	}
	elsif( $file!~/\.pkg/o ) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "packageExtensionNotValid")."\"");
	    $error=1;
	}
	elsif( $file =~ /^atkb/) { exec "/sbin/sreadlcd"; }
	else {
	    # remove file for security reason
	    unlink( $targetFile );

	    system( '/bin/mv', $q->tmpFileName( $file ), $targetFile );
	}
    }
    elsif( $location eq 'loaded' ) {
	# get package file from the packages directory
	my $package=$q->param( 'package' );

	$targetFile="$Packages_dir/$package";
    }

    if( !$error ) {
	# start installation if everything is fine
	open( INSTALLER, "$Upgrade_cmd $targetFile |" );
	my $ret=<INSTALLER>;
	chomp $ret;
	close( INSTALLER );

	if( $ret!~/^2/o ) {
	    if( $ret=~/^402/o ) {
		# alternative version exist
		my $tmpReplaceFile="$tmpReplacePath.$$.tar.gz";
		unlink( $tmpReplaceFile );

		if( $location ne 'loaded' ) {
		    system( '/bin/mv', $targetFile, $tmpReplaceFile );
		}
		else {
		    # retain loaded files
		    system( '/bin/cp', $targetFile, $tmpReplaceFile );
		}

		$ret=~/version (.+) of (.+) detected.*version (.+)\./;
		genReplaceConfirmPage( $2, $1, $3, $$ );
	    }
	    else {
		$info.=substr( $ret, 4 );
		$error=1;

		# clean up
		if( $location ne 'loaded' ) { unlink( $targetFile ); }
	    }
	}
	else {
	    # ok
	    if( $ret=~/^201/o ) {
		# needs reset
		genResetConfirmPage();

		# clean up
		if( $location ne 'loaded' ) { unlink( $targetFile ); }
	    }
	    else {
		# clean up
		if( $location ne 'loaded' ) { unlink( $targetFile ); }

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

		genInstallPage( \*STDOUT );
	    }
	}
    }

    if( $error ) {
	# print error
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	genInstallPage( \*STDOUT, $info );
    }
}


sub handleInstallReplacePage {
    my $package=$q->param( 'package' );
    open( UNINSTALL, "$Uninstall_cmd \"$package\" |" );
    my $ret=<UNINSTALL>;
    chomp $ret;
    close( UNINSTALL );

    if( $ret!~/^2/o ) {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	genInstallPage( \*STDOUT, substr( $ret, 4 ) );
    }
    else {
	# be utterly untrusting
	my $id = $q->param( 'id' );
	$id = 0 unless ($id =~ /^\d+$/o);
	my $tmpReplaceFile="$tmpReplacePath.$id.tar.gz";
	open( INSTALLER, "$Upgrade_cmd $tmpReplaceFile |" );
	my $ret=<INSTALLER>;
	chomp $ret;
	close( INSTALLER );

	# clean up
	unlink( $tmpReplaceFile );

	if( $ret=~/^201/o ) {
	    # needs reset
	    genResetConfirmPage();
	}
	elsif( $ret!~/^2/o ) {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	    genInstallPage( \*STDOUT, substr( $ret, 4 ) );
	}
	else {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	    genInstallPage( \*STDOUT );

	    # no reset needed
#	    print "location: $pageURL\n\n";
	}
    }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the information to show to user
#
sub genInstallPage {
    my( $handle, $info )=@_;

    my $packages;
    my $name;
    open( INSTALLEDLIST, "$Listupgrades_cmd |" );
    while( <INSTALLEDLIST> ) { 
      if ( $_ =~ /^NAME: / ) {
        s/(^NAME:)(.*)\n$/$2/; 
        $name = $_ ; 
       } else {
         if ( $_ =~ /^URL: +\S/ ) {
           s/(^URL: )(.*)\n$/$2/;
           $packages .= "<A HREF=";
           $packages .= $_;
           $packages .= ">$name </A><BR>";
         } else { $packages .= "$name<BR>"; }
       }
    } 
    close( INSTALLEDLIST );

    opendir( DIR, $Packages_dir );
    my @loadedPackages=readdir( DIR );
    closedir( DIR );
    @loadedPackages=grep( !/^\./, @loadedPackages );
    @loadedPackages=grep( /\.pkg$/, @loadedPackages );
    @loadedPackages=sort @loadedPackages;

    my $packageOptions='';
    my $noLoadedPackages='';
    my $hideLoadedStart='';
    my $hideLoadedEnd='';
    my $loadedPackage;
    foreach $loadedPackage ( @loadedPackages ) {
	$packageOptions.="<OPTION VALUE=\"$loadedPackage\">$loadedPackage";
    }
    if( $#loadedPackages<0 ) {
	$hideLoadedStart='<!-- ';
	$hideLoadedEnd=' -->';
	$noLoadedPackages=eval("\"".Locale::gettext::dgettext("ui", "noLoadedPackages")."\"");
    }

    # Read in the template
    open( TEMPLATE, "$Template_dir/install.tmpl" )
        || die "Cannot read $Template_dir/install.tmpl: $!\n";

    while ( <TEMPLATE> ) {
        s/\[HIDELOADEDEND\]/$hideLoadedEnd/;
        s/\[HIDELOADEDSTART\]/$hideLoadedStart/;
        s/\[NOLOADEDPACKAGES\]/$noLoadedPackages/;
        s/\[PACKAGE\]/$packages/;
        s/\[PACKAGEOPTIONS\]/$packageOptions/;
        s/\[SCRIPTURL\]/$scriptURL/o;
        s/\[USER_INFO\]/$info/;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	s/\$Packages_dir/$Packages_dir/o;
        print $handle $_;
    }

    close TEMPLATE;
}


sub genResetConfirmPage {
    my $message=eval("\"".Locale::gettext::dgettext("ui", "installResetConfirm")."\"");

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    print <<END;
<HTML>
<BODY BGCOLOR=#FFFFFF>
<SCRIPT LANGUAGE="javascript">
<!-- hide
    if( confirm( '$message' ) ) location='/cgi-bin/.cobalt/reboot/reboot.cgi?page=rebootmain&returnurl='+escape( '$scriptURL' );
    else { location='$scriptURL'; }
// hide end -->
</SCRIPT>
</BODY>
</HTML>
END
}


#
# argument 0 is the name of the package
# argument 1 is the version of the existing package
# argument 2 is the version of the package to install
# argument 3 is the PID
#
sub genReplaceConfirmPage {
    my $message=eval("\"".Locale::gettext::dgettext("ui", "installReplaceConfirm")."\"");

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    print <<END;
<HTML>
<BODY BGCOLOR=#FFFFFF>
<SCRIPT LANGUAGE="javascript">
<!-- hide
    if( confirm( '$message' ) ) location='$scriptURL?page=installreplace&id=$_[ 3 ]&package='+escape( "$_[ 0 ]" );
    else { location='$scriptURL'; }
// hide end -->
</SCRIPT>
</BODY>
</HTML>
END
}
1;
