#!/usr/bin/perl

# author: Kevin K.M. Chiu
# version: 2.0
# date: 07-01-99
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::mailListAdd;

use Cobalt::Product;
use Cobalt::List;
use Cobalt::Vsite;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::User;
use Cobalt::Util;
use Cobalt::Html;

# use strict;
use Locale::gettext;

my ($q,$locale,$group,$user,$page);
my $mailListAddScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/mailListAdd";
my $mailListListScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/mailListList";
my $forbiddenURL='/.cobalt/error/forbidden.html';


sub main {

$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

# the page that run this script
$page=$q->param( 'page' );

$group=$q->param( 'group' );
$user = $ENV{REMOTE_USER};
unless (user_authok($user, "site-adm", $group)) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorised access by $user, not a member of $group.\n";
} 

if( $page eq 'maillistadd' ) { handleMailListAddPage(); }
else {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    genMailListAddPage( \*STDOUT, $group );
}

}


#
# event handler for add mailing list page
#
sub handleMailListAddPage {
    my $err=0;
    my $info;

    # read form input
    # my $group=$q->param( 'group' );
    my $listName=$q->param( 'listname' );
    my $recipientsArea=$q->param( 'recipients' );
    my @userRecipients=$q->param( '_userrecipients_value' );
    my $usePassword=$q->param( 'passwd' );
    my $spam=$q->param( 'spam' );
    my $open=$q->param( 'open' );
		my $password=$q->param( 'mailpass0' );

		$password = $usePassword ? $password : ''; 

    # drop the first elemet (blank)
    if ($userRecipients[0] eq '') {
	shift @userRecipients;
    }

    # get individual recipients
    my @recipients=split /[\s,]/m, $recipientsArea;

    # error check
    my $recipientErr=0;
    my $allRecipientsOK=1;
    my @resultRecipients;
    my $recipient;
    my @data;

    foreach $recipient ( @recipients ) {
	if( !$recipient ) { next; }

	if( !netutil_test_emailaddr( $recipient ) ) {
	    if( $allRecipientsOK ) { $info.=eval("\"".Locale::gettext::dgettext("ui", "recipientsNotValid")."\""); $allRecipientsOK=0; }
	    $info.=" $recipient";

	    $recipientErr=1;
	}
	else { push @resultRecipients, $recipient; }
    }
    if( !$allRecipientsOK ) { $info.='<BR>'; }
    @recipients=@resultRecipients;

    # check if list name is ok
    my @usersInGroup = vsite_get_users($group);
    foreach ( @usersInGroup ) {
	if ($_ eq $listName) { 
	    $err = 1;
	}
    }
    if ($err) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
    }
    elsif( $listName=~/[^-_0-9a-z]/) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameNotValid")."\"");
	$err=1;
    }
    elsif( $System_users{ $listName } ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
	$err=1;
    }
    elsif( $MailSystemUsers{ $listName } ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
	$err=1;
    }
    else {
	# everything is fine

	# add mailing list
	my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
	my $ret=Cobalt::List::list_create($group, $fqdn, $listName,
			    map("$_\@${fqdn}", @userRecipients), @recipients);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}
	
	# set mailing list password
	$ret = list_config_password($group, $fqdn, $listName, $password);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the open subscription option
	if ($open) {
	    $ret=list_config_open($group, $fqdn, $listName);
	} else {
	    $ret=list_config_closed($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the restricted posting option
	if ($spam) {
	    $ret=list_config_norestrict($group, $fqdn, $listName);
	} else {
	    $ret=list_config_restrict($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}


	# add X-loop headers to messages going to list
	@data = ("X-Loop: Sent from $listName\@$fqdn");
	$ret = Cobalt::List::list_config_update($group, $fqdn, $listName,
				  "message_headers", \@data);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# and don't allow them to be reposted
	@data = ("/^X-Loop: Sent from $listName\\\@$fqdn/i");
	$ret = Cobalt::List::list_config_update($group, $fqdn, $listName,
				  "taboo_headers", \@data);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}
    }

    if( $err || $recipientErr ) {
        print "location: /cgi-bin/.cobalt/turbo_ui/dispatch/mailListList?info=$info&group=$group \n\n";
    }
    else {
        print "location: /cgi-bin/.cobalt/turbo_ui/dispatch/mailListList?group=$group \n\n";
    }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the information shown to user
#
sub genMailListAddPage {
    my ( $handle, $group, $info )=@_;
    my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];

    # display users
    my $userList;

		my $selectallVal = <<END;
<CENTER>
<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0"><TR>
  <TD><A HREF="javascript: setAllUsers('value');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonLeftDot.gif" BORDER=0></A></TD>
  <TD NOWRAP STYLE="[FONTB]" ><A HREF="javascript: setAllUsers('value');" onMouseOver="status='[str.selectAllUsers]'; return true;"><FONT STYLE="[FONTB]">[str.selectAllUsers]</FONT></A></TD>
  <TD><A HREF="javascript: setAllUsers('value');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonRight.gif" BORDER=0></A></TD>
</TR></TABLE></CENTER>

END

		my $selectallEntry = <<END;
<CENTER>
<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0"><TR>
  <TD><A HREF="javascript: setAllUsers('entries');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonLeftDot.gif" BORDER=0></A></TD>
  <TD NOWRAP STYLE="[FONTB]" ><A HREF="javascript: setAllUsers('entries');" onMouseOver="status='[str.selectAllUsers]'; return true;"><FONT STYLE="[FONTB]">[str.selectAllUsers]</FONT></A></TD>
  <TD><A HREF="javascript: setAllUsers('entries');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonRight.gif" BORDER=0></A></TD>
</TR></TABLE></CENTER>

END

    # do not show admin
    my @users=vsite_get_users($group);
		if ($users[0] =~ /^[134]\S*\s/) {
			@users = ();
		}
    my($i,$list);
    if (scalar(@users)) {
			@users = sort @users;	
			my @entries;
			$userList = genSetSelector("userrecipients", \@entries, \@users, '[str.usersSubscribed]', '[str.usersNotSubscribed]', $selectallVal, $selectallEntry);
    } else {
	my $noUsers=eval("\"".Locale::gettext::dgettext("ui", "noUsers")."\"");
	$userList = qq[ <FONT COLOR="#FFFFFF">&nbsp;</FONT><I>$noUsers</I>\n ];
    }

    my $cancelURL = $mailListListScriptURL . "?group=$group";
    my $mailListOpenHelp=MSG_get_ui("mailListOpenHelp", 'majordomo@'.$fqdn);

    # Read in the template
    open (TEMPLATE, "$Template_dir/mailListAdd.tmpl")
	|| die "Cannot read $Template_dir/mailListAdd.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[MAILLISTADDSCRIPTURL\]/$mailListAddScriptURL/o;
	s/\[MAILLISTLISTSCRIPTURL\]/$cancelURL/;
	s/\[MAILLISTOPENHELP\]/$mailListOpenHelp/;
	s/\[USER_INFO\]/$info/;
	s/\[USERLIST\]/$userList/;
	s/\[GROUP\]/$group/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;


	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print $handle $_;
    }
    close TEMPLATE;	    
}

1;
