#!/usr/bin/perl

# author: Kevin K.M. Chiu
# version: 2.0
# date: 07-01-99
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::mailListList;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::List;
use Cobalt::User;

# use strict;
use Locale::gettext;

sub main {

my $q=shift;

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";

my $group;
my $info;
if( $ENV{ PARAM_GROUP } || $ENV{ PARAM_INFO } ) {
    $group=$ENV{ PARAM_GROUP };
    $info=$ENV{ PARAM_INFO };
}
else {

    $group=$q->param( 'group' );
    $info=$q->param( 'info' );
    my $user = $ENV{REMOTE_USER};
    unless ($group and user_authok($user, "site-adm", $group)) {
	print "location: $forbiddenURL\n\n";
	die "Unauthorised access by $user, not a member of $group.\n";
    } 
}

# global variables
my $mailListAddScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/mailListAdd";
my $mailListModScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/mailListMod";
my $mailListDelScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/mailListDel";

# get a list of mailing lists
my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
my @mailLists = list_lists($group, $fqdn);
if ($mailLists[0] =~ /^[134]\S*\s/) {
    # substr( $mailLists[0], 5 ).'<BR>';
    @mailLists = ();
}

# only displays if >=13 mailing lists
my $buttons='';
if( $#mailLists>=13 ) {
    my $addMailList=eval("\"".Locale::gettext::dgettext("ui", "addMailList")."\"");
    $buttons=<<END;
<A HREF="$mailListAddScriptURL?group=$group" onMouseOver="status='$addMailList'; return true;"><IMG SRC="/.cobalt/images/add_mail_list_but" BORDER=0></A>
END
}

# print mailing lists
my $mailListTable;
my $i;
for( $i=0; $i<=$#mailLists; $i++ ) {
    my $listName=$mailLists[$i];
    my @members=list_get_mem($group, $fqdn, $listName);
    if ($members[0] =~ /^[134]\S*\s/) {
	# substr( $members[0], 5 ).'<BR>';
	@members = ();
    }
    map { $_ =~ s/\@${fqdn}//; } @members;

    my $deletable='true';
    my $deleteIcon='/libImage/remove';
    my $recipients=join ', ', sort @members;

    my $modifyMailListX=eval("\"".Locale::gettext::dgettext("ui", "modifyMailListX")."\"");
    my $removeMailList=eval("\"".Locale::gettext::dgettext("ui", "removeMailList")."\"");

    $mailListTable.=<<END;
<TR>
  <TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$listName</TD>
  <TD BGCOLOR="#FFFFFF"><FONT COLOR=#FFFFFF>&nbsp;</FONT>$recipients</TD>
  <TD NOWRAP BGCOLOR="#FFFFFF"><CENTER>
    <A HREF="javascript: modify( '$listName' );" onMouseOver="status='$modifyMailListX'; return true;"><IMG SRC="/libImage/modify" ALT="$modifyMailListX" BORDER="0"></A><FONT COLOR=#FFFFFF>&nbsp;</FONT><A HREF="javascript: del( '$listName', $deletable );" onMouseOver="status='$removeMailList'; return true;"><IMG SRC="$deleteIcon" ALT="$removeMailList" BORDER="0"></A>
  </CENTER></TD>
</TR>
END
}

my $hideTitleStart='';
my $hideTitleEnd='';
if( $#mailLists<0 ) {
    # if no mailing list
    $mailListTable="<TR><TD COLSPAN=3><FONT STYLE=$table_style>&nbsp;<I>".eval("\"".Locale::gettext::dgettext("ui", "noMailLists")."\"")."</I></FONT></TD></TR>";
    $hideTitleStart='<!-- ';
    $hideTitleEnd=' -->';
}

# Read in the template
open( TEMPLATE, "$Template_dir/mailListList.tmpl" )
    || die "Cannot read $Template_dir/mailListList.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


while( <TEMPLATE> ) {
    s/\[BUTTONS\]/$buttons/;
    s/\[GROUP\]/$group/;
    s/\[HIDETITLESTART\]/$hideTitleStart/;
    s/\[HIDETITLEEND\]/$hideTitleEnd/;
    s/\[MAILLIST_TABLE\]/$mailListTable/;
    s/\[MAILLISTADDSCRIPTURL\]/$mailListAddScriptURL/o;
    s/\[MAILLISTDELSCRIPTURL\]/$mailListDelScriptURL/o;
    s/\[MAILLISTMODSCRIPTURL\]/$mailListModScriptURL/o;
    s/\[USER_INFO\]/$info/;
    s/\[FONTH\]/$header_style/g;
    s/\[FONTT\]/$table_style/g;
    s/\[FONTB\]/$button_style/g;


    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

    print;
}
close TEMPLATE;

}
1;
