#!/usr/bin/perl

# author: Kevin K.M. Chiu
# version: 2.0
# date: 07-01-99
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::mailListMod;

use Cobalt::Product;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::User;
use Cobalt::Util;
use Cobalt::Vsite;
use Cobalt::Html;

# use strict;
use Locale::gettext;
use URI::Escape;

my ($q,$locale,$listName,$page,$group,$user);
my $mailListListScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/mailListList';
my $mailListModScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/mailListMod';
my $forbiddenURL="/.cobalt/error/forbidden.html";

# global variables

sub main {

$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

$page=$q->param( 'page' );
$listName=$q->param( 'listname' );

$group=$q->param( 'group' );

$user = $ENV{REMOTE_USER};
unless (user_authok($user, "site-adm", $group)) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorised access by $user, not a member of $group.\n";
}

if( $page eq 'maillistmod' ) { 
    return handleMailListModPage(); 
} else {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    return genMailListModPage( \*STDOUT, $group );
}

}


#
# event handler to modify mailing list page
#
sub handleMailListModPage {
    my $err=0;
    my $info;
    # my $group=$q->param( 'group' );
    my $oldListName=$q->param( 'oldlistname' );
    my $recipientsArea=$q->param( 'recipients' );
    my @userRecipients=$q->param( '_userrecipients_value' );
    my $usePassword=$q->param( 'passwd' );
    my $spam=$q->param( 'spam' );
    my $open=$q->param( 'open' );
		my $password=$q->param( 'mailpass0' );

		$password = $usePassword ? $password : ''; 
    my @data;

    # drop the first elemet (blank)
    if ($userRecipients[0] eq '') {
	shift @userRecipients;
    }

    # get individual recipients
    my @recipients=split /[\s,]/m, $recipientsArea;

    my $recipientErr=0;
    my $allRecipientsOK=1;
    my @resultRecipients;
    my $recipient;
    foreach (@recipients) {
	next unless ($_);
	unless (netutil_test_emailaddr($_)) {
	    if ($allRecipientsOK) {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "recipientsNotValid")."\"");
		$allRecipientsOK=0;
	    }
	    $info.=" $_";
	    $recipientErr=1;
	} else {
	    push @resultRecipients, $_;
	}
    }
    $info.='<BR>' unless ($allRecipientsOK);
    @recipients=@resultRecipients;

    # check if listname is ok
    my @usersInGroup = vsite_get_users($group);
    foreach ( @usersInGroup ) {
	if ($_ eq $listName ) {
	    $err = 1;
	}
    }
    if ($err) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\""); 
    } elsif (! $listName) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "noMailListName")."\"");
	$err=1;
    } elsif ( $System_users {$listName} ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
	$err=1;
    } elsif ( $MailSystemUsers {$listName} ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
	$err=1;
    } else {
	# everything is fine
	my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
	my $ret = list_modify ($group, $fqdn, $oldListName, $listName,
			       map("$_\@${fqdn}", @userRecipients), @recipients);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}
	# set mailing list password
	$ret = list_config_password($group, $fqdn, $listName, $password);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the open subscription option
	if ($open) {
	    $ret=list_config_open($group, $fqdn, $listName);
	} else {
	    $ret=list_config_closed($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the restricted posting option
	if ($spam) {
	    $ret=list_config_norestrict($group, $fqdn, $listName);
	} else {
	    $ret=list_config_restrict($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# add X-loop headers to messages going to list
	@data = ("X-Loop: Sent from $listName\@$fqdn");
	$ret = Cobalt::List::list_config_update($group, $fqdn, $listName,
				  "message_headers", \@data);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# and don't allow them to be reposted
	@data = ("/^X-Loop: Sent from $listName\\\@$fqdn/i");
	$ret = Cobalt::List::list_config_update($group, $fqdn, $listName,
				  "taboo_headers", \@data);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}


	    
    }

    if ($err || $recipientErr) {
        $info = uri_escape($info,"\0-\377");
        print "location: $mailListListScriptURL\?group=$group&info=$info \n\n";
    } else {
        print "location: $mailListListScriptURL\?group=$group \n\n";
    }

  undef ($q,$locale,$listName,$page,$group,$user, @data);

}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the infomation shown to user
#
sub genMailListModPage {
    my ( $handle, $group, $info )=@_;

    my $oldListName=$listName;
    my $mailListNameField="<INPUT TYPE=\"TEXT\" NAME=\"listname\" VALUE=\"$listName\" SIZE=\"15\" onChange=\"checkListName( this, mailListNameNotValid );\">";

		my $selectallVal = <<END;
<CENTER>
<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0"><TR>
  <TD><A HREF="javascript: setAllUsers('value');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonLeftDot.gif" BORDER=0></A></TD>
  <TD NOWRAP STYLE="[FONTB]" ><A HREF="javascript: setAllUsers('value');" onMouseOver="status='[str.selectAllUsers]'; return true;"><FONT STYLE="[FONTB]">[str.selectAllUsers]</FONT></A></TD>
  <TD><A HREF="javascript: setAllUsers('value');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonRight.gif" BORDER=0></A></TD>
</TR></TABLE></CENTER>

END

		my $selectallEntry = <<END;
<CENTER>
<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0"><TR>
  <TD><A HREF="javascript: setAllUsers('entries');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonLeftDot.gif" BORDER=0></A></TD>
  <TD NOWRAP STYLE="[FONTB]" ><A HREF="javascript: setAllUsers('entries');" onMouseOver="status='[str.selectAllUsers]'; return true;"><FONT STYLE="[FONTB]">[str.selectAllUsers]</FONT></A></TD>
  <TD><A HREF="javascript: setAllUsers('entries');" onMouseOver="status='[str.selectAllUsers]'; return true;"><IMG SRC="/libImage/buttonRight.gif" BORDER=0></A></TD>
</TR></TABLE></CENTER>

END

    # get all list members
    my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
    my @listMembers = list_get_mem($group, $fqdn, $listName);
    if ($listMembers[0] =~ /^[134]\S*\s/) {
	@listMembers = ();
    }

    my $mailListName = $oldListName . '@'. $fqdn;

    # get list password
    my $password = (Cobalt::List::list_config_getpassword($group, $fqdn, $listName));
		my $passwordChecked = $password ? 'CHECKED' : '';

    # get list subscription policy
    my $open = (Cobalt::List::list_config_getstate($group, $fqdn, $listName)) ? "CHECKED" : '';

    # get list restricted status
    my $spam = (Cobalt::List::list_config_getrestrict($group, $fqdn, $listName)) ? '' : "CHECKED";

    # do not show admin
    my @users=vsite_get_users($group);
    if ($users[0] =~ /^[134]\S*\s/) {
	@users = ();
    }
    my ($i,@selected,@usersF,$u,$userList);
    if (scalar(@users)) {
			@users = sort @users;
			foreach (@users) {
	    	$u = $_ . '@' . $fqdn;
	    	push @usersF, $u;
				if (grep { /^$u$/ } @listMembers) {
					push @selected, $_; 
				}
			}
			$userList = genSetSelector("userrecipients", \@selected, \@users, '[str.usersSubscribed]', '[str.usersNotSubscribed]', $selectallVal, $selectallEntry);

    } else {
	my $noUsers=eval("\"".Locale::gettext::dgettext("ui", "noUsers")."\"");
	$userList = "<FONT STYLE=\"$table_style\">&nbsp;<I>$noUsers</I></FONT>\n";
    }

    my $cancelURL = $mailListListScriptURL . "?group=$group";
    my $mailListOpenHelp=MSG_get_ui("mailListOpenHelp", 'majordomo@'.$fqdn);

    # find recipients
    my @recipientsList;
    my $lm;
    foreach $lm (@listMembers) {
	unless (grep { /^$lm$/ } @usersF) { push @recipientsList, $lm; }
    }
    my $recipients=join "\n", sort @recipientsList;

    # Read in the template
    open( TEMPLATE, "$Template_dir/mailListMod.tmpl" )
	|| die "Cannot read $Template_dir/mailListMod.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[MAILLISTLISTSCRIPTURL\]/$cancelURL/;
	s/\[MAILLISTMODSCRIPTURL\]/$mailListModScriptURL/o;
	s/\[MAILLISTNAMEFIELD\]/$mailListNameField/;
	s/\[MAILLISTOPENHELP\]/$mailListOpenHelp/;
	s/\[OLDMAILLISTNAME\]/$oldListName/;
	s/\[RECIPIENTS\]/$recipients/;
	s/\[USER_INFO\]/$info/;
	s/\[GROUP\]/$group/;
	s/\[USERLIST\]/$userList/;
	s/\[PASSWORDCHECKED\]/$passwordChecked/;
	s/\[PASSWORD\]/$password/;
	s/\[MAILLISTNAME\]/$mailListName/;
	s/\[OPENCHECKED\]/$open/;
	s/\[SPAMCHECKED\]/$spam/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print $handle $_;
    }
    close TEMPLATE;

    undef ($q,$locale,$listName,$page,$group,$user);
}

1;
