#!/usr/bin/perl

# author: Kevin K.M. Chiu
#         Will DeHaan
#       Cobalt Networks
# version: 4.0
# date: 07-02-99
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::network;

use Cobalt::Product;
use Cobalt::Admserv;
use Cobalt::Email;
use Cobalt::Locale;
use Cobalt::Layout;
use Cobalt::Netutil;
use Cobalt::Network;
use Cobalt::Services;
use Cobalt::Vsite;
use Cobalt::Ftp;
use Cobalt::Meta;
use Cobalt::List;
use Cobalt::Util;
use Cobalt::User;

use DB_File;
# use strict;
use Locale::gettext;

# global variables
my ( $q, $page, $locale, $button_style, $table_style, $header_style );
my $networkScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/network';

sub main {

$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
  $button_style=$locale->getProperty("buttonStyle");
  $header_style=$locale->getProperty("headerStyle");
  $table_style=$locale->getProperty("tableStyle");
 }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

my $adminPageURL='/.cobalt/admin/indexw.html?reload';

$page=$q->param( 'page' );

if( $page eq 'network1' || $page eq 'network2' ) { handleNetworkPage($q); }
elsif( $page eq 'networkconfirm1' || $page eq 'networkconfirm2' ) { handleNetworkConfirmPage( $q ); }
else { genNetworkPage( $q ); }

}


sub handleRebootPage {
    my $url = shift;

    # Read in the template
    open( TEMPLATE, "$Template_dir/rebooting.tmpl" )
	|| die "Cannot read $Template_dir/rebooting.tmpl: $!\n";

    my $msg = MSG_get_ui("rebooturl", $url); 

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    while( <TEMPLATE> ) {
	s/\[RETURNURLMSG\]/$msg/;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/ge;
	print;
    }
    close TEMPLATE;
}


sub handleNetworkPage {
    my $q=shift;
    # network parameter submitted
    my $hostName=$q->param( 'hostname' );
    my $domainName=$q->param( 'domainname' );
    my $dnsAddr1=$q->param( 'dnsaddr1' );
    my $dnsAddr2=$q->param( 'dnsaddr2' );
    my $ipAddr0=$q->param( 'ipaddr0' );
    my $ipAddr1=$q->param( 'ipaddr1' );
    my $netMask0=$q->param( 'netmask0' );
    my $netMask1=$q->param( 'netmask1' );
    my $gateway=$q->param( 'gateway' );
    my $route=$q->param( 'route' );
    my $nat0=$q->param( 'nat0' );
    my $wizard=$q->param( 'wizard' );

    my $result='';
    my $err=0;
    my $ret;

    # We can't duplicate hostnames
    my($vsite_instance);
    my(@vsite_data)=vsite_list();
    my($name)="$hostName.$domainName";
    foreach $vsite_instance (@vsite_data)
    {
	if ((@$vsite_instance[1] eq $name) &&
	    (@$vsite_instance[2] ne $MainGroup))
	{
	    $result.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_hostnameinuse")."\"");
	    $err=1;
	    last;
	}
    }
    my $oldIPAddr0=( Cobalt::Network::net_get_ipaddr( $Netif ) )[ 0 ];
    my(%ftpconfig)=Cobalt::Ftp::ftp_list_config();
    # error check
    if ($err) {}
    elsif($ftpconfig{$ipAddr0} && $ftpconfig{$oldIPAddr0} && $ipAddr0 ne $oldIPAddr0) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "ftp_cant_ipalreadyinuse")."\"");
	$err=1;
    }
    elsif( !$hostName ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "noHostName")."\"");
	$err=1;
    }
    elsif( $hostName=~/localhost/io ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "localhostNotAllowed")."\"");
	$err=1;
    }
    elsif( $hostName && !Cobalt::Netutil::netutil_test_domainname( $hostName ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "hostNameNotValid")."\"");
	$err=1;
    }
    elsif( $domainName && !Cobalt::Netutil::netutil_test_domainname( $domainName ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "domainNameNotValid")."\"");
	$err=1;
    }
    elsif( !$wizard && !$ipAddr0 ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "noIPAddr0")."\"");
	$err=1;
    }
    elsif( !$wizard && !Cobalt::Netutil::netutil_test_ipaddr( $ipAddr0 ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "addrNotValid0")."\"");
	$err=1;
    }
    elsif( $ipAddr1 && !Cobalt::Netutil::netutil_test_ipaddr( $ipAddr1 ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "addrNotValid1")."\"");
	$err=1;
    }
    elsif( $page eq 'network2' && $ipAddr0 eq $ipAddr1 ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "addrNotUnique")."\"");
	$err=1;
    }
    elsif( !$wizard && !$netMask0 ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "noNetMask0")."\"");
	$err=1;
    }
    elsif( $ipAddr1 && !$netMask1 ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "noNetMask1")."\"");
	$err=1;
    }
    elsif( !$wizard && !Cobalt::Netutil::netutil_test_ipaddr( $netMask0 ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "maskNotValid0")."\"");
	$err=1;
    }
#    elsif( !$wizard && $page eq 'network2' && !Cobalt::Netutil::netutil_test_ipaddr( $netMask1 ) ) {
    elsif( $netMask1 && !Cobalt::Netutil::netutil_test_ipaddr( $netMask1 ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "maskNotValid1")."\"");
	$err=1;
    }
    elsif( $dnsAddr1 && !Cobalt::Netutil::netutil_test_ipaddr( $dnsAddr1 ) ) {
	my $dnsAddr=$dnsAddr1;
	$result.=eval("\"".Locale::gettext::dgettext("ui", "dnsNotValid")."\"");
	$err=1;
    }
    elsif( $dnsAddr2 && !Cobalt::Netutil::netutil_test_ipaddr( $dnsAddr2 ) ) {
	my $dnsAddr=$dnsAddr2;
	$result.=eval("\"".Locale::gettext::dgettext("ui", "dnsNotValid")."\"");
	$err=1;
    }
    elsif( $gateway && !Cobalt::Netutil::netutil_test_ipaddr( $gateway ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "gatewayNotValid")."\"");
	$err=1;
    }
    elsif( $gateway && $page eq 'network1' && !Cobalt::Netutil::netutil_test_network_in_network( $gateway, '255.255.255.255', $ipAddr0, $netMask0 ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "gatewayNotInNetwork1")."\"");
	$err=1;
    }
    elsif( $gateway && $ipAddr1 && !Cobalt::Netutil::netutil_test_network_in_network( $gateway, '255.255.255.255', $ipAddr0, $netMask0 ) && !Cobalt::Netutil::netutil_test_network_in_network( $gateway, '255.255.255.255', $ipAddr1, $netMask1 ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "gatewayNotInNetwork2")."\"");
	$err=1;
    }
    elsif( Cobalt::Netutil::netutil_get_hostip( $ipAddr0, $netMask0 ) eq '0.0.0.0' ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "ipAddrNoHost0")."\"");
	$err=1;
    }
    elsif( $ipAddr1 && Cobalt::Netutil::netutil_get_hostip( $ipAddr1, $netMask1 ) eq '0.0.0.0' ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "ipAddrNoHost1")."\"");
	$err=1;
    }
    elsif( $ipAddr1 && Cobalt::Netutil::netutil_get_network( $ipAddr0, $netMask0 ) eq Cobalt::Netutil::netutil_get_network( $ipAddr1, $netMask1 ) ) {
	$result.=eval("\"".Locale::gettext::dgettext("ui", "interfaceNotSubnets")."\"");
	$err=1;
    }
    else {
	# everything is fine

	# change IP address?
	my $ipConfirm=0;
	if( !$wizard && ( $oldIPAddr0 ne $ipAddr0 ) ) { $ipConfirm=1; }
	my $oldIPAddr1=( Cobalt::Network::net_get_ipaddr( $Netif1 ) )[ 0 ];
	if( $oldIPAddr1 eq '0.0.0.0' ) { $oldIPAddr1=''; }
	if( !$wizard && ( $page eq 'network2' ) && ( $oldIPAddr1 ne $ipAddr1 ) ) { $ipConfirm=1; }

	# check domain name, dns servers and gateway
        my $oldHostName=net_get_hostname();
        $oldHostName='' if ($oldHostName =~ /^4\d\d\d /);

	my $dns1Confirm=0;
	my $dns2Confirm=0;
	my $gatewayConfirm=0;
	my ( $oldDomainName, $oldDNSAddr1, $oldDNSAddr2 )=( Cobalt::Network::net_get_dnsclient() );
	if( $dnsAddr1 && ( $dnsAddr1 ne $oldDNSAddr1 ) && Cobalt::Netutil::netutil_test_dns_server( $dnsAddr1 )!~/^2/o ) { $dns1Confirm=1; }
	if( $dnsAddr2 && ( $dnsAddr2 ne $oldDNSAddr2 ) && Cobalt::Netutil::netutil_test_dns_server( $dnsAddr2 )!~/^2/o ) { $dns2Confirm=1; }
	if( !$wizard && $gateway && ( $gateway ne ( Cobalt::Network::net_get_gateway() )[ 0 ] ) && Cobalt::Netutil::netutil_ping( $gateway )!~/^2/o ) { $gatewayConfirm=1; }

	# Generate confirmation if necessary
	if( $dns1Confirm || $dns2Confirm || $gatewayConfirm || $ipConfirm ) {
	    genNetworkConfirmPage( !$wizard, $hostName, $domainName, $dnsAddr1, $dnsAddr2, $ipAddr0, $netMask0, $ipAddr1, $netMask1, $gateway, $route, $nat0, 0, $dns1Confirm, $dns2Confirm, $ipConfirm, $gatewayConfirm );
	    return;
	}
	else {
	    if( !$wizard ) {
		# remove any virt interfaces at dest. ip
		my (@allSites)=Cobalt::Vsite::vsite_list();
		my ($vSite);
		foreach $vSite (@allSites) {
			Cobalt::Network::net_del_vinterface( $ipAddr0 )
				if ( @$vSite[0] eq $ipAddr0 );
		}
		$ret=Cobalt::Network::net_set_ipaddr( $ipAddr0, $netMask0, $Netif );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}

		if( $page eq 'network2' ) {
		    if ($ipAddr1) {
			$ret=Cobalt::Network::net_set_ipaddr( $ipAddr1, 
							      $netMask1, 
							      $Netif1 ); 
			if( $ret!~/^2/o ) {
			    $result.=substr( $ret, 4 ).'<BR>';
			    $err=1;
			}
		    } else {
			&disable_eth1($oldIPAddr1);
		    }
		}

		# gatewayInterface must be either of the interfaces because of the error checking above
		# $Netif1 is chosen if the gateway is on the network of both interfaces because it's supposed to be the Internet side
		my $gatewayInterface=$Netif;
		if( $ipAddr1 && Cobalt::Netutil::netutil_test_network_in_network( $gateway, '255.255.255.255', $ipAddr1, $netMask1 ) ) { $gatewayInterface=$Netif1; }
#		if( $page eq 'network2' && Cobalt::Netutil::netutil_test_network_in_network( $gateway, '255.255.255.255', $ipAddr1, $netMask1 ) ) { $gatewayInterface=$Netif1; }

		my $gatewayParam=$gateway;
		if( !$gateway ) { $gatewayParam='none'; }
		$ret=Cobalt::Network::net_set_gateway( $gatewayParam, $gatewayInterface );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
	    }

	    $ret=Cobalt::Network::net_set_dnsclient( $domainName, $dnsAddr1, $dnsAddr2 );
	    if( $ret!~/^2/o ) {
		$result.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }

	    if( !$err ) {
		$ret=Cobalt::Network::net_set_hostname( $hostName );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		} else { 
		    my($oldfqdn)="$oldHostName.$oldDomainName";
		    my($fqdn)="$hostName.$domainName";

                    layout_siteManage($MainGroup, $fqdn);

		    # update sendmail.cw and email routes
		    my(%routes)=Cobalt::Email::mail_virtuser_list_route();
		    if( $routes{$oldDomainName} eq $oldfqdn && $oldfqdn ne $fqdn) {
			$ret=Cobalt::Email::mail_del_cw( $oldDomainName );
			if( $ret!~/^2/o ) {
			    $result.=substr( $ret,4 ).'<BR>';
			    $err=1;
			}
			$ret=Cobalt::Email::mail_del_relay( $oldDomainName );
			if( $ret!~/^2/o ) {
			    $result.=substr( $ret, 4 ).'<BR>';
			    $err=1;
			}
			$ret=Cobalt::Email::mail_virtuser_del_route( $oldDomainName, $oldfqdn );
			if( $ret!~/^2/o ) {
			    $result.=substr( $ret,4 ).'<BR>';
			    $err=1;
			}
		    }

		    if( $oldfqdn ne $fqdn ) {
			# replace sendmail.cw fqdn
			$ret=Cobalt::Email::mail_del_cw( $oldfqdn );
			if( $ret!~/^2/o ) { 
			    $result.=substr( $ret,4 ).'<BR>'; 
			    $err=1;
			}
			$ret=Cobalt::Email::mail_add_cw( $fqdn );
			if( $ret!~/^2/o ) { 
			    $result.=substr( $ret,4 ).'<BR>'; 
			    $err=1;
			}			

			# change email aliases
			$ret=mail_virtuser_change_domain( $oldfqdn, $fqdn );
			if( $ret!~/^2/o ) {
			    $result.=substr( $ret, 4 ).'<BR>';
			    $err=1;
			}

			# update the majordomo stuff
			$ret=list_modify_virtual($MainGroup,$oldfqdn,$fqdn);
			if( $ret!~/^2/o ) {
			    $result.=substr( $ret, 4 ).'<BR>';
			    $err=1;
			}
		    }			

                    # update the database
		    my ($obj);
		    if (! $err) {
		      Cobalt::Meta::update("type"   => "vsite",
					   "change" => ["ipaddr", "$ipAddr0",
							"domain", "$domainName",
							"hostname", "$hostName",
							"fqdn", "$fqdn"],
					   "where"  => ["name", "=", "home"]);
		    }
	        }
	    }

	    if( $route ) {
		# call net_set_forwarding() after nat_set_rules()
		# because nat_set_rules() changes IP forwarding
		if( $route eq 'off' ) { $ret=Cobalt::Network::net_set_forwarding( 0 ); }
		else { $ret=Cobalt::Network::net_set_forwarding( 1 ); }
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}

		# set gated
		if( $route eq 'dynamic' ) {
		    $ret=Cobalt::Services::set_gated_interfaces( $Netif, $Netif1 );
		    if( $ret=~/^2/o ) { $ret=Cobalt::Services::set_gated_on( 1 ); }
		}
		else { $ret=Cobalt::Services::set_gated_on( 0 ); }
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
	    }

	    # turn on DNS server if necessary
	    tryTurnOnDNS( $ipAddr0, $ipAddr1, $dnsAddr1, $dnsAddr2 );

	    # generate modified pages
	    # update host name and domain name related information
	}
    }

    if( !$err ) {
	if( $wizard ) { print "location: $adminPageURL\n\n"; }
	else { genNetworkPage( $q ); }
	
	# Update Apache
	&Cobalt::Vsite::vsite_sync();

	# Update the main site admin site

    }
    else { genNetworkPage( $q, $result, 1 ); }
}

sub handleNetworkConfirmPage {
    my $q=shift;
    # read form inputs
    my $hostName=$q->param( 'hostname' );
    my $domainName=$q->param( 'domainname' );
    my $dnsAddr1=$q->param( 'dnsaddr1' );
    my $dnsAddr2=$q->param( 'dnsaddr2' );
    my $ipAddr0=$q->param( 'ipaddr0' );
    my $netMask0=$q->param( 'netmask0' );
    my $ipAddr1=$q->param( 'ipaddr1' );
    my $netMask1=$q->param( 'netmask1' );
    my $gateway=$q->param( 'gateway' );
    my $route=$q->param( 'route' );
    my $nat0=$q->param( 'nat0' );
    my $receiveEmail=$q->param( 'receiveemail' );
    my $wizard=$q->param( 'wizard' );

     my $oldHostName=net_get_hostname();
     $oldHostName='' if ($oldHostName =~ /^4\d\d\d /);

#    print "$hostName:$domainName:$ipAddr:$netMask:$dnsAddr1:$dnsAddr2:$gateway:$wizard";
#    return;

    my ( $oldDomainName, $oldDNSAddr1, $oldDNSAddr2 )=Cobalt::Network::net_get_dnsclient();
    if( !$dnsAddr1 ) { $dnsAddr1=$oldDNSAddr1; }
    if( !$dnsAddr2 ) { $dnsAddr2=$oldDNSAddr2; }

    if( !$gateway ) { $gateway=( Cobalt::Network::net_get_gateway() )[ 0 ]; }

    my ( $oldIPAddr0, $oldNetMask0 )=Cobalt::Network::net_get_ipaddr( $Netif );
#    if( !$ipAddr0 ) { $ipAddr0=$oldIPAddr0; }
#    if( !$netMask0 ) { $netMask0=$oldNetMask0; }

    my ( $oldIPAddr1, $oldNetMask1 );
    if( $page eq 'networkconfirm2' ) {
	( $oldIPAddr1, $oldNetMask1 )=Cobalt::Network::net_get_ipaddr( $Netif1 );
	if( $oldIPAddr1 eq '0.0.0.0' ) { $oldIPAddr1=''; }
	if( $oldNetMask1 eq '0.0.0.0' ) { $oldNetMask1=''; }
#	if( !$ipAddr1 ) { $ipAddr1=$oldIPAddr1; }
#	if( !$netMask1 ) { $netMask1=$oldNetMask1; }
    }

    my $result='';
    my $err=0;
    my $ret;

    # set configuration
    $ret=Cobalt::Network::net_set_dnsclient( $domainName, $dnsAddr1, $dnsAddr2 );
    if( $ret!~/^2/o ) {
	$result.=substr( $ret, 4 ).'<BR>';
	$err=1;
    }

    if( !$err ) {
	$ret=Cobalt::Network::net_set_hostname( $hostName );
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	} else {
	    my($oldfqdn)="$oldHostName.$oldDomainName";
	    my($fqdn)="$hostName.$domainName";
	    
	    layout_siteManage($MainGroup, $fqdn);
	    
	    # update sendmail.cw and email routes
	    my(%routes)=Cobalt::Email::mail_virtuser_list_route();
	    if( $routes{$oldDomainName} eq $oldfqdn && $oldfqdn ne $fqdn) {
		$ret=Cobalt::Email::mail_del_cw( $oldDomainName );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret,4 ).'<BR>';
		    $err=1;
		}
		$ret=Cobalt::Email::mail_del_relay( $oldDomainName );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
		$ret=Cobalt::Email::mail_virtuser_del_route( $oldDomainName, $oldfqdn );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret,4 ).'<BR>';
		    $err=1;
		}
	    }
	    
	    if( $oldfqdn ne $fqdn ) {
		# replace sendmail.cw fqdn
		$ret=Cobalt::Email::mail_del_cw( $oldfqdn );
		if( $ret!~/^2/o ) { 
		    $result.=substr( $ret,4 ).'<BR>'; 
		    $err=1;
		}
		$ret=Cobalt::Email::mail_add_cw( $fqdn );
		if( $ret!~/^2/o ) { 
		    $result.=substr( $ret,4 ).'<BR>'; 
		    $err=1;
		}
		# change email aliases
		$ret=mail_virtuser_change_domain( $oldfqdn, $fqdn );
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
		
		# update the majordomo stuff
		$ret=list_modify_virtual($MainGroup,$oldfqdn,$fqdn);
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
	    }

	    # update the database
	    my ($obj);
	    if (! $err) {
	      Cobalt::Meta::update("type"   => "vsite",
				   "change" => ["ipaddr", "$ipAddr0",
						"domain", "$domainName",
						"hostname", "$hostName",
						"fqdn", "$fqdn"],
				   "where"  => ["name", "=", "home"]);
	    }
        }
    }

    my $reboot = 0;
    if( !$err ) {
	# we have to do this here because it will be too late to do
	# so if net_set_ipaddr succeeds.
        if (($ipAddr0 ne $oldIPAddr0) or ($ipAddr1 ne $oldIPAddr1)) {
	    $reboot = 1;
	    # fork to avoid problems with apache closing the process.
	    my $pid;
	    FORK: {
		if ($pid = fork) {
		    handleRebootPage("http://$ipAddr0/admin/ TARGET=\"_top\"");
		    exit 0;
		} elsif (defined $pid) {
		    close(\*STDOUT);
		    last;
		} elsif ($! =~ /No more process/) {
		    sleep 5;
		    redo FORK;
		} # if we can't fork, just continue and hope for the best.
	    }
        }

	# update vsite ip's in database
	if ("$ipAddr0" ne "$oldIPAddr0") {
	    &Cobalt::Meta::update("type"   => "bw",
				  "change" => ["name", "$ipAddr0"],
				  "where"  => ["name", "=", "$oldIPAddr0"]);
	    &Cobalt::Meta::update("type"   => "vsite",
				  "change" => ["ipaddr", "$ipAddr0"],
				  "where"  => ["ipaddr", "=", "$oldIPAddr0"]);
	}

	# remove any virt interfaces at dest. ip
	my (@allSites)=Cobalt::Vsite::vsite_list();
	my ($vSite);
	foreach $vSite (@allSites) {
		Cobalt::Network::net_del_vinterface( $ipAddr0 )
			if ( @$vSite[0] eq $ipAddr0 );
	}

	# do this last
	$ret=Cobalt::Network::net_set_ipaddr( $ipAddr0, $netMask0, $Netif );
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	if( $page eq 'networkconfirm2' ) {
	    if( $ipAddr1 ) { 
		$ret=Cobalt::Network::net_set_ipaddr( $ipAddr1, $netMask1, 
						      $Netif1 ); 
		if( $ret!~/^2/o ) {
		    $result.=substr( $ret, 4 ).'<BR>';
		    $err=1;
		}
	    } else {
		&disable_eth1( (Cobalt::Network::net_get_ipaddr( $Netif1 ) )[ 0 ] );
	    }
	}

	# $Netif1 is chosen if the gateway is on the network of both interfaces because it's supposed to be the Internet side
	my $gatewayInterface=$Netif;
	if( $ipAddr1 && Cobalt::Netutil::netutil_test_network_in_network( $gateway, '255.255.255.255', $ipAddr1, $netMask1 ) ) { $gatewayInterface=$Netif1; }
#	if( $page eq 'networkconfirm2' && Cobalt::Netutil::netutil_test_network_in_network( $gateway, '255.255.255.255', $ipAddr1, $netMask1 ) ) { $gatewayInterface=$Netif1; }

	my $gatewayParam=$gateway;
	if( !$gateway ) { $gatewayParam='none'; }
	$ret=Cobalt::Network::net_set_gateway( $gatewayParam, $gatewayInterface );
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if( $route ) {
	# call net_set_forwarding() after nat_set_rules()
	# because nat_set_rules() changes IP forwarding
	if( $route eq 'off' ) { $ret=Cobalt::Network::net_set_forwarding( 0 ); }
	else { $ret=Cobalt::Network::net_set_forwarding( 1 ); }
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# set gated
	if( $route eq 'dynamic' ) {
	    $ret=Cobalt::Services::set_gated_interfaces( $Netif, $Netif1 );
	    if( $ret=~/^2/o ) { $ret=Cobalt::Services::set_gated_on( 1 ); }
	}
	else { $ret=Cobalt::Services::set_gated_on( 0 ); }
	if( $ret!~/^2/o ) {
	    $result.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    # turn on DNS server if necessary
    tryTurnOnDNS( $ipAddr0, $ipAddr1, $dnsAddr1, $dnsAddr2 );

    if( $reboot ) {
	# no more DHCP
	# AAAAAAAAAAAAAIIIIIIIIIIIIIIIIIEEEEEEEEEEEEEEEEEEE!
	# WHY IS THE UI DIRECTLY HACKING THE SYSTEM?  THIS IS A
	# VERY BAD PRACTICE!
	# If the sauce doesn't do what you need, change the sauce;
	# don't bypass it.
	open( NETCONFIG, ">$Netconfig" );
	print NETCONFIG "BOOTPROTO=lcd";
	close( NETCONFIG );

	# Update Apache
	&Cobalt::Vsite::vsite_sync();

        # Migrate sites for non-$Netif ethernet interface
        &Cobalt::Vsite::vsite_migrate_ipaddr($oldIPAddr1,$ipAddr1) if ($oldIPAddr1 && $ipAddr1);

	if ($ipAddr0 ne $oldIPAddr0) {
	    &Cobalt::Network::net_del_vinterface($oldIPAddr0);
	}

	`/sbin/shutdown -r now`;
    } else {
	if( !$err ) {
	    if( $wizard ) { print "location: $adminPageURL\n\n"; }
	    else { genNetworkPage( $q ); }

	    # Update Apache
	    &Cobalt::Vsite::vsite_sync();
	}
	else { genNetworkPage( $q, $result, 1 ); }
    }
}

#
# argument 0 is true if page is for system management, setup wizard otherwise
# argument 1 is host name
# argument 2 is domain name
# argument 3 is primary DNS
# argument 4 is secondary DNS
# argument 5 is IP address for eth0
# argument 6 is network mask for eth0
# argument 7 is IP address for eth1
# argument 8 is network mask for eth1
# argument 9 is gateway
# argument 10 is true if dynamic routing is on
# argument 11 is true if Nat for primary interface is on
# argument 12 is true if receive email confirmation is needed, false otherwise
# argument 13 is true if primary DNS confirmation is needed, false otherwise
# argument 14 is true if secondary DNS confirmation is needed, false otherwise
# argument 15 is true if IP address will be changed, false otherwise
# argument 16 is true if gateway confirmation is needed, false otherwise
#
sub genNetworkConfirmPage {
    my ( $isSysManage, $hostName, $domainName, $dnsAddr1, $dnsAddr2, $ipAddr0, $netMask0, $ipAddr1, $netMask1, $gateway, $route, $nat0, $receiveEmailConfirm, $dnsConfirm1, $dnsConfirm2, $ipConfirm, $gatewayConfirm )=@_;

    my $pageName='networkconfirm1';
    if( $page eq 'network2' ) { $pageName='networkconfirm2'; }

    my $wizardField;
    if( !$isSysManage ) { $wizardField='<INPUT TYPE=HIDDEN NAME="wizard" VALUE="t">'; }

    my $receiveEmailCode;
    if( $receiveEmailConfirm ) {
	my $dnsAddr=$dnsAddr1;
	$receiveEmailCode="if( !confirm( '".eval("\"".Locale::gettext::dgettext("ui", "receiveEmailConfirm")."\"")."' ) ) document.forms[ 0 ].receiveemail.value='';";
    }

    my $dnsConfirmCode1;
    if( $dnsConfirm1 ) {
	my $dnsAddr=$dnsAddr1;
	$dnsConfirmCode1="if( !confirm( '".eval("\"".Locale::gettext::dgettext("ui", "dnsChangeConfirm")."\"")."' ) ) document.forms[ 0 ].dnsaddr1.value='';";
    }

    my $dnsConfirmCode2;
    if( $dnsConfirm2 ) {
	my $dnsAddr=$dnsAddr2;
	$dnsConfirmCode2="if( !confirm( '".eval("\"".Locale::gettext::dgettext("ui", "dnsChangeConfirm")."\"")."' ) ) document.forms[ 0 ].dnsaddr2.value='';";
    }

    my $gatewayConfirmCode;
    if( $gatewayConfirm ) {
	$gatewayConfirmCode="if( !confirm( '".eval("\"".Locale::gettext::dgettext("ui", "gatewayChangeConfirm")."\"")."' ) ) document.forms[ 0 ].gateway.value='';";
    }

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);
    print <<END;
<HTML>
<BODY BGCOLOR=#FFFFFF onLoad="interact()">

<FORM METHOD="POST" ACTION="$networkScriptURL">
<INPUT TYPE=HIDDEN NAME="hostname" VALUE="$hostName">
<INPUT TYPE=HIDDEN NAME="domainname" VALUE="$domainName">
<INPUT TYPE=HIDDEN NAME="dnsaddr1" VALUE="$dnsAddr1">
<INPUT TYPE=HIDDEN NAME="dnsaddr2" VALUE="$dnsAddr2">
<INPUT TYPE=HIDDEN NAME="ipaddr0" VALUE="$ipAddr0">
<INPUT TYPE=HIDDEN NAME="netmask0" VALUE="$netMask0">
<INPUT TYPE=HIDDEN NAME="ipaddr1" VALUE="$ipAddr1">
<INPUT TYPE=HIDDEN NAME="netmask1" VALUE="$netMask1">
<INPUT TYPE=HIDDEN NAME="gateway" VALUE="$gateway">
<INPUT TYPE=HIDDEN NAME="route" VALUE="$route">
<INPUT TYPE=HIDDEN NAME="nat0" VALUE="$nat0">
<INPUT TYPE=HIDDEN NAME="receiveemail" VALUE="t">

<INPUT TYPE=HIDDEN NAME="page" VALUE="$pageName">
$wizardField
</FORM>

<SCRIPT LANGUAGE="javascript">
<!-- hide
function interact() {
$receiveEmailCode
$dnsConfirmCode1
$dnsConfirmCode2
$gatewayConfirmCode
document.forms[ 0 ].submit();
}
// hide end -->
</SCRIPT>
</BODY>
</HTML>
END
}

#
# argument 0 is the information shown to the user
# argument 1 is true if error generated
#
sub genNetworkPage {
    my ( $q, $info, $isError )=@_;

    my $hostName=Cobalt::Network::net_get_hostname();
    if( $hostName=~/localhost/io ) { $hostName=''; }
    my ( $domainName, $dnsAddr1, $dnsAddr2 )=Cobalt::Network::net_get_dnsclient();
    my $gateway=( Cobalt::Network::net_get_gateway() )[ 0 ];
    my $dynamicRouteSelected='';
    my $staticRouteSelected='';
    my $offRouteSelected='';
    if( !Cobalt::Network::net_get_forwarding() ) { $offRouteSelected='SELECTED'; }
    else {
	if( Cobalt::Services::get_gated_on() ) { $dynamicRouteSelected='SELECTED'; }
	else { $staticRouteSelected='SELECTED'; }
    }

    $gateway='' unless($gateway);

    # If we're using a dual-nic machine, always report the second nic, 
    # whether it's defined or otherwise
    my $primaryOnly=1;
    my(@nics,$nicInst)=(net_get_interfaces(),'');
    foreach $nicInst (@nics) { $primaryOnly=0 if ($nicInst eq 'eth1'); }

    if( $primaryOnly || $q->param( 'wizard' ) ) {
	# only primary interface hooked up
	# update network information
	my $success='true';
	my ( $ipAddr0, $netMask0, $mac0 )=Cobalt::Network::net_get_ipaddr( $Netif );
	my($templateFile)="$Template_dir/network1.tmpl";

        $templateFile="$Template_dir/networkW1.tmpl" if($q->param('wizard'));

	if (-r $templateFile)
	{
	    # Read in the template
	    open( TEMPLATE, $templateFile )
		|| die "Cannot read $templateFile: $!\n";
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	    if( $isError ) { $success='false'; }

	    while( <TEMPLATE> ) {
	        s/\[DNSADDR1\]/$dnsAddr1/;
    	        s/\[DNSADDR2\]/$dnsAddr2/;
    	        s/\[DOMAINNAME\]/$domainName/;
	        s/\[GATEWAY\]/$gateway/g;
	        s/\[HOSTNAME\]/$hostName/;
	        s/\[IPADDR0\]/$ipAddr0/g;
	        s/\[MAC0\]/$mac0/;
	        s/\[NETMASK0\]/$netMask0/g;
	        s/\[USER_INFO\]/$info/;
		s/\[FONTH\]/$header_style/g;
		s/\[FONTT\]/$table_style/g;
		s/\[FONTB\]/$button_style/g;

	        s/\[SCRIPTURL\]/$networkScriptURL/;

	        s/\[SUCCESS\]/$success/; 

		s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	        print;
	    }
	}

	close TEMPLATE;
    }
    else {
	# primary and secondary interfaces hooked up
	# update network information
	my ( $ipAddr0, $netMask0, $mac0 )=Cobalt::Network::net_get_ipaddr( $Netif );
	my ( $ipAddr1, $netMask1, $mac1 )=Cobalt::Network::net_get_ipaddr( $Netif1 );

	if( $ipAddr1 eq '0.0.0.0' ) { $ipAddr1=''; }
	if( $netMask1 eq '0.0.0.0' ) { $netMask1=''; }

	my $success='true';
#	if( !$q->param( 'wizard' ) ) {
	    # system management

	    # Read in the template
	    open( TEMPLATE, "$Template_dir/network2.tmpl" )
		|| die "Cannot read $Template_dir/network2.tmpl: $!\n";
#	}
#	else {
	    # setup wizard

	    # Read in the template
#	    open( TEMPLATE, "$Template_dir/networkW2.tmpl" )
#		|| die "Cannot read $Template_dir/networkW2.tmpl: $!\n";

#	    if( $isError ) { $success='false'; }

#	    if( !$gateway ) { $gateway='<FONT COLOR=#FFFFFF>&nbsp;</FONT>'; }
#	}

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


	while( <TEMPLATE> ) {
	    s/\[DNSADDR1\]/$dnsAddr1/;
	    s/\[DNSADDR2\]/$dnsAddr2/;
	    s/\[DOMAINNAME\]/$domainName/;
	    s/\[DROUTESELECTED\]/$dynamicRouteSelected/;
	    s/\[GATEWAY\]/$gateway/g;
	    s/\[HOSTNAME\]/$hostName/;
	    s/\[IPADDR0\]/$ipAddr0/g;
	    s/\[IPADDR1\]/$ipAddr1/g;
	    s/\[MAC0\]/$mac0/;
	    s/\[MAC1\]/$mac1/;
	    s/\[NETMASK0\]/$netMask0/g;
	    s/\[NETMASK1\]/$netMask1/g;
	    s/\[OFFROUTESELECTED\]/$offRouteSelected/;
	    s/\[SROUTESELECTED\]/$staticRouteSelected/;
	    s/\[USER_INFO\]/$info/;
            s/\[FONTH\]/$header_style/g;
            s/\[FONTT\]/$table_style/g;
            s/\[FONTB\]/$button_style/g;

	    s/\[SCRIPTURL\]/$networkScriptURL/;

	    s/\[SUCCESS\]/$success/; 

	    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	    print;
	}

	close TEMPLATE;
    }
}


# turn on DNS server if necessary
sub tryTurnOnDNS {
    my ( $ipAddr0, $ipAddr1, $dnsAddr1, $dnsAddr2 )=@_;

    if( ( $dnsAddr1 && ( $dnsAddr1 eq $ipAddr0 || $dnsAddr1 eq $ipAddr1 ) ) ||
       ( $dnsAddr2 && ( $dnsAddr2 eq $ipAddr0 || $dnsAddr2 eq $ipAddr1 ) ) ) {
	# no check for return because this is not mission critical
	Cobalt::Services::set_dns_server_on( 1 );
    }
}

sub disable_eth1 {
    my $oldip = $_[0];

    sleep 1 if (-e "$Lockdir/hosts");
    sleep 1 if (-e "$Lockdir/hosts");
    unless(-e "$Lockdir/hosts") {
    	open(NUHOSTS, "> $Lockdir/hosts");
    	open(OLDHOSTS, "/etc/hosts");
    	while(<OLDHOSTS>) {
		print NUHOSTS unless (/^$oldip\s/);
    	}
    	close(OLDHOSTS);
	close(NUHOSTS);
    	rename("$Lockdir/hosts","/etc/hosts") if (-s "$Lockdir/hosts"); # Full / test
    }
    unlink("$If_cfg_dir/ifcfg-$Netif1");
}

1;
