#!/usr/bin/perl

# author: John D. Blair
#       Cobalt Networks
# version: 1.0
# date: 20.jul.99
#
# Copyright (C) 1999, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::networker;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Services;
use Cobalt::Legato;

use Locale::gettext;

# global variables
my ($q, $locale, $page, $button_style, $header_style, $table_style );
my $networkerScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/networker';
my $networkerPageURL='/cgi-bin/.cobalt/turbo_ui/dispatch/services';

sub main {

$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
  $button_style=$locale->getProperty("buttonStyle");
  $header_style=$locale->getProperty("headerStyle");
  $table_style=$locale->getProperty("tableStyle");
 }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

$page=$q->param( 'page' );

if( $page ne 'networker' ) { genPage( \*STDOUT ); }
else {
    my $err=0;
    my $info='<IMG SRC="/.cobalt/images/bang" ALIGN=CENTER>';

    # read form input
    my $hostname=$q->param( 'hostname' );
    my $ipaddr=$q->param( 'ipaddr' );

    # let the sauce call do the error checking
    my $ret=Cobalt::Legato::legato_set_serveraddr( $hostname, $ipaddr );
    if( $ret!~/^2/o ) {
      $info.=substr( $ret, 4 ).'<BR>';
      $err=1;
    }

    if( !$err ) { print "location: $networkerPageURL\n\n"; }
    else { genPage( \*STDOUT, $info ); }
}
  undef ($q, $locale, $page );

}

#
# argument 0 is the file handle the page goes to
# argument 1 is the infomation shown to user
#
sub genPage {
    my ( $handle, $info )=@_;
    my $hostname;
    my $ipaddr;
    my ( $hostname, $ipaddr )=Cobalt::Legato::legato_get_serveraddr();

    # Read in the template
    open( TEMPLATE, "$Template_dir/networker.tmpl" )
          || die "Cannot read $Template_dir/networker.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    while( <TEMPLATE> ) {
        s/\[HOSTNAME\]/$hostname/;
        s/\[SCRIPTURL\]/$networkerScriptURL/o;
        s/\[SERVICESPAGEURL\]/$networkerPageURL/o;
        s/\[IPADDR\]/$ipaddr/;
        s/\[USER_INFO\]/$info/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

        s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

        print $handle $_;
    }
    close TEMPLATE;
}

1;
