#!/usr/bin/perl

# author: Kevin K.M. Chiu
#         Timothy Stonis
#       Cobalt Networks
# version: 2.0
# date: 7-2-99
#
# Copyright (C) 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::raidSetup;

use Cobalt::Product;
use Cobalt::RAID;
use Cobalt::Locale;
use Cobalt::Admserv;
use Cobalt::Network;
use Cobalt::User;

use Locale::gettext;

# global variables
my ($q,$page,$locale);
my $scriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/raidSetup';

sub main {

$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
# This is a nasty hack to see if we've gone through the setup wizard
if ( -e "/usr/admserv/cgi-bin/.cobalt/turbo_ui/.htaccess") {
  unless (user_authok($user, "admin")) {
      print "location: $forbiddenURL\n\n";
      die "Unauthorized access by $user, not an admin.\n";
  }
}

$page=$q->param( 'page' );
my $info=$q->param( 'info' );

if ( $page eq 'raidStatus' ) {
	genRaidStatusPage();
}
elsif ( $page eq 'raidSetup' ) {
  # Handle page
  my $raidchoice=$q->param( 'raidtype' );

  system( "echo raid: $raidchoice > /tmp/raid");

  if ( $raidchoice == 0 ) {
    # Nothing to do for RAID 0, that's how we ship
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n" .
      "<HTML><HEAD><META HTTP-EQUIV=\"expires\" CONTENT=\"0\"></HEAD>\n" . 
      "<BODY><SCRIPT LANGUAGE=\"javascript\">\n" .
      "top.code.flow_forceGoNext();\n" .
	    "</SCRIPT></BODY></HTML>";
    undef($charset, $charsetText);

  } else {
    # make sure things are cleaned up first
    system('rm -f /tmp/raid*');

    # Fork off our build script
    # now fork and format everything.
    my $pid;
    FORK: {
        if ($pid = fork) {
            genRaidStatusPage();
            exit 0;
        } elsif (defined $pid) {
            last;
        } elsif ($! =~ /No more process/) {
            sleep 5;
            redo FORK;
        } else {
            return "Can't fork: $!\n";
        }
    }
    # child process
    close (\*STDOUT);
    close (\*STDIN);
    Cobalt::RAID::raid_changelevels( $raidchoice );
    exit 0;
 
  } 
}
else {
   my $disks=Cobalt::RAID::raid_get_numdisk();
   genRaidPage( $disks ); 
}


}

#
# argument 0 is the number of disks 
#
sub genRaidPage {

    my $disks = shift;
 
    # Read in the template
    open( TEMPLATE, "$Template_dir/raidSetup.tmpl" )
        || die "Cannot read $Template_dir/raidSetup.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
	s/\[SCRIPTURL\]/$scriptURL/o;

        if( $disks == 2 ) {
            s/\[HIDERAID1START\]//o;
            s/\[HIDERAID1END\]//o;
            s/\[HIDERAID5START\]/<!-- /o;
            s/\[HIDERAID5END\]/ -->/o;
        }
        else {
            s/\[HIDERAID1START\]/<!-- /o;
            s/\[HIDERAID1END\]/ -->/o;
            s/\[HIDERAID5START\]//o;
            s/\[HIDERAID5END\]//o;
        }

        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	s/\[NUMDISK\]/$disks/o;

	print;
    }

    close TEMPLATE;

    exit 0;
}

#
# Produces the page of RAID conversion status
#
sub genRaidStatusPage {

    my $i, $err;

    # spin until raidstatus appears
    while (not -f '/tmp/raidstatus') {
	if ($i++ gt 4) {
	    $err = 1;
	    last;
	}
	sleep 1;
    }

    open( STATUS, "/tmp/raidstatus" ); 
    my $bstat;
    while (<STATUS>) {
      $bstat .= $_;
    }
    close(STATUS);

    if ( $bstat =~ /^DONE.*/ ) {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n" .        "<HTML><HEAD><META HTTP-EQUIV=\"expires\" CONTENT=\"0\"></HEAD>\n" . 
        "<BODY><SCRIPT LANGUAGE=\"javascript\">\n" .
        "top.code.flow_showNavigation ( true );\n" .
        "top.code.flow_goNext();\n" .
	      "</SCRIPT></BODY></HTML>";
    undef($charset, $charsetText);
    
      unlink("/tmp/raidstatus");
      return 0;
    }

    if ($err or ($bstat =~ /^ERROR:(.*)/)) {
      ##print "location: $raidsetupURL?error=$1\n\n";
      ##error msgs aren't currently used by $raidsetupURL
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n" .        "<HTML><HEAD><META HTTP-EQUIV=\"expires\" CONTENT=\"0\"></HEAD>\n" . 
        "<BODY><SCRIPT LANGUAGE=\"javascript\">\n" .
        "top.code.flow_showNavigation ( true );\n" .
        "top.code.flow_goBack();\n" .
	      "</SCRIPT></BODY></HTML>";
    undef($charset, $charsetText);

      unlink("/tmp/raidstatus");
      return 0;
    }

    # Read in the template
    open( TEMPLATE, "$Template_dir/raidBuild.tmpl" )
        || die "Cannot read $Template_dir/raidBuild.tmpl: $!\n";
    $scriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/raidSetup?page=raidStatus';

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
      	s/\[SCRIPTURL\]/$scriptURL/o;
	s/\[BUILD_STATE_MSG\]/$bstat/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;


	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	print;
    }

    close TEMPLATE;

}

1;
