#!/usr/bin/perl

# Failover Management - Cluster Reporting
# Will DeHaan - Cobalt Networks, Inc.
# Copyright 2000

package Cobalt::UI::raidUsage;

use Cobalt::Product;
use Cobalt::RAID; # Cobalt RAID monitoring serivces
use Cobalt::Locale;
use Locale::gettext;
use Cobalt::User;

my($Locale,$q);
my ($button_style, $header_style, $table_style);

sub main {

$q=shift;
$locale=Cobalt::Locale->new();
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); 
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

###########
# entre main
#

# Parse CGI post
my(%fields) = $q->Vars;
my($USER_INFO) = $fields{'USER_INFO'};

# Global
my $iam = '/cgi-bin/.cobalt/turbo_ui/dispatch/raidUsage';

# http(s) invocation preferred
# die "Web administrative interface use only\n\n" unless($ENV{'REMOTE_ADDR'});

# parse raid state
# State #       State Description                       UI Descriptor
#  0            Unconfigured/disabled/na                gray
#  1            Enabled, working great (UU)             green
#  2            Sync (RAID rebuild), non-fatal          yellow
#  3            Fatal error, single disk operation      red
#
# Integer RAID mode
#  null                 RAID unused (state 0)
#  0                    Stripe set
#  1                    Mirror (dual disk only)
#  3                    parity disk
#  5                    Stripe set w/striped parity
#
# @RAID_state array definitions
#  0 state code, 
#  1 integer RAID mode (/home only!),
#  2 number of disks used for /home, 
#  3 %completed for sync,
#  4 sync eta in minutes, 
#  5 failed device, 
#  6 number of live disks used for /home

my(@RAID_state) = raid_get_state();

# $RAID_state[0] ||= 3; # this CGI should never be invoked unless we're actively using RAID 

my($template_state, $complete_pixel_width, $remainder_pixel_width, $sync_complete, $sync_remainder, $template_states);

if ($RAID_state[0] == 2) # RAID Rebuild/resync
  {
  # Prep progress bar
  $sync_complete = $RAID_state[3]; # Sync in progress, numeric 0-100
  $sync_complete = 100 if ($sync_complete > 100); 
  $sync_complete = 0 unless ($sync_complete =~ /^[\.\d]+$/);
  $sync_remainder = 100 - $sync_complete;

  # Scale completion values from percent to double-proof
  $complete_pixel_width = $sync_complete * 4;
  $complete_pixel_width ||= 1; # some browsers can't render a 0 pixel image
  $remainder_pixel_width = $sync_remainder * 4;
  $remainder_pixel_width ||= 1;

  # ETA is rounded by RAID.pm
  $RAID_state[4] ||= '0.5';
  $RAID_state[4] = '0.5' if (($sync_complete == 100) && ($RAID_state[4] > 19));
  }

# Set refresh based on problem set
my($status_refresh) = 900; 
$status_refresh = 15 if ($RAID_state[0] == 2); 
$status_refresh = 45 if ($RAID_state[0] == 3);

$sync_complete = int($sync_complete + 0.5);

my $failed_disk_msg = Locale::gettext::dgettext('ui', "raid.failure.$RAID_state[5]");
if ( ! ($RAID_state[5] eq 'hda' || $RAID_state[5] eq 'hdc') ) {
	$failed_disk_msg .= " <P> " .  Locale::gettext::dgettext('ui', "raid.sled.replacement");
}

# Build the graphical disk table, nifty!
my %images;
$images{'unknown'} = '<img src="/.cobalt/images/driveDisabled.gif" alt="" width="64" height="86">';
$images{'0'} = '<img src="/.cobalt/images/blank.gif" alt="" width="64" height="86">';
$images{'1'} = '<img src="/.cobalt/images/driveOK.gif" alt="" width="64" height="86">';
$images{'2'} = '<img src="/.cobalt/images/driveSync.gif" alt="" width="64" height="86">';
$images{'3'} = '<img src="/.cobalt/images/driveToast.gif" alt="" width="64" height="86">';
my $disk_table = <<EOF;
<p><center>
<table border="0" width="266" cellspacing="0" cellpadding="0">
        <TR>
                <TD ROWSPAN="3" ALIGN="right" WIDTH="5"><img src="/.cobalt/images/boxLeft.gif" alt="" width="5" height="119"></TD>
                <TD COLSPAN="4" VALIGN="top"><img src="/.cobalt/images/spacer1.gif" alt="" width="256" height="5"></TD>
                <TD ROWSPAN="3" ALIGN="right" WIDTH="5"><img src="/.cobalt/images/boxRight.gif" alt="" width="5" height="119"></TD> 
        </TR>
        <tr ALIGN="center">
        		
                <TD WIDTH="64">[DISK0]</td>
                <TD WIDTH="64">[DISK1]</td>
                <TD WIDTH="64">[DISK2]</td>
                <TD WIDTH="64">[DISK3]</td>
        </tr>
        <tr>
                <td COLSPAN="4" VALIGN="bottom" ALIGN="left"><img src="/.cobalt/images/boxBottom.gif" alt="" width="256" height="12"></td>
        </tr>
</table>
<P><FONT COLOR="#FFFFFF">&nbsp;</FONT>

EOF
# Four graphic disks: hdc/hde hdd/hdg hda/hdi hdb/hdk
my $default_state = 1;
$default_state = 2 if ($RAID_state[0] == 2);
my ($disk0, $disk1, $disk2, $disk3);
$disk0 = $disk1 = $disk2 = $disk3 = $images{'0'};
$disk0 = $images{$default_state} if ($RAID_state[6] =~ /hd(c|e)/);
$disk0 = $images{'3'} if ($RAID_state[5] =~ /^hd(c|e)/);
$disk1 = $images{$default_state} if ($RAID_state[6] =~ /hd(d|g)/);
$disk1 = $images{'3'} if ($RAID_state[5] =~ /^hd(d|g)/);
$disk2 = $images{$default_state} if ($RAID_state[6] =~ /hd(a|i)/);
$disk2 = $images{'3'} if ($RAID_state[5] =~ /^hd(a|i)/);
$disk3 = $images{$default_state} if ($RAID_state[6] =~ /hd(b|k)/);
$disk3 = $images{'3'} if ($RAID_state[5] =~ /^hd(b|k)/);

print STDERR "RAID disks: $RAID_state[6]\n" if ($DEBUG);

open( TMPL, "$Template_dir/raidUsage.tmpl") ||
  die "Content-type: text/html\n\nMissing file: $Template_dir/raidUsage.tmpl\n\n"; 

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


while(<TMPL>)
  {
  $template_states = $1 if (/\<HA STATE (\d[\s\d]*)\>/);
  if(/\<HA STATE [^\>]+\>/ ... /\<\/HA STATE\>/)
    {
    next unless ($template_states =~ /$RAID_state[0]/);
    }

  next if (/HA STATE/); # Entire lines ignored!

  if(/\[/o)
    {
    s/\[RAID_MODE\]/$RAID_state[1]/;
    s/\[FAILED_DEV\]/$RAID_state[5]/;
		s/\[FAILED_DISK_MSG\]/$failed_disk_msg/;
    s/\[str\.([^\]]+)\]/Locale::gettext::dgettext('ui', "$1")/ge;

    s/\[DEFCON(\d)\]/$defcon_images{ $1 }/o;

    s/\[IMGCOMPLETE\]/$complete_pixel_width/g;
    s/\[REMAINDER\]/$remainder_pixel_width/g;

    s/\[DRIVETABLE\]/$disk_table/g;
    s/\[DISK0\]/$disk0/g;
    s/\[DISK1\]/$disk1/g;
    s/\[DISK2\]/$disk2/g;
    s/\[DISK3\]/$disk3/g;

    s/\[NUM_DISKS\]/$RAID_state[2]/;
    s/\[RAID_MODE\]/$RAID_state[1]/;
    s/\[REFRESH\]/$status_refresh/;
    s/\[IAM\]/$iam/;

    s/\[USER_INFO\]//o; # error message
      
    s/\[SYNC_COMPLETE\]/$sync_complete/;
    s/\[RAID_STATE\]/$RAID_state[2]/;
    s/\[TIME_EST\]/$RAID_state[4]/;
    s/\[FONTH\]/$header_style/g;
    s/\[FONTT\]/$table_style/g;
    s/\[FONTB\]/$button_style/g;
    }
              
  print;
  }
close( TMPL );

}

#
# Fin main
###########

1;
