#!/usr/bin/perl

# author: Kevin K.M. Chiu
#         Timothy Stonis
#       Cobalt Networks
# version: 2.0
# date: 7-2-99
#
# Copyright (C) 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::register;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Admserv;
use Cobalt::Network;
use Cobalt::User;
use Cobalt::Http;

use Locale::gettext;

# global variables
my ($q,$page,$locale,$button_style,$header_style,$table_style);
my $scriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/register';

sub main {


$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
# This is a nasty hack to see if we've gone through the setup wizard
if ( -e "/usr/admserv/cgi-bin/.cobalt/turbo_ui/.htaccess") {
  unless (user_authok($user, "admin")) {
      print "location: $forbiddenURL\n\n";
      die "Unauthorized access by $user, not an admin.\n";
  }
}

$page=$q->param( 'page' );

if( $page ne 'registermain' ) {

   # First check to see if we registered already
   my $reg = (-e "/var/lib/cobalt/.registered") ? 1 : 0;
   if( $reg ) {
     genGeneralRegPage( "[str.alreadyRegistered]" ); 
   } else {

      my( $dns1, $dns2 )=( Cobalt::Network::net_get_dnsclient() )[ 1, 2 ];
      my $gateway=( Cobalt::Network::net_get_gateway() )[ 0 ];

      if( ( !$dns1 && !$dns2 ) || !$gateway ) {
  	# cannot register without DNS or gateway to send email
        genGeneralRegPage( "[str.noEmailRegister]" );	
      }
      else { genRegisterPage(); }
    }
}
else {
    # get user information

    #
    # some get used while some don't depending on locale
    #
    my $fullName=$q->param( 'fullname' );
    my $title=$q->param( 'title' );
    my $company=$q->param( 'company' );
    my $address=$q->param( 'address' );
    my $country=$q->param( 'country' );
    my $phone=$q->param( 'phone' );
    my $fax=$q->param( 'fax' );
    my $email=$q->param( 'email' );

    # if required fields aren't filled in, they selected register later
   if( !$fullName || (!$fax && !$email) ) {  
      genGeneralRegPage( "[str.howToRegLater]" );	
    }
    else {
	# get system information
	my $hostName=Cobalt::Network::net_get_hostname();
	my ( $domain, $dns1, $dns2 )=Cobalt::Network::net_get_dnsclient();
	my ( $ipAddr, $netMask, $mac )=Cobalt::Network::net_get_ipaddr();

	open( BUILD, '/etc/build' );
	my $build=join "\t", <BUILD>;
	close( BUILD );

	my $buildlabel = 'RaQ 4r';
	$buildlabel = 'RaQ 5i' if ($build =~ /3500R/);
	$buildlabel = 'RaQ 5' if ($build =~ /3500R/);
	$buildlabel = 'RaQ 4i' if ($build =~ /3001R/);
	$buildlabel = 'RaQ 3' if ($build =~ /3000R/);
	$buildlabel = 'Qube 3' if ($build =~ /3000WG/);

	if( open( MAIL, "|$Sendmail -oi -t" ) ) {
	    print MAIL <<END;
From: $fullName <admin\@$hostName.$domain>, <$email>
To: register\@cobalt.com
Subject: Registration ($buildlabel)

Full name: $fullName
Title: $title
Company: $company
Address: $address
Country: $country
Phone: $phone
Fax: $fax
Email: $email

IP address: $ipAddr
Netmask: $netMask
Host name: $hostName
Domain: $domain
DNS 1: $dns1
DNS 2: $dns2
MAC address: $mac

Build: $build
END
        close( MAIL );
	}

        open( REGFILE, ">/var/lib/cobalt/.registered" );
        close ( REGFILE ); 
        genGeneralRegPage( "[str.successReg]" );	
    }
}

}

#
# argument 0 is the infomation shown to user
#
sub genRegisterPage {

    my $registerButton;
    if ( 'wizard' ne $page ) {
       $registerButton = <<END;
 <TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0"><TR>
  <TD><A HREF="javascript: if( trySubmit() ) document.forms[ 0 ].submit();" onMouseOver="status='[str.registerThroughEmail]'; return true;"><IMG SRC="/libImage/buttonLeftDot.gif" BORDER=0></A></TD>
  <TD NOWRAP STYLE="[FONTB]" ><A HREF="javascript: if( trySubmit() ) document.forms[ 0 ].submit();" onMouseOver="status='[str.registerThroughEmail]'; return true;"><FONT STYLE="[FONTB]">[str.registerThroughEmail]</FONT></A></TD>
  <TD><A HREF="javascript: if( trySubmit() ) document.forms[ 0 ].submit();" onMouseOver="status='[str.registerThroughEmail]'; return true;"><IMG SRC="/libImage/buttonRight.gif" BORDER=0></A></TD>

</TR></TABLE>

END
    }


    # Read in the template
    open( TEMPLATE, "$Template_dir/register.tmpl" )
        || die "Cannot read $Template_dir/register.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
	s/\[SCRIPTURL\]/$scriptURL/o;
	s/\[REGISTERBUTTON\]/$registerButton/;
        s/\[PAGE\]/$page/;
	s/\[USER_INFO\]/$_[ 0 ]/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;


	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print;
    }

    close TEMPLATE;
    exit 0;
}

#
# Argument 0 is the text you want to put in the dialog
#
sub genGeneralRegPage {
    # Read in the template
    open( TEMPLATE, "$Template_dir/genregister.tmpl" )
        || die "Cannot read $Template_dir/register.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
        s/\[REGMESSAGE\]/$_[ 0 ]/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

        s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

        print;
    }

    close TEMPLATE;

   # I'm going to be real nasty here... I don't want the user to have
   # to authenticate right after completing the first page of the setup
   # wizard, so I'll throw a .htaccess file in here after I present
   # the page.
   admserv_set_access( "$AdminPages_dir/register/", 'admin' );
   admserv_set_access( "$AdminCGIs_dir/turbo_ui/", 'valid' );
   admserv_set_access( "$AdminCGIs_dir/message/", 'valid' );


    my $pid;
    FORK: {
        if ($pid = fork) {
            exit 0;
        } elsif (defined $pid) {
            last;
        } elsif ($! =~ /No more process/) {
            sleep 5;
            redo FORK;
        } else {
            return "Can't fork: $!\n";
        }
    }
    
   # create symlinks for /tmp
   system("/etc/rc.d/init.d/tmpinit");
   # now we need to restart everything that held state in /tmp
   # postgres, httpd (hard restart), and (after a slight wait) admserv
   system("/etc/rc.d/init.d/postgresql restart");
   Cobalt::Http::http_restart(1,1);

   sleep 2;
   Cobalt::Admserv::admserv_restart();

}

1;
