#!/usr/bin/perl

# Restore menu V3
# Copyright 1998 Cobalt Networks Inc

package Cobalt::UI::restoremenu;

use Cobalt::Product;
require Cobalt::Vsite;
require Cobalt::Language;
use strict;
use Cobalt::User;
use Cobalt::UI qw( output_page get_style_info );
use Cobalt::Util qw( read_status untar_name );

###############
# configure environment
sub main {

	my ($user, $q);
	
	# fastcgi serves up CGI.pm fresh; daily
	$q = shift;
	$user = $ENV{"REMOTE_USER"};
	
	# check if we are site or personal restore
	my $site = $q->param('site');
	my $type = ($q->param('site_restore') ? 'site' : 'personal');

	my $forbiddenURL="/.cobalt/error/forbidden.html";
	if( $type eq 'site' && not user_authok($user, "site-adm", $site)) {
    		print "location: $forbiddenURL\n\n";
    		die "Unauthorized access by $user, not an admin.\n";
	}

	my ($user_id, $user_dir) = (getpwnam($user))[2,7] if (($user =~ /^[a-z0-9][a-z0-9\.\-\_]+$/) && (length($user) < 32));

	# figure out what we should display
	my @status = ($$, 'MENU');
	my $flag = "$user_dir/.restore.$type.running";
	if (-f $flag) {
		@status = read_status($flag);
		# make sure the process is still alive and well
		if(stale($status[0])) {
	
			# process may have died before finishing
			@status = read_status($flag);

			if ($status[1] !~ /^(\[[^\]]+\]|SELECT|DONE)$/) {
				$status[1] = '[str.system.error]';
			}
		}
	
		if ($q->param('done') || $status[1] =~ /^\[[^\]]+\]$/) {
			# clean up all the files we use
			unlink $flag;
			unlink glob("$flag*");
		}
	}

	if ($q->param('done')) {
		$status[1] = 'MENU';
	}

	if ($status[1] =~ /^(MENU|\[[^\]]+\])$/) {
		&display_menu($q, $user_dir, 
				($status[1] =~ /^\[[^\]]+\]$/ ? $status[1] : ''), $type);
	} elsif ($status[1] eq 'SELECT') {
		&display_select_or_results($q, $user_dir, 1, \@status);
	} elsif ($status[1] eq 'DONE') {
		&display_select_or_results($q, $user_dir, 0, \@status);
	} else {
		&display_status($q, @status);
	}

} # fin main()

sub display_menu
{
	my $cgi = shift;
	my $user_dir = shift;
	my $user_error = shift;
	my $type = shift;

	my $user = $ENV{REMOTE_USER};
	my $site = ($cgi->param('site') ? $cgi->param('site') : Cobalt::Vsite::vsite_getgroup($user));
	my $is_site = $cgi->param('site_restore');
	my ($target_dir, $subs); 

	# Don't restrict restores to home directory if we're a site admin
	if(Cobalt::User::user_authok($user, 'site-adm')) {
    		$user_dir =~ s#/users/$user$##;
    		$target_dir = "users/$user/";
    		$target_dir = '' if ($is_site);
	} 

	if(($user eq $Adminuser) || ($user eq $Alteradmin)) {
    		$user_dir = '';
    		$site ||= $MainGroup;
    		$target_dir = "home/sites/$site/";
    		$target_dir = "home/sites/$site/users/$Adminuser/" unless ($is_site);
	}

	# Select site restore vs. personal profile restore msgs
	my $table_title = '[str.restore_title]'; 
	$table_title = '[str.backup.site.restore_title]' if ($is_site);

	# set up parameters for substitution into template
	$subs->{USER} = $user;
	$subs->{SITE} = $site;
	$subs->{IS_SITE} = $is_site;
	$subs->{BACKUPS} = &get_homedir_archives($user);
	$subs->{USERERROR} = $user_error;
	$subs->{GROUP} = $site;
	$subs->{USERDIR} = $user_dir . '/';
	$subs->{TARGETDIR} = $target_dir;
	$subs->{TITLE} = $table_title;

	if ($is_site) {
		$subs->{DIRECTHELP} = '[str.restore_help_site_directed]';
	} else {
		$subs->{DIRECTHELP} = '[str.restore_help_directed]';
	}

	# output form
	output_page("$Template_dir/restoremenu.tmpl", $subs);
}

sub get_homedir_archives
{
	my $homedir = (getpwnam($_[0]))[7];

	my $homedirheader = "\n\t<tr>\n\t\t<td width=\"26\">\n\t\t\t<center><a href=\"javascript: top.showInfo( homedirHelp );\" onMouseOver=\"status=\'Help\'; top.showInfo( homedirHelp ); return true;\"><img src=\"/.cobalt/images/smallest_help_but\" align=\"bottom\" border=\"0\"></a></center>\n\t\t</td>\n\t\t<td><font bgcolor=#FFFFFF>&nbsp;</font><font style=\"[FONTT]\">[str.backup.select.archive]</font>\n\t\t</td>\n\t\t<td>";

	my $backups = $homedirheader."<FONT STYLE=\"[FONTT]\"><select name=\"homedirarchive\"><option value=\"\">[str.backup.defaultselection]</option>\n";

	if (-d $homedir) {
  		opendir(HOME,"$homedir");
  		my @files = readdir(HOME);
  		for my $file (@files) {
    			if ($file =~ /\.tgz$/) {
      				my $size_unit;
	      			my $file_size = (stat("$homedir/$file"))[7];
      				if (length($file_size) > 6) {
        				$file_size = int($file_size/1048576);
        				$size_unit = '[str.backup.megabytes]';
	      			} elsif (length($file_size) > 3) {
        				$file_size = int($file_size/1024);
        				$size_unit = '[str.backup.kilobytes]';
	      			} else {
        				$size_unit = '[str.backup.bytes]';
	      			}
      			
				$backups .= "<option value=\"$file\">$file ($file_size $size_unit)</option>\n" if ($file =~ /\.tgz$/);
    			}
  		}
  		closedir(HOME);
	}
    
	return ($backups . "</select></FONT></td></tr>"); 

}

sub display_select_or_results
{
	
	my $cgi = shift;
	my $user_dir = shift;
	my $select = shift;
	my $status = shift;
	my $archive = $$status[2];

	my $subs;
	my $site = ($cgi->param('site') ? $cgi->param('site') : Cobalt::Vsite::vsite_getgroup($ENV{REMOTE_USER}));
	my $type = ($cgi->param('site_restore') ? 'site' : 'personal');
	my $directed = ($cgi->param('directed') ? $cgi->param('directed') : $$status[3]);


	# choose template based on whether we are displaying as select menu
	# or results
	my $template = ($select ? "$Template_dir/selective_restore.tmpl" : "$Template_dir/restore.tmpl");
	
	# Rather RAM intensive
	my ($file_table, $file_instance);
	my $selection_index = 0;
	my $icon_folder = "/cobalt-images/icon_folder";
	my $icon_generic = "/cobalt-images/icon_generic";
	my $icon_mail = "/cobalt-images/icon_text";
	my $restored_bytes = 0;
	my $size_unit;
	my $stage = 0;
	my %map = ();

	if ($select) {
		open LIST, "$user_dir/.restore.$type.running.index" or die "$!\n";
		open MAP, "$user_dir/.restore.$type.running.map" or die "$!\n";

		for my $line (<MAP>) {
			chomp $line;

			my @parts = split /\t/, $line;

			$map{$parts[1]} = $parts[0];
		}
		close MAP;
	} else {
		open LIST, "$user_dir/.restore.$type.running.info" or die "$!\n";
	}

	$directed = $directed . '/' if ($directed =~ /[^\/]$/);
	
	$subs->{FILECOUNT} = 0;

	my (%file_sizes, %file_dates, %dir_names, %file_errors);

	while(<LIST>) {	
		
		chomp;
		# skip blank lines and lines for directories we tried restoring
		next if (!$_ || /^\s*directory:\s*$/);
		
		my @info = split /\t/;
		$file_instance = $info[0];

		next if ($info[0] =~ /^\.+\/$/);

		if ($info[0] =~ /.\/$/) {
			$dir_names{$info[0]} = $info[0];
		}

		$file_sizes{$info[0]} = $info[1];
	
		if ($info[2]) { $file_dates{$info[0]} = $info[2]; }
	
		$restored_bytes += $file_sizes{$file_instance};
	
		# hack off leading ./ from large archive fix
		my $foo_name = $file_instance;
		$foo_name =~ s/^\.+\///;

		# take out extra slashes that tar adds for display
		$foo_name =~ s/\\\\/\\/g;

		my @locale_info = get_style_info();	
		if ($locale_info[1]->getLocale() =~ /^ja/) {
			$foo_name = untar_name($foo_name, $locale_info[1]->getProperty("ui_encoding"));
		}

		# next unless ($foo_name !~ /^\s*$/);
	
		if ($subs->{FILECOUNT} < $MaxFileList) {
	
			my $file_image = $icon_generic;
			my ($tree_select, $tree_select1);
	
			if ($foo_name =~ /^email_spool/) {
				$file_image = $icon_mail;
			} elsif ($foo_name =~ /.\/$/) {
				$file_image = $icon_folder;
				$tree_select = "<A HREF=\"javascript: selectTree( 'FILE$selection_index' );\" onMouseOver=\"status='[str.restore_sel_tree]'; return true;\">";
				$tree_select1 = '</A>';
			}
			
			if (length($file_sizes{$file_instance}) > 6) {
				$file_sizes{$file_instance} = int($file_sizes{$file_instance}/1048576);
				$size_unit = '[str.backup.megabytes]';
			} elsif (length($file_sizes{$file_instance}) > 3) {
				$file_sizes{$file_instance} = int($file_sizes{$file_instance}/1024);
				$size_unit = '[str.backup.kilobytes]';
			} else {
				$size_unit = '[str.backup.bytes]';
			}
	 
			if ($select) {
				my $tree_select_box;
	
				if ($foo_name =~ /.\/$/) {
					$tree_select_box = "onClick=\"if (this.checked) this.checked = false; else this.checked = true; selectTree( \'FILE$selection_index\' ); }\"";
				}
				$file_table .= "
	<TR><TD ALIGN=CENTER><INPUT TYPE=CHECKBOX NAME=\"FILE$selection_index\" VALUE=\"$map{$file_instance}\" $tree_select_box></TD>";
				$file_table .= "<TD><FONT STYLE=\"[FONTT]\">&nbsp;$tree_select<IMG SRC=\"$file_image\" BORDER=0>$tree_select1&nbsp;".$directed.$foo_name;
				if($file_instance =~ /\/$/) {
					$file_table .= '</FONT></TD>';
				} else {
					$file_table .= ' <FONT SIZE="-1">('.$file_sizes{$file_instance}." $size_unit)</FONT></FONT></TD>";
				}
				$file_table .= "<TD><FONT STYLE=\"[FONTT]\">&nbsp;".$file_dates{$file_instance}."</FONT></TD></TR>\n";
				$selection_index++;
			} else {
				# Fudge folder reporting--We can't pass folders to tar for restore
				my $file_base = $file_instance;
				$file_base =~ s#\/[^\/]+$##;
	
				if($dir_names{$file_base}) {
					# hack off ./ from large archive fix
					$dir_names{$file_base} =~ s/^\.*\///;
					
					my $display_dir;
					if ($locale_info[1]->getLocale() =~ /^ja/) {
						$display_dir = untar_name($dir_names{$file_base}, $locale_info[1]->getProperty("ui_encoding"));
					} else {
						$display_dir = $dir_names{$file_base};
					}
					$file_table .= "<TR><TD><FONT STYLE=\"[FONTT]\">&nbsp;<IMG SRC=\"".$icon_folder."\" BORDER=0>&nbsp;".$directed.$display_dir.'</FONT></TD>';
					$file_table .= "<TD><FONT STYLE=\"[FONTT]\">&nbsp;[str.backup.ok]</FONT></TD></TR>\n";
					undef($dir_names{$file_base});
				} else {
		
					# File handling
					$file_errors{$file_instance} ||= '[str.backup.ok]';
					$file_table .= "<TR><TD><FONT STYLE=\"[FONTT]\">&nbsp;<IMG SRC=\"".$file_image."\" BORDER=0>&nbsp;".$directed.$foo_name.'</FONT></TD>';
					$file_table .= "<TD><FONT STYLE=\"[FONTT]\">&nbsp;".$file_errors{$file_instance}."</FONT></TD></TR>\n";
				}
			}
		}
	
		$subs->{FILECOUNT}++;
	}
	
	close LIST;
	
	$subs->{FILETABLE} = $file_table;

	# fill in some fields for template
	my $restored_mb = int($restored_bytes/1048576);
	$subs->{FILEMB} = $restored_mb;
	$subs->{ARCHIVE} = $archive;
	$subs->{UPLOAD} = $cgi->param('upload');
	$subs->{SITE} = $site;
	$subs->{IS_SITE} = $cgi->param('site_restore');
	$subs->{DIRECTED} = $directed;
	$subs->{CANCELURL} = "/cgi-bin/.cobalt/turbo_ui/dispatch/restoremenu?site_restore=" . $cgi->param('site_restore') . "&site=$site&done=1";

	# change the filename help based on whether it is personal or site or
	# admin doing a restore
	if ($ENV{REMOTE_USER} eq $Adminuser || $ENV{REMOTE_USER} eq $Alteradmin) {
		$subs->{FILE_HELP} = '[str.restore.filename.admin.help]';
	} elsif ($type eq 'site' 
		|| Cobalt::User::user_authok($ENV{REMOTE_USER}, 'site-adm', $site)) {
		$subs->{FILE_HELP} = '[str.restore.filename.site.help]';
	} else {
		$subs->{FILE_HELP} = '[str.restore.filename.help]';
	}

	# Replace our file table with summary stats if we have too many files to render a complete listing
	if ($subs->{FILECOUNT} > $MaxFileList) {
		# Clobber the original file table
		$subs->{FILETABLE} = <<EOF;
	<TR><TD>
	    <FONT STYLE=\"[FONTT]\">[str.backup.large.filecount.restored]</FONT>
	    </TD>
	    <TD>
	    <FONT STYLE=\"[FONTT]\">[str.backup.ok]</FONT>
	    </TD>
	</TR>
EOF
	}

	if ($subs->{FILECOUNT} == 0) {
		# Build null selective restore menu
		$subs->{FILETABLE} = <<EOF;
	<TR><TD COLSPAN="3">
	    <FONT STYLE=\"[FONTT]\">[str.backup.no.files]</FONT>
	    </TD>
	</TR>
EOF
		$subs->{USERERROR} = "'[str.backup.no.files.found.in.archive]', 'error'";
		&output_page($template, $subs);
		return;
	}

	if (not $select) {
		$subs->{USERERROR} = "'[str.backup.restore.stage.done]', 'status'";
	} else {
		$subs->{USERERROR} = "'[str.backup.restore.selective]', 'status'"
	}

	output_page($template, $subs);
}

sub display_status
{
	my ($cgi, @status) = @_;
	my ($progress, $subs);
	my $site = ($cgi->param('site') ? $cgi->param('site') : Cobalt::Vsite::vsite_getgroup($ENV{REMOTE_USER}));

	my $style = get_style_info();

	if($status[1] =~ /^(INDEX|UPLOAD|RESTORE)$/) {
		$progress = int($status[2]/$status[3] * 100);
	} else {
		$progress = 0;
	}
	
	# spoof progress for > 100 since we can't get real size
	# for some tgz files, but this should be fairly close
	$progress = ($progress < 100) ? $progress : 99;

	my $remaining = 100 - $progress;

	$subs->{PROGRESS_BAR} = '<IMG SRC="/libImage/bar0.jpg" HEIGHT=20 BORDER=0>';
	if ($progress > 0) {
		$subs->{PROGRESS_BAR} .= "<IMG SRC=\"/libImage/bar1.jpg\" WIDTH=$progress HEIGHT=20 BORDER=0>";
	}

	if ($remaining > 0) {
		$subs->{PROGRESS_BAR} .= "<IMG SRC=\"/libImage/bar2.jpg\" WIDTH=$remaining HEIGHT=20 BORDER=0>";
	}

	$subs->{PROGRESS_BAR} .= "<IMG SRC=\"/libImage/bar3.jpg\" HEIGHT=20 BORDER=0><FONT STYLE=\"$style->{table}\">&nbsp;$progress%</FONT>";

	$subs->{REFRESH} = "/cgi-bin/.cobalt/turbo_ui/dispatch/restoremenu?site_restore="
			. $cgi->param('site_restore') . '&site=' . 
			$site . '&directed=' . $cgi->param('directed')
			. '&upload=' . $cgi->param('upload'); 

	# check for errors
	if ($status[1] =~ /^\[[^]]+\]$/) {
		$subs->{STATUS} = '[str.restore.error]';
		$subs->{PROGRESS_BAR} = $status[1];
	} else {
		$subs->{STATUS} = '[str.backup.restore.stage.' . lc($status[1]) . ']';
	}

	$subs->{SITE} = $site;
	$subs->{IS_SITE} = $cgi->param('site_restore');
	$subs->{DIRECTED} = $cgi->param('directed');

	output_page("$Template_dir/restore_status.tmpl", $subs);
}

sub stale
{
	my $pid = shift;

	my $user = $ENV{REMOTE_USER};

	# normally the restore will run as the user who invoked it
	# unless the user is admin or alteradmin
	if ($user eq $Adminuser || $user eq $Alteradmin) {
		$user = 'root';
	}

	my @ps = split /\n/, `/bin/ps -w -o 'pid user' --no-headers -C restore.cgi 2>/dev/null`;

	chomp @ps;
	my ($line) = grep /^\s*$pid/, @ps;
	my ($ps_pid, $ps_user) = split ' ', $line;

	if($user =~ /^$ps_user/ && $ps_pid == $pid) {
		return 0;
	} else {
		return 1;
	}
}

