#!/usr/bin/perl
#
# Copyright 1998-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteAdd;

use Cobalt::Product;
use Cobalt::Meta::vsite;
use Cobalt::DiskUtils;
use Cobalt::DNS;
use Cobalt::Util;
use Cobalt::User;
use Cobalt::Locale;
use Cobalt::Email;
use Locale::gettext;

my $CK           = qq[CHECKED];
my $siteAdd_TMPL = qq[$Template_dir/siteAdd.tmpl];
my $siteAdd_URL  = qq[/cgi-bin/.cobalt/turbo_ui/dispatch/siteAdd];
my $siteList_URL = qq[/cgi-bin/.cobalt/turbo_ui/dispatch/siteList];

my ($locale, $site, $query, $Q, @emailAlias, @webAlias, $header_style,
    $table_style, $button_style );

sub main {

@emailAlias = ();
@webAlias = ();

$locale = Cobalt::Locale->getByHTTPHeader($ENV{'HTTP_ACCEPT_LANGUAGE'});
$button_style=$locale->getProperty("buttonStyle");
$header_style=$locale->getProperty("headerStyle");
$table_style=$locale->getProperty("tableStyle"); 

$site   = new Cobalt::Meta::vsite;
$query  = shift;

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

$query->import_names('Q');
if ($locale) { $ENV{'LANGUAGE'}=$locale->getLocale }

##
## handle siteadd submit()
if ($Q::page eq 'siteadd') {
    # clean the www and email alias lists
    $Q::webserveralias =~ s/[\s\,\r]+/ /g;
    $Q::emailserveralias =~ s/[\s\,\r]+/ /g;

    # strip zero padding from IP address
    $Q::ipaddr =~ s/\.0+(\d)/\.$1/g;
    $Q::ipaddr =~ s/^0//g;

    # read form input into Meta::vsite object
    $site->hostname ($Q::hostname);
    $site->domain   ($Q::domain);
    $site->ipaddr   ($Q::ipaddr);
    $site->quota    ($Q::diskquota);
    $site->maxusers ($Q::maxusers);
    $site->ftp      ($Q::ftp);
    $site->ftpquota ($Q::ftpquota);
    $site->ftpusers ($Q::ftpusers);
    $site->shell    ($Q::shell);
    $site->apop     ($Q::apop);
    $site->dns_auto ($Q::dnsauto);
    $site->cgi      ($Q::cgi);
    $site->ssi      ($Q::ssi);
    $site->ssl      ($Q::ssl);
    $site->fpx      ($Q::fpx);
    $site->casp     ($Q::casp);
    $site->php      ($Q::php);
    $site->suspend  ($Q::suspend);
    $site->volume   ($Q::volume);

    @webAlias   = split /[\s,]+/, $Q::webserveralias;
    @emailAlias = split /[\s,]+/, $Q::emailserveralias;

    if (scalar(@webAlias))   { $site->webServerAlias(@webAlias)     }
    if (scalar(@emailAlias)) { $site->emailServerAlias(@emailAlias) }
    if ($site->fpx)          { $site->fpxPassword($Q::fpxpass0)     }

    my ($ret,$info) = $site->add();

    # auto-config DNS
    dns_import($Q::hostname, $Q::domain, $Q::ipaddr, $Q::webserveralias, $Q::emailserveralias) if ($Q::dnsauto eq 'on');

    if (!$ret) {
	&genSiteAddPage($locale, $query, $site, @$info );
    }
    elsif (scalar(@$info)) {
	print "Location: $siteList_URL\?info=", join('<BR>', @$info), "\n\n";
    }
    else {
	print "Location: $siteList_URL\n\n";
    }

  undef ($locale, $site, $query, $Q, @emailAlias, @webAlias );

}
else {

    $site->retrieve('default');
    &genSiteAddPage( $locale, $query, $site );

}

}

sub genSiteAddPage
{
    my ($locale, $query, $site, @info) = @_;

    FILE_lock_r($siteAdd_TMPL, *TEMPLATE)
	or die("Cannot read $siteAdd_TMPL : $!\n");

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    while (<TEMPLATE>) {

	s|\[USER_INFO\]| (scalar(@info)) ? join('<BR>', @info) : q[] |e;
	s|\[SITELISTSCRIPTURL\]|$siteList_URL|o;
	s|\[SITEADDSCRIPTURL\]|$siteAdd_URL|o;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s|\[SHELLCHECKED\]| ($site->shell) ? q[CHECKED] : q[] |e;
	s|\[CASPCHECKED\]|   ($site->casp) ? q[CHECKED] : q[] |e;
	s|\[PHPCHECKED\]|     ($site->php) ? q[CHECKED] : q[] |e;
	s|\[APOPCHECKED\]|   ($site->apop) ? q[CHECKED] : q[] |e;
	s|\[DNSAUTO\]|   ($site->dns_auto) ? q[CHECKED] : q[] |e;
	s|\[FPXCHECKED\]|     ($site->fpx) ? q[CHECKED] : q[] |e;
	s|\[CGICHECKED\]|     ($site->cgi) ? q[CHECKED] : q[] |e;
        s|\[SSICHECKED\]|     ($site->ssi) ? q[CHECKED] : q[] |e;
	s|\[SSLCHECKED\]|     ($site->ssl) ? q[CHECKED] : q[] |e;
	s|\[FTPCHECKED\]|     ($site->ftp) ? q[CHECKED] : q[] |e;

	s|\[WEBSERVERALIAS\]|           join(", ", @webAlias) |e;
	s|\[EMAILSERVERALIAS\]|       join(", ", @emailAlias) |e;
	s|\[HOSTNAME\]|                $site->hostname or q[] |e;
	s|\[DOMAIN\]|                  $site->domain   or q[] |e;
	s|\[IPADDR\]|                  $site->ipaddr   or q[] |e;
	s|\[DEFAULTDISKQUOTA\]|        $site->quota    or q[] |e;
	s|\[MAXFTPUSERS\]|             $site->ftpusers or q[] |e;
	s|\[MAXUSERS\]|                $site->maxusers or q[] |e;
	s|\[FTPQUOTA\]|                $site->ftpquota or q[] |e;

	s|\[VOLLISTCODE\]|             &makeVolList()         |e;
	s|\[str\.([^\]]+)\]|           &MSG_get_ui($1)        |eg;

	print;

    }
    FILE_unlock(*TEMPLATE);

    return 1;
}


#
# make a list of all the currently available disks.
#
sub makeVolList
{
    my %mounts = Cobalt::DiskUtils::disk_getmounts($Vol_sitepattern);
    my (%maxfree, %list);
    my ($key, $text);

    foreach $key (keys %mounts) {
	# special case $Vol_home 
	if ($mounts{$key}{MountPoint} =~ /^\/$Vol_home/) {
	    $list{$Vol_home} = $mounts{$key}{Free};
	    if ($list{$Vol_home} > $maxfree{Value}) {
		$maxfree{Disk} = $Vol_home;
		$maxfree{Value} = $list{$Vol_home};
	    }
	    next;
	} 

	# everything else 
	$mounts{$key}{MountPoint} =~ /^$Vol_rootdir\/(\S+)$/o;
	next unless (-e "$mounts{$key}{MountPoint}/.cobaltsig"); 
	$list{$1} = $mounts{$key}{"Free"};
	if ($list{$1} > $maxfree{Value}) {
	    $maxfree{Disk} = $1;
	    $maxfree{Value} = $list{$1};
	}
    }

    $text = "<SELECT NAME=\"volume\">\n";
    foreach $key (sort keys %list) {
	$text .= "<OPTION VALUE=\"$key\" ";
	$text .= "SELECTED" if ($maxfree{Disk} eq $key);
	$text .= ">$key (" . int($list{$key} / 1024);
	$text .= " " . MSG_get_ui("volMBFree");
	$text .= ")</OPTION>\n";
    }
    $text .= "</SELECT>\n";

    return $text;
}

1;
