#!/usr/bin/perl
#
# Copyright 1998-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteDefaults;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Util;
use Cobalt::Meta::vsite;
use Cobalt::User;

use DB_File;
use Fcntl qw(O_RDWR O_CREAT);
use CGI;
use Locale::gettext;

# global variables
my ($q,$locale,$page,$button_style,$header_style,$table_style);
my $siteDefaultsScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/siteDefaults';
my $siteListScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/siteList';

sub main {

$q=shift;
$page=$q->param( 'page' );
$locale=Cobalt::Locale->getByHTTPHeader($ENV{HTTP_ACCEPT_LANGUAGE});
if ($locale) { $ENV{LANGUAGE}=$locale->getLocale;
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

# check for error condition
#
if ($q->param('error') eq 'yes') {

    my $errobj = new Cobalt::Meta::vsite("name"     => "default",
					 "hostname" => "",
					 "domain"   => $q->param('domain'),
					 "ipaddr"   => $q->param('ipaddr'),
					 "fpx"      => $q->param('fpx'),
					 "casp"     => $q->param('casp'),
					 "php"      => $q->param('php'),
					 "maxusers" => $q->param('maxusers'),
					 "quota"    => $q->param('quota'),
					 "shell"    => $q->param('shell'),
					 "cgi"      => $q->param('cgi'),
					 "ssl"      => $q->param('ssl'),
					 "ssi"      => $q->param('ssi'),
					 "apop"     => $q->param('apop'),
					 "dns_auto" => $q->param('dnsauto'),
					 "ftp"      => $q->param('ftp'),
					 "ftpusers" => $q->param('ftpusers'),
					 "ftpquota" => $q->param('ftpquota'));

    genSiteDefaultsPage($errobj, $q->param('info'));

} elsif( $page ne 'sitedefaults' ) {

    my $defobj = new Cobalt::Meta::vsite;
    $defobj->retrieve("default");
    genSiteDefaultsPage($defobj);

} else {

    my $cgi=$q->param('cgi');
    my $shell=$q->param('shell');
    my $ssi=$q->param('ssi');
    my $ssl=$q->param('ssl');
    my $ipaddr=$q->param('ipaddr');
    my $fpx=$q->param('fpx');
    my $casp=$q->param('casp');
    my $php=$q->param('php');
    my $domain=$q->param('domain');
    my $maxusers=$q->param('maxusers');
    my $diskQuota=$q->param('diskquota');
    my $ftp=$q->param('ftp');
    my $apop=$q->param('apop');
    my $dnsauto=$q->param('dnsauto');
    my $ftpQuota=$q->param('ftpquota');
    my $ftpUsers=$q->param('ftpusers');

    # the checkboxes...
    $fpx     ? ($fpx     = 'on') : ($fpx     = 'off');
    $casp    ? ($casp    = 'on') : ($casp    = 'off');
    $php     ? ($php     = 'on') : ($php     = 'off');
    $ftp     ? ($ftp     = 'on') : ($ftp     = 'off');
    $cgi     ? ($cgi     = 'on') : ($cgi     = 'off');
    $ssl     ? ($ssl     = 'on') : ($ssl     = 'off');
    $ssi     ? ($ssi     = 'on') : ($ssi     = 'off');
    $apop    ? ($apop    = 'on') : ($apop    = 'off');
    $dnsauto ? ($dnsauto = 'on') : ($dnsauto = 'off');
    $shell   ? ($shell   = 'on') : ($shell   = 'off');

    my $maxQuotaInMB=$MaxQuota*$BytesPerBlock/1048576;

    my $info='';
    my $err=0;

    # check disk quota
    if ($diskQuota!~/^\d+$/o || $diskQuota==0) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"") . "<BR>";
	$err=1;
    }
    elsif ($diskQuota>$maxQuotaInMB) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaTooBig")."\"") . "<BR>";
	$err=1;
    }

    # if anonymous ftp is checked then check other ftp options for syntax
    if ($ftp eq 'on') {

	if ($ftpQuota !~ /^\d+$/o) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "ftpQuotaNotInteger")."\"") . "<BR>";
	    $err=1;
	}
	elsif ($ftpQuota > $diskQuota) {
	    my $siteDiskQuota=$diskQuota;
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "ftpQuotaTooBig")."\"") . "<BR>";
	    $err=1;
	}
	if ($ftpUsers!~/^\d+$/o || $ftpUsers==0) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "ftpUsersNotPositive")."\"") . "<BR>";
	    $err=1;
	}
    }

    if ($maxusers !~ /^\d*$/ || $maxusers <= 0) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "maxUsersNotValid")."\"") . "<BR>";
	$err=1;
    }

    if ($domain && !netutil_test_domainname($domain)) {
	my($domainName) = $domain; # Match embedded variable
	$info.=eval("\"".Locale::gettext::dgettext("ui", "domainNameNotValid")."\"") . "<BR>";
	$err=1;
    }

    if ($ipaddr && !netutil_test_ipaddr($ipaddr)) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "addrNotValid")."\"") . "<BR>" . "<BR>";
	$err=1;
    }

		undef $ipaddr if ( !$ipaddr ); 
   
    # save all vsite info in the database
    my $obj = new Cobalt::Meta::vsite;
    $obj->retrieve("default");

    my $o_user_quota     = $obj->get("user_quota");
    my $o_user_fpx       = $obj->get("user_fpx");
    my $o_user_casp      = $obj->get("user_casp");
    my $o_user_shell     = $obj->get("user_shell");
    my $o_user_apop      = $obj->get("user_apop");
    my $o_user_namegen   = $obj->get("user_namegen");
    my $o_userlist_sort  = $obj->get("userlist_sort");
    my $o_userlist_range = $obj->get("userlist_range");

    my $default = Cobalt::Meta->new("type" => "vsite",
				    "name" => "default",
				    "hostname" => "",
				    "domain" => $domain,
				    "ipaddr" => $ipaddr,
				    "fpx" => $fpx,
				    "casp" => $casp,
				    "php" => $php,
				    "maxusers" => $maxusers,
				    "quota" => $diskQuota,
				    "shell" => $shell,
				    "cgi" => $cgi,
				    "ssl" => $ssl,
				    "ssi" => $ssi,
				    "ftp" => $ftp,
				    "ftpusers" => $ftpUsers,
				    "ftpquota" => $ftpQuota,
				    "apop" => $apop,
				    "dns_auto" => $dnsauto,
				    "suspend" => "no",
				    "user_quota" => $o_user_quota,
				    "user_fpx" => $o_user_fpx,
				    "user_casp" => $o_user_casp,
				    "user_shell" => $o_user_shell,
				    "user_apop" => $o_user_apop,
				    "user_namegen" => $o_user_namegen,
				    "userlist_sort" => $o_userlist_sort,
				    "userlist_range" => $o_userlist_range);

    if (!$err) {

	$default->save();
	print "Location: $siteListScriptURL\n\n";

    }
    else {

	&genSiteDefaultsPage($default, $info);

    }
}

  undef ($q,$locale,$page);

}

#
# argument 0 is the object with site info
# argument 1 is the object with bandwidth limit info
#
sub genSiteDefaultsPage
{
    my($obj,$info) = (@_);

    # find default CGI
    my $cgiChecked='';
    if ($obj->get("cgi") eq 'on') { $cgiChecked='CHECKED'; }

    # find default SSI
    my $ssiChecked='';
    if ($obj->get("ssi") eq 'on') { $ssiChecked='CHECKED'; }

    # find default SSL
    my $sslChecked='';
    if ($obj->get("ssl") eq 'on') { $sslChecked='CHECKED'; }

    # find default APOP
    my $apopChecked='';
    if ($obj->get("apop") eq 'on') { $apopChecked='CHECKED'; }

    # auto-DNS configuration
    my $dnsautoChecked='';
    if ($obj->get("dns_auto") eq 'on') { $dnsautoChecked='CHECKED'; }

    # find default FTP state
    my $ftpChecked='';
    if ($obj->get("ftp") eq 'on') { $ftpChecked='CHECKED'; }

    # find default FPX state
    my $fpxChecked='';
    if ($obj->get("fpx") eq 'on') { $fpxChecked='CHECKED'; }

    # find default CASP state
    my $caspChecked='';
    if ($obj->get("casp") eq 'on') { $caspChecked='CHECKED'; }

    # find default PHP state
    my $phpChecked='';
    if ($obj->get("php") eq 'on') { $phpChecked='CHECKED'; }

    # find default shell account state
    my $shellChecked='';
    if ($obj->get("shell") eq 'on') { $shellChecked='CHECKED'; }

    my $ftpQuota  = $obj->get("ftpquota");
    my $ftpUsers  = $obj->get("ftpusers");
    my $ipaddr    = $obj->get("ipaddr");
    my $domain    = $obj->get("domain");
    my $maxusers  = $obj->get("maxusers");
    my $diskQuota = $obj->get("quota");

    if ($info) {
	$info = "'$info', 'error'";
    } else {
	$info = "'', 'status'";
    }

    # Read in the template
    open (TEMPLATE, "$Template_dir/siteDefaults.tmpl")
	|| die "Cannot read $Template_dir/siteDefaults.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while (<TEMPLATE>) {

	s/\[SITEDEFAULTSSCRIPTURL\]/$siteDefaultsScriptURL/o;
	s/\[SITELISTSCRIPTURL\]/$siteListScriptURL/o;
	s/\[CGICHECKED\]/$cgiChecked/;
	s/\[SHELLCHECKED\]/$shellChecked/;
	s/\[SSICHECKED\]/$ssiChecked/;
	s/\[SSLCHECKED\]/$sslChecked/;
	s/\[DISKQUOTA\]/$diskQuota/;
	s/\[FTPCHECKED\]/$ftpChecked/;
	s/\[APOPCHECKED\]/$apopChecked/;
	s/\[FTPQUOTA\]/$ftpQuota/;
	s/\[MAXFTPUSERS\]/$ftpUsers/;
	s/\[USER_INFO\]/$info/;
	s/\[IPADDR\]/$ipaddr/;
	s/\[MAXUSERS\]/$maxusers/;
	s/\[FPXCHECKED\]/$fpxChecked/;
	s/\[CASPCHECKED\]/$caspChecked/;
	s/\[PHPCHECKED\]/$phpChecked/;
	s/\[DOMAIN\]/$domain/;
	s/\[DNSAUTO\]/$dnsautoChecked/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print;
    }
    close TEMPLATE;
}

1;
