#!/usr/bin/perl
#
# Copyright 1998-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteDel;

use Cobalt::Meta;
use Cobalt::Meta::vsite;
use Cobalt::Vsite;
use Cobalt::Locale;
use Cobalt::DNS;
use Cobalt::User;
use Quota;
use FileHandle;
use Cobalt::Util qw( update_status );

sub main {

my $siteListScript="/cgi-bin/.cobalt/turbo_ui/dispatch/siteList";
my $q=shift;
my $group = $q->param( 'nic' );
my $errstr = undef;

my $locale=Cobalt::Locale->getByHTTPHeader($ENV{HTTP_ACCEPT_LANGUAGE});
if ($locale) { $ENV{LANGUAGE}=$locale->getLocale }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (Cobalt::User::user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

# No group specified or group does not exist.  IE sucks.
unless ($group && Cobalt::Vsite::vsite_get_bygroup($group)) {
    $errstr .= "invalid group: $group";
    print "Location: $siteListScript\?info=$errstr\n\n";
}

my $site=new Cobalt::Meta::vsite;
$site->retrieve($group);

# lock the ui
Cobalt::Util::lock_ui(1);

# depopulate DNS records
if($site->get("dns_auto") eq 'on') {
    my $fqdn = $site->hostname.'.'.$site->domain;
    my %emailalii = Cobalt::Email::mail_virtuser_list_route($fqdn);
    my @email = keys %emailalii;
    my @fqdns = ($fqdn);
    push @fqdns, Cobalt::Vsite::vsite_list_alias($group);
    push @fqdns, @email;
    Cobalt::DNS::dns_remove(@fqdns);
}

my ($ret,$info);

# check disk usage to see if we need to fork off
my $gid = getgrnam($group);
my ($blocks, $files) = (Quota::query(Quota::getqcarg('/home'), $gid, 1))[0, 4];
my $num_users = $site->SQL("SELECT COUNT(*) FROM users WHERE vsite = '$group'");

if ($num_users > 30 || ($files > 1000 && $blocks > 327680) || $blocks > 409600) {
	# need to fork off deletion process

	my $log_fh = new FileHandle(">/home/sites/.sitedelstat");
	my $old_fh = select $log_fh;
	$| = 1;
	select $old_fh;

	my $pid;
	my @status = ($$, $site->fqdn, 'START');

	$SIG{CHLD} = "IGNORE";
	$CGI::Fast::fcgi_request->Detach();

	if ($pid = fork) {
		# parent
		$CGI::Fast::fcgi_request->Attach();
		print STDERR "PID: $pid\n" if ($DEBUG);

		# write status file with child process pid before finishing
		$status[0] = $pid;
		update_status($log_fh, \@status);

		$log_fh->close();
		print $q->redirect($siteListScript);
		
		exit 0;
	} elsif ($pid == 0) {
		# child
		$status[0] = $$;
		update_status($log_fh, \@status);

		# because of forking need to really force the db connection
		# to reset otherwise users don't get deleted from the system
		my $dbc = Cobalt::Meta::getdb;
		$dbc->reset;

		($ret, $info) = $site->del();

		$status[3] = join '<BR>', @{ $info };

		print STDERR $status[3] if ($DEBUG);

		if (!$ret) {
			$status[2] = 'ERROR';
		} else {
			$status[2] = 'DONE';
		}

		update_status($log_fh, \@status);

		$log_fh->close();

		# exit because we are no longer connected to the server
		Cobalt::Util::lock_ui(0);
		undef ($ret, $info);
		exit 0;

	} elsif ($pid != 0) {
		# fork failed
		my $msg = "foo";

		print "Location: $siteListScript?info=$msg\n\n";
	}

} else {
	($ret, $info) = $site->del();

	Cobalt::Util::lock_ui(0);

	$errstr = join("<BR>", @$info);

	if (!$ret) {
    		print "Location: $siteListScript\?info=$errstr\n\n";
	}
	elsif (scalar(@$info)) {
    		print "Location: $siteListScript\?info=$errstr\n\n";
	} else {
		print "Location: $siteListScript\n\n";
	}
}

undef ($ret, $info);

}

1;


