#!/usr/bin/perl

# Site FTP daemon control and configuration
#
# Copyright 1999 Cobalt Networks, Inc
# Will DeHaan <will@cobaltnet.com>
# Version 1.0

# If the authenticating user is the RaQ administrator named $Adminuser,
# then first post a configurable form and process its submission.
# If the user is a site admin, only report the existing configuration

package Cobalt::UI::siteFTP;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Ftp;	# Anonymous config, quota control
use Cobalt::Vsite;	# Find IP used by site
use Cobalt::Group;  	# Site quota reporting
use Cobalt::Meta;
use Cobalt::User;
use Cobalt::Util;

use DB_File; 		# UI database reporting
use Fcntl qw(O_RDWR O_CREAT O_RDONLY);
use Locale::gettext;

## Globals
my($siteListScriptPath)="$AdminCGIs_dir/turbo_ui/dispatch/siteList";
my($showTemplate)="$Template_dir/showFTP.tmpl";
my($editTemplate)="$Template_dir/editFTP.tmpl";
my($siteFTP_script)="$AdminCGIs_dir/turbo_ui/dispatch/siteFTP";
my $forbiddenURL="/.cobalt/error/forbidden.html";

my ($group,$q,$page,$locale, $siteObj, $maxUsers, $quota, $ftpquota, $ftpusers, $anon, $ipaddr, $button_style, $header_style, $table_style);

sub main {

if ($ENV{'PARAM_GROUP'}) {
    $group = $ENV{'PARAM_GROUP'};
} else {
    $q=shift;
    $group = $q->param('group');
    $page=$q->param('page');
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "site-adm", $group)) {
	print "location: $forbiddenURL\n\n";
	die "Unauthorised access by $user, not a member of $group.\n";
    } 
}

$siteObj = Cobalt::Meta->create("type" => "vsite");
$siteObj->retrieve($group);

$anon = $siteObj->get("ftp");
$maxUsers = $siteObj->get("ftpusers");
$quota = $siteObj->get("ftpquota");
$ipaddr = $siteObj->get("ipaddr");
$ftpquota = $quota;
$ftpusers = $maxUsers;

$quota*=$BytesPerBlock/1048576; # convert from block to megabyte

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

## main()
die "Invalid site affiliated group: $group\n" unless(group_exist($group));

my $ret;
if ($page && (($ENV{'REMOTE_USER'} eq $Adminuser) || ($ENV{'REMOTE_USER'} eq $Alteradmin)) ) {
    $ret=update_config();
    $anon = $siteObj->get("ftp");
    $maxUsers = $siteObj->get("ftpusers");
    $ftpquota = $siteObj->get("ftpquota");
    generate_config($ret);
} elsif (($ENV{'REMOTE_USER'} eq $Adminuser) || ($ENV{'REMOTE_USER'} eq $Alteradmin)) {
    generate_config();	# Create the form
} else {
    report_config(); 	# Un-editable page, not a form
}
}
## fin main()


## Subs
sub update_config
{
    my $anonftp;
    if ($q->param('ftp') eq 'on') {
	$anon=1;
	$anonftp = 'on';
    } else {
	$anon=0;
	$anonftp = 'off';
    }

    $quota=$q->param('ftpquota');
    $maxUsers=$q->param('ftpusers');
    
    my $ftpquota = $quota;
	#  $quota*=1048576/$BytesPerBlock; # megabyte to blocks
	# Ftp.pm should do this.
    
    return MSG_get_ui("ftp_invalid_quota") unless ($quota =~ /^\d+$/);
    return MSG_get_ui("ftp_invalid_users") unless ($maxUsers =~ /^\d+$/ && $maxUsers > 0);
    return MSG_get_ui("ftp_cant_overquota") if ($quota > (group_get_quota($group))[1]);
    
    # Configure server
    my($status)=ftp_config($ipaddr,1,$anon,$group,$maxUsers,$quota);
    return substr($status,4) if ($status !~ /^2/); # Check for errors

    $siteObj->put("ftp" => $anonftp,
		  "ftpusers" => $maxUsers,
		  "ftpquota" => $ftpquota);
    $siteObj->save;
	
    return;
}

sub generate_config
{ 
    my($info)=@_;
    my $err;

    if ($info) {	
	$err = "'$info', 'error'";
    } else {
	$err = "'', 'status'";
    }

    my $ftpChecked;
    if ($anon eq 'on') {
	$ftpChecked = 'CHECKED';
    }
    
    # display page
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    open(TMPL,"$editTemplate") || die "Can not open template $editTemplate for reading: $!";
    while(<TMPL>)
    {
	#s/\[\]//g;
	s/\[USER_INFO\]/$err/g;
	s/\[MAXFTPUSERS\]/$maxUsers/g;
	s/\[FTPCHECKED\]/$ftpChecked/g;
	s/\[FTPQUOTA\]/$ftpquota/g;
	s/\[GROUP\]/$group/g;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;
	
	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	
	print;
    }
    close(TMPL);
} 

sub report_config
{
    my $rowSpan = 1;
    my $check = MSG_get_ui("disabled");
    my $configRow;

    if($anon eq 'on') {
	my $BCC_010=MSG_get_ui("BCC_010");
	my $BCC_011=MSG_get_ui("BCC_011");

	$check=<<END;
<FONT COLOR="#FFFFFF">&nbsp;&nbsp;</FONT> $ftpquota <FONT>&nbsp;</FONT>$BCC_010
<BR><FONT COLOR="#FFFFFF">&nbsp;&nbsp;</FONT> $maxUsers <FONT>&nbsp;</FONT>$BCC_011
END
    ;
  }

    # display page
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    open(TMPL,"$showTemplate") || die "Can not open template $showTemplate for reading: $!";
    while(<TMPL>)
    {
	s/\[FTP\]/$check/g;
	s/\[ROWSPAN\]/$rowSpan/g;
	s/\[FTPCONFIG\]/$configRow/g;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;
	
	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	
	print;
    }
    close(TMPL);
}
1;
