#!/usr/bin/perl
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved
#

package Cobalt::UI::siteList;

use Cobalt::Meta;
use Cobalt::Vsite;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Util qw(
			FILE_lock_r FILE_lock_w FILE_lock_rw
			FILE_unlock FILE_unlock_rw
       			MSG_error MSG_abort MSG_ok MSG_get MSG_get_ui
             		parse_cgipost read_status 
		); 
use Cobalt::UI qw( output_page );
use Cobalt::Product;

my ($button_style, $table_style, $header_style, $locale, $query, $info, $sort, $order, $search, $PATH_search_vsite, @where);
my $URL_siteList      = "/cgi-bin/.cobalt/turbo_ui/dispatch/siteList";

sub main {

$query = shift;

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

# check if there is an in progress or finished site deletion
my $delete_flag = '/home/sites/.sitedelstat';

if (-f $delete_flag) {
	# short circuit to deletion status
	gen_site_del_status($query, $delete_flag);
} else {
	# just generate site list
	$locale = Cobalt::Locale->getByHTTPHeader($ENV{HTTP_ACCEPT_LANGUAGE});
	
	if ($locale) { 
		$ENV{LANGUAGE}=$locale->getLocale();
		$button_style=$locale->getProperty("buttonStyle");
		$header_style=$locale->getProperty("headerStyle");
		$table_style=$locale->getProperty("tableStyle"); 
	}
	
	$PATH_search_vsite = "/usr/admserv/html/.cobalt/search/search_vsite.html." . $locale->getLocale();
	
	# define html for search
	my($search_html);
	my $ret = &Cobalt::Util::FILE_lock_r($PATH_search_vsite,*SBOX);
	if ($ret) {
	    while (<SBOX>) {
	        $search_html .= $_;
	    }
	    FILE_unlock(*SBOX);
	}
	
	$query->import_names('Q');
	
	if ($ENV{PARAM_INFO}) {
	    $info = $ENV{PARAM_INFO};
	} else {
	    $info = $Q::info;
	}
	    
	$search=0;
	@where=();
	
	# if ($Q::search eq "yes" && $Q::searchtext ne "") {
	if ($Q::search eq 'yes') {
	
	    $search = 1;
	
	    my($searchby, $searchop) = ($Q::searchby, $Q::searchop);
	
	    if ($searchby eq 'ipaddr') {
		if ($searchop eq 'match') {
		    push @where, $searchby;
		    push @where, '<<=';
		} elsif ($searchop eq 'nomatch') {
		    push @where, "not $searchby";
		    push @where, '<<=';
		} else {
		    push @where, $searchby;
		    push @where, $searchop;
		}
		push @where, $Q::searchtext;
	    } elsif ($searchby eq 'service') {
		# search hostname-contains for per-service search
	        $searchby = 'fqdn'; 
	        $searchop = 'match';
	    } else {
		push @where, $searchby;
	
		if ($searchop eq 'match') {
		    push @where, '~*';
		    push @where, '.*' . $Q::searchtext . '.*';
		} elsif ($searchop eq 'nomatch') {
		    push @where, '!~*';
		    push @where, '.*' . $Q::searchtext . '.*';
		} else {
		    push @where, $searchop;
		    push @where, $Q::searchtext;
		}
	    }
	    if (!@where) {
		push @where, 'name', '!=', 'default';
	    } else {
		push @where, 'AND', 'name', '!=', 'default';
	    }
	}
	
	&genSiteListPage("sort"        => ($Q::sort) ? $Q::sort : "hostname",
			 "order"       => $Q::order,
			 "info"        => $info,
			 "search"      => $search,
			 "searchop"    => $Q::searchop,
			 "searchby"    => $Q::searchby,
			 "searchtext"  => $Q::searchtext,
			 "search_html" => $search_html,
			 "where"       => \@where);
}

undef ($locale, $query, $info, $sort, $order, $search, $PATH_search_vsite, @where, $Q);

}

sub genSiteListPage {
    my (%info)=@_;
    my ($hostname,$domain,$fqdn,$ipaddr,$name,$suspend,$ftp,$ssl,$apop,$fpx,$asp,$php,@o,$obj,$options);
    my ($html,$sorted);
    my ($manageHostX,$removeHostX,$manageHost,$removeHost);
    
    my $script_siteList = "/cgi-bin/.cobalt/turbo_ui/dispatch/siteList";
    my $script_siteDel = "/cgi-bin/.cobalt/turbo_ui/dispatch/siteDel";
    my $script_siteAdd = "/cgi-bin/.cobalt/turbo_ui/dispatch/siteAdd";
    my $script_siteDefaults = "/cgi-bin/.cobalt/turbo_ui/dispatch/siteDefaults";

    my $str_vsite = MSG_get_ui("virtualSiteList");
    my $str_ipaddr = MSG_get_ui("ipAddr");
    my $str_fqdn = MSG_get_ui("hostName");
    my $str_domain = MSG_get_ui("domainName");
    my $str_help = MSG_get_ui("help");
    my $str_saving = MSG_get_ui("saving");
    my $str_virtualSiteList = MSG_get_ui("virtualSiteList");

    my $str_sort_vsite_hostname_asc = MSG_get_ui("sort_vsite_list_hostname_asc");
    my $str_sort_vsite_hostname_desc = MSG_get_ui("sort_vsite_list_hostname_desc");
    my $str_sort_vsite_ipaddr_asc = MSG_get_ui("sort_vsite_list_ipaddr_asc");
    my $str_sort_vsite_ipaddr_desc = MSG_get_ui("sort_vsite_list_ipaddr_desc");
    my $str_sitedefaultsbut = MSG_get_ui("sitedefaultsbut");
    my $str_siteaddbut = MSG_get_ui("siteaddbut");

    my $help_vsiteList = MSG_get_ui("vsiteListHelp");
    my $help_ipaddr = MSG_get_ui("ipAddrHelp");
    my $help_fqdn = MSG_get_ui("siteHostNameHelp");
    my $help_access = MSG_get_ui("siteAccessHelp");
    my $help_action = MSG_get_ui("siteManageDelHelp");
    my $help_del_confirm = MSG_get_ui("siteRemoveConfirm");
    my $help_del_confirmnodisk = MSG_get_ui("siteRemoveConfirmNoDisk");
    my $help_sitedefaults = MSG_get_ui("setSiteDefaults");
    my $help_siteadd = MSG_get_ui("addSite");
    my $help_sitelist_options = MSG_get_ui("help_sitelist_options");
    my $help_search = MSG_get_ui("site_search");

    my $javascript = <<END;
top.showInfo( '$info{"info"}', 0, 'error' );

var help_vsiteList='$help_vsiteList';
var help_fqdn='$help_fqdn';
var help_ipaddr='$help_ipaddr';
var help_action='$help_action';
var help_access='$help_access';
var help_search='$help_search';

function delSite( nic ) {
  if( confirm( "$help_del_confirm" ) ) {
    top.showInfo( '$str_saving', 0, 'wait' );
    location="${script_siteDel}?nic="+escape( nic );
  }
}

function delSiteNoDisk( nic ) {
  if( confirm( "$help_del_confirmnodisk" ) ) {
    top.showInfo( '$str_saving', 0, 'wait' );
    location="${script_siteDel}?nic="+escape( nic );
  }
}
END

    my $noIcon='<IMG SRC="/.cobalt/images/blank">';
    my $spacer='<FONT COLOR="#FFFFFF">&nbsp;</FONT>';
    my $suspendIcon='<IMG SRC="/.cobalt/images/suspend_icon_small" ALT="' .
	MSG_get_ui("suspended_site") . '">';
    my $ftpIcon='<IMG SRC="/.cobalt/images/small_file_icon" ALT="' .
	MSG_get_ui("anonymousFTP") . '">';
    my $aspIcon='<IMG SRC="/.cobalt/images/asp-icon" ALT="' .
	MSG_get_ui("asp") . '">';
    my $phpIcon='<IMG SRC="/.cobalt/images/php" ALT="' .
	MSG_get_ui("php") . '">';
    my $fpxIcon='<IMG SRC="/.cobalt/images/counter_icon_small" ALT="' .
	MSG_get_ui("fpxShowSite") . '">';
    my $sslIcon='<IMG SRC="/.cobalt/images/ssl_icon_small" ALT="' .
	MSG_get_ui("activeSSL") . '">';
    my $apopIcon='<IMG SRC="/.cobalt/images/apop_icon_small" ALT="' .
	MSG_get_ui("apopUser") . '">';
    my $bwIcon='<IMG SRC="/.cobalt/images/bwmgmt_icon_small" ALT="' .
	MSG_get_ui("bw_setip") . '" BORDER=0>';
    my $deleteIconGray='<IMG SRC="/libImage/removeGray">';

    my $meta = new Cobalt::Meta("type" => "bw");
    my %limits = map { $_ => 1; } $meta->SQL(qq[ SELECT DISTINCT name FROM bw ]);

    my @where = ($search) ? @{ $info{"where"} } : undef;
    @o=Cobalt::Meta::query("type"    => "vsite",
			   "keys"    => [ "ipaddr", "fqdn", "name", "ftp", "fpx",
					  "apop", "ssl", "suspend", "casp", "php",
                                          "shell", "cgi", "ssi" ],
			   "sort"    => $info{"sort"},
			   "order"   => $info{"order"},
			   "where"   => \@where);

    # our query set may be too large...
    if (($info{'searchby'} eq 'service') && ($info{'searchop'} ne 'any')) {
        my(@reduced);
	foreach (@o) {
            if($info{'searchop'} eq 'band') {
                push(@reduced, $_) if (exists($limits{$_->get('ipaddr')}));
            } else {
                push(@reduced, $_) if ($_->get($info{'searchop'}) eq 'on');
            }
        }
        @o = @reduced;
    }

    my $numfound = scalar(@o);
    my $noSites = MSG_get_ui("search_vsite_nonefound");

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    
    print $query->header(-type => 'text/html' . $charsetText,
			 -expires => '-1d',
			 -pragma => 'no-cache');
    undef($charset, $charsetText);
    
    print $query->start_html(-title => "$str_virtualSiteList",
			     -BGCOLOR => "#FFFFFF",
			     -script => $javascript);

    my $searchstr;
    if ($info{"search"}) {
	if ($numfound == 1) {
	    $str_vsite = MSG_get_ui("search_vsite_results_1");
	} else {
	    $str_vsite = MSG_get_ui("search_vsite_results", $numfound);
	}
	$searchstr = "&search=yes";
	$searchstr .= "&searchby=" . $info{"searchby"};
	$searchstr .= "&searchop=" . $info{"searchop"};
	$searchstr .= "&searchtext=" . $info{"searchtext"};
    }

    my ($sort_hostname_icon,$sort_ipaddr_icon);

    my $sort_hostname_icon_go = "<A HREF=\"$script_siteList?sort=hostname$searchstr\" onMouseOver=\"status=\'$str_sort_vsite_hostname_asc\'; return true;\"><IMG SRC=\"/libImage/downArrowGray\" BORDER=0 ALT=\"$str_sort_vsite_hostname_asc\"></A>";
    my $sort_hostname_icon_asc = "<A HREF=\"$script_siteList?sort=hostname$searchstr\" onMouseOver=\"status=\'$str_sort_vsite_hostname_asc\'; return true;\"><IMG SRC=\"/libImage/upArrow\" BORDER=0 ALT=\"$str_sort_vsite_hostname_asc\"></A>";
    my $sort_hostname_icon_desc = "<A HREF=\"$script_siteList?sort=hostname&order=desc$searchstr\" onMouseOver=\"status=\'$str_sort_vsite_hostname_desc\'; return true;\"><IMG SRC=\"/libImage/downArrow\" BORDER=0 ALT=\"$str_sort_vsite_hostname_desc\"></A>";

    my $sort_ipaddr_icon_go = "<A HREF=\"$script_siteList?sort=ipaddr$searchstr\" onMouseOver=\"status=\'$str_sort_vsite_ipaddr_asc\'; return true;\"><IMG SRC=\"/libImage/downArrowGray\" BORDER=0 ALT=\"$str_sort_vsite_ipaddr_asc\"></A>";
    my $sort_ipaddr_icon_asc = "<A HREF=\"$script_siteList?sort=ipaddr$searchstr\" onMouseOver=\"status=\'$str_sort_vsite_ipaddr_asc\'; return true;\"><IMG SRC=\"/libImage/upArrow\" BORDER=0 ALT=\"$str_sort_vsite_ipaddr_asc\"></A>";
    my $sort_ipaddr_icon_desc = "<A HREF=\"$script_siteList?sort=ipaddr&order=desc$searchstr\" onMouseOver=\"status=\'$str_sort_vsite_ipaddr_desc\'; return true;\"><IMG SRC=\"/libImage/downArrow\" BORDER=0 ALT=\"$str_sort_vsite_ipaddr_desc\"></A>";

    if ($info{"order"}) {
	if ($info{"sort"} eq "hostname") {
	    $sort_hostname_icon = $sort_hostname_icon_asc;
	    $sort_ipaddr_icon = $sort_ipaddr_icon_go;
	    $str_fqdn = "<B>$str_fqdn</B>";
	    $sorted = "&sort=hostname";
	} else {
	    $sort_hostname_icon = $sort_hostname_icon_go;
	    $sort_ipaddr_icon = $sort_ipaddr_icon_asc;
	    $str_ipaddr = "<B>$str_ipaddr</B>";
	    $sorted = "&sort=ipaddr";
	}
    } else {
	if ($info{"sort"} eq "hostname") {
	    $sort_hostname_icon = $sort_hostname_icon_desc;
	    $sort_ipaddr_icon = $sort_ipaddr_icon_go;
	    $str_fqdn = "<B>$str_fqdn</B>";
	    $sorted = "&sort=hostname&order=desc";
	} else {
	    $sort_hostname_icon = $sort_hostname_icon_go;
	    $sort_ipaddr_icon = $sort_ipaddr_icon_desc;
	    $str_ipaddr = "<B>$str_ipaddr</B>";
	    $sorted = "&sort=ipaddr&order=desc";
	}
    }

    # display html for search
    print $info{'search_html'};

    $html = <<END;
<CENTER>
<TABLE BORDER="0" CELLSPACING="0" CELLPADDING="0" >
<TR>

<TD NOWRAP>
<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0"><TR>
  <TD><A HREF="$script_siteDefaults" onMouseOver="status='$help_sitedefaults'; return true;"><IMG BORDER="0" SRC="/libImage/buttonLeftDot.gif"></A></TD>

  <TD NOWRAP STYLE="$header_style"><A HREF="$script_siteDefaults" onMouseOver="status='$help_sitedefaults'; return true;"><FONT STYLE="$button_style">
$str_sitedefaultsbut
</FONT></A></TD>

  <TD><A HREF="$script_siteDefaults" onMouseOver="status='$help_sitedefaults'; return true;"><IMG BORDER="0" SRC="/libImage/buttonRight.gif"></A></TD>

</TR></TABLE></TD>
<TD><IMG BORDER="0" SRC="/libImage/spaceHolder" WIDTH="5" HEIGHT="30"></TD>
<TD NOWRAP>

<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0"><TR>
  <TD><A HREF="$script_siteAdd" onMouseOver="status='$help_siteadd'; return;"><IMG BORDER="0" SRC="/libImage/buttonLeftDot.gif"></A></TD>

  <TD NOWRAP STYLE="$button_style"><A HREF="$script_siteAdd" onMouseOver="status='$help_siteadd'; return true;"><FONT STYLE="$button_style">
$str_siteaddbut
</FONT></A></TD>

  <TD><A HREF="$script_siteAdd" onMouseOver="status='$help_siteadd'; return true;"><IMG BORDER="0" SRC="/libImage/buttonRight.gif"></A></TD>

</TR></TABLE></TD>
</TR>
</TABLE>

<TABLE BORDER="1" CELLSPACING="0" WIDTH="600">
    <TR>
	<TH COLSPAN="4" BGCOLOR="#000099" ALIGN="CENTER" VALIGN="middle" HEIGHT="22">
	    <FONT STYLE="$header_style">$str_vsite</FONT>
        </TH>
    </TR>
END
    ;
    print $html;

    if ($numfound == 0) {
	$html = <<END;
<TR>
    <TD BGCOLOR="#FFFFFF" COLSPAN="4">
	$spacer
	<I>$noSites</I>
    </TD>
</TR>
END
    ;
    } else {
	$html = <<END;
    <TR>
	<TD WIDTH="320" ALIGN="LEFT" NOWRAP>
	    $spacer<A HREF="javascript: top.showInfo( help_fqdn );" onMouseOver="status=' $str_help '; top.showInfo( help_fqdn ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A>
	    <FONT STYLE="$table_style" SIZE="+1">$str_fqdn</FONT>$sort_hostname_icon
	</TD>
	<TD WIDTH="120" ALIGN="LEFT" NOWRAP>
	    $spacer<A HREF="javascript: top.showInfo( help_ipaddr );" onMouseOver="status=' $str_help '; top.showInfo( help_ipaddr ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A>
	    <FONT STYLE="$table_style" SIZE="+1">$str_ipaddr</FONT>$sort_ipaddr_icon
        </TD>
	<TD WIDTH="50" ALIGN="CENTER" NOWRAP>
	    <A HREF="javascript: top.showInfo( help_action );" onMouseOver="status=' $str_help '; top.showInfo( help_action ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A>
	</TD>
    </TR>
END
    ;
    }

    print $html;

    foreach (@o) {
	$name = $_->get("name");
	next if ($name eq "default");
	$fqdn = $_->get("fqdn");
	my $tfqdn = $fqdn;
	$ipaddr = $_->get("ipaddr");
	$ftp = $_->get("ftp");
	$fpx = $_->get("fpx");
	$apop = $_->get("apop");
	$ssl = $_->get("ssl");
	$asp = $_->get("casp");
	$php = $_->get("php");
	$suspend = $_->get("suspend");
	my $real_suspend = Cobalt::Vsite::vsite_issuspend($name);

	if ($suspend eq 'on' or $real_suspend) {
	    $fqdn = "<FONT COLOR=\"#AAAAAA\">$fqdn</FONT>";
	    $ipaddr = "<FONT COLOR=\"#AAAAAA\">$ipaddr</FONT>";
	}

	print "<TR VALIGN=MIDDLE>\n\t<TD ALIGN=LEFT VALIGN=MIDDLE NOWRAP><FONT COLOR=#FFFFFF>&nbsp;</FONT>";

	($suspend eq 'on' or $real_suspend) ? (print "$fqdn") : (print "<A HREF=\"http://$fqdn/\" TARGET=NEW>$fqdn</A>");
        # print $suspendIcon if ($suspend eq 'on' or $real_suspend);
        print "$spacer</TD>\n";

	print "\t<TD ALIGN=LEFT VALIGN=MIDDLE NOWRAP><FONT COLOR=#FFFFFF>&nbsp;</FONT>";
        print $ipaddr;
        # (exists($limits{$ipaddr})) ? (print "<A HREF=\"/cgi-bin/.cobalt/turbo_ui/dispatch/bwMod?ipaddr=$ipaddr\" BORDER=0>".$ipaddr.'</A>') : print $ipaddr;
        print "$spacer</TD>\n";

	$manageHostX = MSG_get_ui("manageHostX", $tfqdn);
	$manageHost = '<A HREF="/nav/cList.php?root=sitemanageRoot&group=' . $name . '&hostname=' . $tfqdn  . '&goto=base_user" TARGET="_top" onMouseOver="status=\'' . $manageHostX . '\'; return true;"><IMG SRC="/libImage/modify" ALT="' . $manageHostX . '" BORDER="0"></A>';

	$removeHostX = MSG_get_ui("removeHostX", $tfqdn);
	if ($real_suspend lt 0) {
	    $removeHost = '<A HREF="javascript: delSiteNoDisk(\'' . $name . '\');" onMouseOver="status=\'' . $removeHostX . '\'; return true;"><IMG SRC="/libImage/remove" ALT="' . $removeHostX . '" BORDER="0"></A>';
	    $manageHost = '<IMG SRC="/libImage/modifyGray" BORDER="0">';	    
	} else {
	    $removeHost = '<A HREF="javascript: delSite(\'' . $name . '\');" onMouseOver="status=\'' . $removeHostX . '\'; return true;"><IMG SRC="/libImage/remove" ALT="' . $removeHostX . '" BORDER="0"></A>';
	}

	print '<TD ALIGN=LEFT VALIGN=MIDDLE NOWRAP>';

	print $spacer;

	print $manageHost;
	print $spacer;
	($name eq "home") ? print $deleteIconGray : print $removeHost;
	print $spacer;
	print "</TD>\n</TR>\n";
    }
    print "\n</TABLE>\n";
    print $query->end_html;
    undef ($hostname,$domain,$fqdn,$ipaddr,$name,$suspend,$ftp,$ssl,$apop,$fpx,$asp,$php,@o,$obj,$options);
}

sub gen_site_del_status
{
	my $cgi = shift;
	my $delete_flag = shift;

	my ($subs, @status);
	my $header = undef;
	my $template = "$Template_dir/delete_status.tmpl";

	@status = read_status($delete_flag);

	$subs->{STATUS} = "[str.site.delete.$status[2]]";

	if ($status[2] =~ /DONE|ERROR/) {
		$subs->{INFO} = "'[STATUS]', 'status'";
	} else {
		$subs->{INFO} = "'[STATUS]', 'wait'";
	}

	$subs->{TITLE} = '[str.site.delete]';
	$subs->{VSITE} = $status[1];
	
	# the default button blank it out later if still in the process of
	# deletion
	$subs->{BUTTON} = Cobalt::UI::button({
		'action' => $URL_siteList,
		'help' => '[str.back]',
		'text' => '[str.back]' });

	if (`/bin/ps --no-headers -C dispatch` !~ /$status[0]/) {

		# possible system error, check status again
		@status	= read_status($delete_flag);

		unlink $delete_flag;

		if ($status[2] !~ /DONE|ERROR/) {
			print STDERR "SITE DELETION FAILED WITHOUT ERROR\n" if ($DEBUG);
			$subs->{STATUS} = '[str.system.error]';
		} else {
			$subs->{STATUS} = "[str.site.delete.$status[2]]";
		}

	} elsif ($status[2] =~ /DONE|ERROR/) {
		
		# catch all just in case the process was still
		# around in the check above
		print STDERR "CLEANING UP PREVIOUS SITE DELETION\n" if ($DEBUG);
		unlink $delete_flag;

	} else {

		print STDERR "SITE DELETION IN PROGRESS\n" if ($DEBUG);
		$subs->{BUTTON} = '';
		$header = $cgi->header(-Refresh => ("4; URL=$URL_siteList"));

	}

	output_page($template, $subs, $header);

}

1;
