

# $Id: siteMod.pm,v 1.18 2001/01/17 01:57:50 jcheng Exp $

# siteMod is intended to report site settings to site administrators.
# Site admins manage users, data, backups, that's it.  RaQ admins manage
# services and sites.  Thus, this page and other per-site service 
# configuration utilities (ftp) are only editable by the RaQ admins
# named $Adminuser and $Alteradmin.

package Cobalt::UI::siteMod;

use Cobalt::Product;
use Cobalt::DNS;
use Cobalt::Email;
use Cobalt::Fpx;
use Cobalt::Ftp;
use Cobalt::Group;
use Cobalt::List;
use Cobalt::Meta::vsite;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Network;
use Cobalt::User;
use Cobalt::Vsite;
use Cobalt::Http;
use Cobalt::Util;
use Cobalt::Locale;
use Cobalt::Layout;
use Cobalt::DiskUtils;

use DB_File;
use Fcntl qw(O_RDONLY O_RDWR O_CREAT);

require Locale::gettext;
Locale::gettext::textdomain("ui");

# global variables
my ($q, $locale, $site, $page, $group, $user );
my $siteModScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/siteMod';
my $forbiddenURL="/.cobalt/error/forbidden.html";

sub main {

$q=shift;
$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

$site = new Cobalt::Meta::vsite;

# 'page' is a hidden field used to differentiate a completed form from 
# a form request
$page  =$q->param('page');
$group =$q->param('group');
$user  =$ENV{REMOTE_USER};

use vars qw( @wsAlias @esAlias );
@wsAlias=();
@esAlias=();

unless (Cobalt::User::user_authok($user, "site-adm", $group)) {
    print "location: $forbiddenURL\n\n";    
    die "Unauthorised access by $user, not a member of $group.\n";
}

if ($page eq 'sitemod') {

    # read form input
    my $h_ipaddr=$q->param('ipaddr');
    my $h_hostname=$q->param('hostname');
    my $h_domain=$q->param('domain');
    my $h_fqdn=$h_hostname.'.'.$h_domain;
    my $h_cgi=$q->param('cgi');
    my $h_shell=$q->param('shell');
    my $h_ssi=$q->param('ssi');
    my $h_ssl=$q->param('ssl');
    my $h_fpx=$q->param('fpx');
    my $h_casp=$q->param('casp');
    my $h_php=$q->param('php');
    my $h_dnsauto=$q->param('dnsauto');
    my $h_quota=$q->param('diskquota');
    my $h_maxusers=$q->param('maxusers');
    my $h_apop=$q->param('apop');
    my $h_suspend=$q->param('suspend');

    my $h_wsAlias=$q->param('webserveralias');
    @wsAlias = split /[\s,]+/m, $h_wsAlias;

    my $h_esAlias=$q->param('emailserveralias');
    @esAlias = split /[\s,]+/m, $h_esAlias;

    # strip zero padding from IP addresse
    $h_ipaddr =~ s/\.0+(\d)/\.$1/g;
    $h_ipaddr =~ s/^0//g;

    # set nulls to "off" for database
    $h_fpx ? ($h_fpx = 'on') : ($h_fpx = 'off');
    $h_casp ? ($h_casp = 'on') : ($h_casp = 'off');
    $h_php ? ($h_php = 'on') : ($h_php = 'off');
    $h_dnsauto ? ($h_dnsauto = 'on') : ($h_dnsauto = 'off');
    $h_cgi ? ($h_cgi = 'on') : ($h_cgi = 'off');
    $h_ssi ? ($h_ssi = 'on') : ($h_ssi = 'off');
    $h_ssl ? ($h_ssl = 'on') : ($h_ssl = 'off');
    $h_apop ? ($h_apop = 'on') : ($h_apop = 'off');
    $h_suspend ? ($h_suspend = 'on') : ($h_suspend = 'off');
    $h_shell ? ($h_shell = 'on') : ($h_shell = 'off');

    my $hobj = Cobalt::Meta->new("type" => "vsite");
    $hobj->put("name"     => $group,
	       "hostname" => $h_hostname,
	       "domain"   => $h_domain,
	       "fqdn"     => $h_fqdn,
	       "ipaddr"   => $h_ipaddr,
	       "quota"    => $h_quota,
	       "maxusers" => $h_maxusers,
	       "shell"    => $h_shell,
	       "cgi"      => $h_cgi,
	       "ssi"      => $h_ssi,
	       "ssl"      => $h_ssl,
	       "fpx"      => $h_fpx,
	       "casp"     => $h_casp,
	       "dns_auto"  => $h_dnsauto,
	       "php"      => $h_php,
	       "apop"     => $h_apop,
	       "suspend"  => $h_suspend);
    
    my $oldhobj = new Cobalt::Meta::vsite;
    $oldhobj->retrieve($group);

    # bail with error unless user logged in
    unless ($ENV{REMOTE_USER}) {
	genSiteModPage($group, "Unauthenticated site update attempt");
    }

    # verify the input data
    my $result = verifySiteModPage($group, $hobj, $oldhobj);

    # null returned if no error
    if ($result) {
	genSiteModPage($group,$result);
    } else {
	$result=handleSiteModPage($group, $hobj, $oldhobj);
	genSiteModPage($group,$result);
    }

} else {
    genSiteModPage($group);
}

}

sub verifySiteModPage
{
    my($group,$vobj,$oldvobj) = (@_);

    my $ret;
    my $err=0;

    my $info='';

    my $ipAddr=$vobj->get('ipaddr');
    my $hostName=$vobj->get('hostname');
    my $domain=$vobj->get('domain');
    my $cgi=$vobj->get('cgi');
    my $shell=$vobj->get('shell');
    my $ssi=$vobj->get('ssi');
    my $ssl=$vobj->get('ssl');
    my $fpx=$vobj->get('fpx');
    my $casp=$vobj->get('casp');
    my $php=$vobj->get('php');
    my $diskQuota=$vobj->get('quota');
    my $maxUsers=$vobj->get('maxusers');
    my $apop=$vobj->get('apop');
    my $suspend=$vobj->get('suspend');

    # find out if the same fqdn is used
    if (scalar($site->SQL(qq[SELECT name FROM vsite WHERE name <> 'default'],
			  qq[AND name <> '$group' AND hostname = '$hostName' AND domain = '$domain']))) {
	my($name)=$hostName;
	$info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_cant_hostnameinuse")."\"") . "<BR>";
	$err=1;
    }

    # Enforce the necessity of a maximum number of users for this site
    if ($maxUsers!~/^\d+$/o || $maxUsers==0)
    {
	$info.=MSG_get_ui("maxUsersNotValid") . "<BR>";
	$err=1;
    }

    my @siteusers = vsite_get_users($group);
    my($usercount)=scalar @siteusers;
    if ($maxUsers < $usercount) {
	$info.=MSG_get_ui("maxUsersLessthanCurrentUsers") . "<BR>";
	$err=1;
    }

    unless(netutil_test_domainname($hostName) && netutil_test_domainname($domain)) {
	$info.=MSG_get_ui("hostNameRequired") . "<BR>";
	$err=1;
    }
	
    # enforce only one site per IP with SSL enabled
    if ($ssl eq 'on') {
	if (scalar($site->SQL(qq[SELECT name FROM vsite WHERE name <> 'default'],
			      qq[AND name <> '$group' AND ipaddr = '$ipAddr' AND ssl IS TRUE]))) {
	    $info.=Cobalt::Util::MSG_get_ui("vsite_cant_ipinuse_ssl");
	    $err=1;
	}
    }

    if( !$hostName || !$ipAddr || !$diskQuota ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "vsite_essential_data")."\"") . "<BR>";
	$err=1;
    }
    if( $hostName=~/localhost/io ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "localhostNotAllowed")."\"") . "<BR>";
	$err=1;
    }
    if( $hostName && !netutil_test_domainname( $hostName ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "hostNameNotValid")."\"") . "<BR>";
	$err=1;
    }
    if( !netutil_test_ipaddr( $ipAddr ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "addrNotValid")."\"") . "<BR>";
	$err=1;
    }
    if( $diskQuota!~/^\d+$/o || $diskQuota==0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"") . "<BR>";
	$err=1;
    }

    if ($err) {
	return($info);
    } else {
	return;
    }
}

sub handleSiteModPage
{
    my($group,$vobj,$oldvobj) = (@_);
    my(@siteUsers) = Cobalt::Vsite::vsite_get_users($group);

    my $ret=0;
    my $err=0;
    my $info='';

    my $ipAddr=$vobj->get('ipaddr');
    my $hostName=$vobj->get('hostname');
    my $domain=$vobj->get('domain');
    my $cgi=$vobj->get('cgi');
    my $shell=$vobj->get('shell');
    my $ssi=$vobj->get('ssi');
    my $ssl=$vobj->get('ssl');
    my $fpx=$vobj->get('fpx');
    my $casp=$vobj->get('casp');
    my $php=$vobj->get('php');
    my $dnsauto=$vobj->get('dns_auto');
    my $diskQuota=$vobj->get('quota');
    my $maxUsers=$vobj->get('maxusers');
    my $apop=$vobj->get('apop');
    my $suspend=$vobj->get('suspend');

    # don't lose other things
    my $o_user_quota     =$oldvobj->get("user_quota");
    my $o_user_fpx       =$oldvobj->get("user_fpx");
    my $o_user_casp      =$oldvobj->get("user_casp");
    my $o_user_shell     =$oldvobj->get("user_shell");
    my $o_user_apop      =$oldvobj->get("user_apop");
    my $o_user_namegen   =$oldvobj->get("user_namegen");
    my $o_userlist_sort  =$oldvobj->get("userlist_sort");
    my $o_userlist_range =$oldvobj->get("userlist_range");

    # obtain old settings
    my $oldipAddr   =$oldvobj->get("ipaddr");
    my $oldhostName =$oldvobj->get("hostname");
    my $olddomain   =$oldvobj->get("domain");
    my $oldfpx      =$oldvobj->get("fpx");
    my $oldcasp     =$oldvobj->get("casp");
    my $oldphp      =$oldvobj->get("php");
    my $oldftp      =$oldvobj->get("ftp");
    my $oldapop     =$oldvobj->get("apop");
    my $oldcgi      =$oldvobj->get("cgi");
    my $oldssi      =$oldvobj->get("ssi");
    my $oldssl      =$oldvobj->get("ssl");
    my $oldshell    =$oldvobj->get("shell");
    my $oldsuspend  =$oldvobj->get("suspend");
    my $oldftpquota =$oldvobj->get("ftpquota");
    my $oldftpusers =$oldvobj->get("ftpusers");
    my $olddns_auto =$oldvobj->get("dns_auto");
    
    $vobj->put("ftp"            => $oldftp,
	       "ftpusers"       => $oldftpusers,
	       "ftpquota"       => $oldftpquota,
	       "user_quota"     => $o_user_quota,
	       "user_fpx"       => $o_user_fpx,
	       "user_casp"      => $o_user_casp,
	       "user_shell"     => $o_user_shell,
	       "user_apop"      => $o_user_apop,
	       "user_namegen"   => $o_user_namegen,
	       "userlist_sort"  => $o_userlist_sort,
	       "userlist_range" => $o_userlist_range);
    

    my $fqdn    = qq[$hostName.$domain];
    my $oldfqdn = qq[$oldhostName.$olddomain];

    ## for config of bandwidth limits
    my @other_newip = $site->SQL(qq[SELECT name FROM vsite WHERE name <> 'default'],
				 qq[AND name <> '$group' AND ipaddr = '$ipAddr']);

    # Begin the upgrade process, concatonate errors, function order should
    # clear dependencies

    # Site will be re-added with new parameters immediately
    # viste_add/del manage virtual network interfaces so it is necessary
    # to formally delete the "old" site leaving site data intact and 
    # create its replacement
    if ($fqdn ne $oldfqdn || $ipAddr ne $oldipAddr) {
	# Fetch old FPX configuration, to be re-added if FPX remains enabled

	# disable casp if it was on 
	if ($oldcasp eq 'on') {
	    $ret = vsite_set_casp($group, 0);
	    if ($ret!~/^2/o) {
		$info.=substr( $ret, 4 ) . "<BR>";
		$err=1;
	    }
	}

	$ret = vsite_del($group);
	if ($ret!~/^2/o) {
	    $info.=substr( $ret, 4 ) . "<BR>";
	    $err=1;
	}
	
	# Set the default webmaster email address to the site group
	$ret = vsite_add($ipAddr,$fqdn,$group,$group);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ) . "<BR>";
	    $err=1;
	}

	my $fA = ($oldftp eq 'on') ? 1 : 0;
	$ret=ftp_config($ipAddr,1,$fA,$group,$oldftpusers,$oldftpquota);
	if ($ret!~/^2/o) {
	    $info.=substr ($ret, 4) . "<BR>";
	    $err=1;
	}

	# log file fragments in MB not blocks
	my $logRotateLimit=int($diskQuota/10);
	$logRotateLimit=1 unless ($logRotateLimit);
	# Rotate split log fragments for the site
	$ret=vsite_set_logrotate($group,$logRotateLimit);
	if ($ret!~/^2/) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	if ($fpx eq 'on') {
	    $ret=vsite_set_fpx( $group, 1 );
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}
	
	if ($casp eq 'on') {
	    $oldcasp = $casp;
	    $ret=vsite_set_casp( $group, 1 );
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}
    
	if ($php eq 'on') {
	    $oldphp = $php;
	    $ret=vsite_set_php( $group, 1 );
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}
    
	# things to do if the fqdn changed
	if ($fqdn ne $oldfqdn) {

            # we need to tell frontpage that we've changed the web site
            # or else it gets confused
            if ($oldfpx eq 'on') {
                fpx_rename_web($oldfqdn, $fqdn);
            }

	    # delete old cw entry
	    $ret=Cobalt::Email::mail_del_cw($oldfqdn);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	    # and add a new one
	    $ret=Cobalt::Email::mail_add_cw($fqdn);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	    
	    # update the emailServerAliases to point at new FQDN
	    my %cural=Cobalt::Email::mail_virtuser_list_route($oldfqdn);
	    $ret=Cobalt::Email::mail_virtuser_del_route(%cural);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    foreach (keys %cural) {
		$cural{$_} = $fqdn;
	    }
	    $ret=Cobalt::Email::mail_virtuser_add_route(%cural);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }

	    # delete the virtual list server
	    my @siteusers = vsite_get_users($group);
	    $ret=Cobalt::List::list_modify_virtual($group, $oldfqdn, $fqdn);
	    if ($ret!~/^2/) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    my %active = map {$_ => 1} @siteusers;
	    
	    my($siteUser,$alias); # for-loop instances 
	    my(%aliasDel,%aliasAdd);
	    my(%allAlias)=mail_virtuser_list_alias();
	    foreach $alias (keys %allAlias)
	    {
		if ($active{ $allAlias{$alias} })
		{
		    my($userAlias,$dom)=split(/\@/,$alias);
		    $aliasDel{$alias}=$allAlias{$alias};
		    $aliasAdd{ $userAlias.'@'.$fqdn }=$allAlias{$alias};
		} 
	    }
	    
	    # Delete old/pre-existing aliases, then (re)add
	    $ret=Cobalt::Email::mail_virtuser_del_alias(%aliasDel);
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    $ret=Cobalt::Email::mail_virtuser_add_alias(%aliasAdd);
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	    
	    # generate management site (make layout file)
	    &Cobalt::Layout::layout_siteManage($group,$fqdn);
	    
	} # end [fqdn changed]

	# Updates specific to only modified IP address
	if($oldipAddr ne $ipAddr) 
	{
	    # Allow relaying to modified IP
	    $ret = mail_del_relay($oldipAddr);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }
	    $ret = mail_add_relay($ipAddr);
	    if ($ret!~/^2/o) {
		$info.=substr($ret, 4) . "<BR>";
		$err=1;
	    }

	} # end [IP changed]
          
        # migrate DNS records
        dns_migrate($oldhostName, $hostName, $olddomain, $domain, $oldipAddr, $ipAddr) if ($dnsauto eq 'on');

    } # end [FQDN or IP changed]

    #### handle changes to web server aliases
    my %allal = map { $_ => 1; } Cobalt::Vsite::vsite_list_alias();
    my %cural = map { $_ => 1; } Cobalt::Vsite::vsite_list_alias($group);
    my @goodal;

    foreach (@wsAlias) {
	unless (/^[a-zA-Z0-9\-\.]+$/) {
	    push @info, Cobalt::Util::MSG_get_ui("webServerAliasInvalid", $_);
	}
	elsif (exists($cural{$_})) {
	    push @goodal, $_;
	}
	elsif (exists($allal{$_})) {
	    push @info, Cobalt::Util::MSG_get_ui("webServerAliasInUse", $_);
	}
	else {
	    push @goodal, $_;
	}
    }

    # now add those that passed the tests
    $ret = Cobalt::Vsite::vsite_set_alias($group, @goodal);
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

    #### handle email server aliases
    my %emailal = Cobalt::Email::mail_virtuser_list_route($fqdn);
    Cobalt::Email::mail_virtuser_del_route(%emailal);
    Cobalt::Email::mail_del_cw(keys %emailal);

    if (scalar(@esAlias)) {
	# tell sendmail that this address is local
	$ret = Cobalt::Email::mail_add_cw(@esAlias);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

	# route email @server.alias.com --> @host.domain.com
	%emailal = map { $_ => $fqdn } @esAlias;
	$ret = Cobalt::Email::mail_virtuser_add_route(%emailal);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
    }

    # Auto-DNS 
    if (($dnsauto eq 'on') && ($olddns_auto ne 'on')) {
    	dns_import($hostName, $domain, $ipAddr, join(' ',@goodal), join(' ',@esAlias));
    } elsif ($dnsauto eq 'on') {
        # Update auto-DNS for site web and email alias changes
	Cobalt::DNS::dns_remove(keys %emailal);
	Cobalt::DNS::dns_remove(keys %cural);
    	dns_import($hostName, $domain, $ipAddr, join(' ',@goodal), join(' ',@esAlias));
    }

    # Revoke site user(s) shell access if previously granted
    if ($oldshell eq 'on' && $shell eq 'off') {
	foreach (@site_users) 
	{
	    Cobalt::User::user_mod($_,'','','',$Badshell);
	}
	# Mark site users new shell in postgresql
	Cobalt::User::user_strip_feature('shell', @siteUsers); 
    }
		    
    # Revoke site user(s) apop access if previously granted
    if ($oldapop eq 'on' && $apop eq 'off') {
	foreach (@site_users) 
	{
	    Cobalt::Email::mail_apop_deluser($_);
	}
	# Mark site users new apop in postgresql
	Cobalt::User::user_strip_feature('apop', @siteUsers); 
    }
		    
    # set disk quota
    $diskQuota=0 if ($diskQuota =~ /unlimited/i);
    $diskQuota*=1048576/$BytesPerBlock; # megabyte to block
    $ret=Cobalt::Group::group_set_quota($group,$diskQuota);
    if ($ret!~/^2/o) {
	$info.=substr($ret, 4).'<BR>';
	$err=1;
    }

    # config cgi
    $ret=Cobalt::Vsite::vsite_set_cgis($group , ($cgi eq 'on'));
    if ($ret!~/^2/) {
	$info.=substr ($ret, 4).'<BR>';
	$err=1;
    }

    # config ssi
    $ret=Cobalt::Vsite::vsite_set_ssi($group , ($ssi eq 'on'));
    if ($ret!~/^2/) {
	$info.=substr ($ret, 4).'<BR>';
	$err=1;
    }

    # config ssl
    vsite_set_sslconf($group, ($ssl eq 'on') ? 1 : 0);
    
    # config fpx
    if ($oldfpx eq 'off' && $fpx eq 'on') {
	$ret=Cobalt::Fpx::fpx_set_web($group, 1, $q->param('fpxpass0'));
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    } elsif ($fpx eq 'off' && $oldfpx eq 'on') {
	$ret=Cobalt::Fpx::fpx_set_web( $group, 0 );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    # config casp
    if ($oldcasp eq 'off' && $casp eq 'on') {
	$ret=Cobalt::Vsite::vsite_set_casp($group, 1);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    } elsif ($casp eq 'off' && $oldcasp eq 'on') {
	$ret=Cobalt::Vsite::vsite_set_casp( $group, 0 );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    # config php
    if ($oldphp eq 'off' && $php eq 'on') {
	$ret=Cobalt::Vsite::vsite_set_php($group, 1);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    } elsif ($php eq 'off' && $oldphp eq 'on') {
	$ret=vsite_set_php( $group, 0 );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if (($oldsuspend eq 'off') && ($suspend eq 'on')) {
	$ret = vsite_disable($group);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    } elsif (($suspend eq 'off') && ($oldsuspend eq 'on')) {
	$ret = vsite_enable($group);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }
	    
    Cobalt::Http::http_restart(1);
	Cobalt::Email::mail_sendmail_hup();

    # there was too many chances for a site to be almost added but
    # not get in the database, so i'd rather there be an entry in
    # the database but not have a complete site than have a bad
    # database so we save no matter what
    $vobj->save;

    if ($err) {
	return($info);
    } else {
	return;
    }
}

sub genSiteModPage
# Parses & prints the siteMod template
# Arguments: optional error text with image tags prepended
# Return value: true
# Site effects: none 
{
    my $group = shift;
    my $info = shift;

    my $vsite = new Cobalt::Meta::vsite;
    $vsite->retrieve($group);
    
    my $diskQuota  = $vsite->get("quota");
    my $ipAddr     = $vsite->get("ipaddr");
    my $domainName = $vsite->get("domain");
    my $hostName   = $vsite->get("hostname");
    my $apop       = $vsite->get("apop");
    my $suspend    = $vsite->get("suspend");
    my $shell      = $vsite->get("shell");
    my $maxUsers   = $vsite->get("maxusers");
		my $fqdn       = $hostName . "." . $domainName;

    @wsAlias=Cobalt::Vsite::vsite_list_alias($group);
    my %eals=Cobalt::Email::mail_virtuser_list_route($hostName.'.'.$domainName);
    @esAlias=keys %eals;

    # figure out the disk label.
    my $volume = disk_getlabel("$Htgroup_dir/$group");

    my $fpxJscriptTestPW;
  
    # $xRow consists of simple reported data or default-valued form fields
    # RaQ admin gets the editable form, site admin gets read-only data
    my ($cgiRow,$shellRow,$ssiRow,$sslRow,$fpxRow,$caspRow,$phpRow,$emailRow,$diskQuotaRow,$dnsRow);
    my ($maxUsersRow,$ipRow,$hostRow,$domainRow,$suspendRow,$apopRow);
    my ($webServerAliasRow,$emailServerAliasRow);

    if(($user eq $Alteradmin) || ($user eq $Adminuser)) 
    {
        # Disk quota set to "0" implies unlimited
	if($diskQuota==0) {
	    $diskQuotaRow="<INPUT TYPE=TEXT NAME=\"diskquota\" VALUE=\"\" SIZE=\"5\">"; 
	}
	else {
	    $diskQuotaRow="<INPUT TYPE=TEXT NAME=\"diskquota\" VALUE=\"$diskQuota\" SIZE=\"5\" onChange=\"checkDiskQuota( this );\">";
        }

	$webServerAliasRow='<INPUT TYPE="TEXT" NAME="webserveralias" SIZE="28" VALUE="' . join(", ", @wsAlias) . '" onChange="lineUpCommaList( this )">';
	$emailServerAliasRow='<INPUT TYPE="TEXT" NAME="emailserveralias" SIZE="28" VALUE="' . join(", ", @esAlias) . '" onChange="lineUpCommaList( this )">';

        # $xChecked represents either a default-valued checkbox or a 
        # yes/no image gif
        # Identify which services are enabled
 	my $cgiChecked    ='CHECKED' if ($vsite->get("cgi")     eq 'on');
	my $ssiChecked    ='CHECKED' if ($vsite->get("ssi")     eq 'on');
	my $sslChecked    ='CHECKED' if ($vsite->get("ssl")     eq 'on');
        my $fpxChecked    ='CHECKED' if ($vsite->get("fpx")     eq 'on');
        my $caspChecked   ='CHECKED' if ($vsite->get("casp")    eq 'on');
        my $phpChecked    ='CHECKED' if ($vsite->get("php")     eq 'on');
	my $suspendChecked='CHECKED' if ($vsite->get("suspend") eq 'on');
        my $shellChecked  ='CHECKED' if ($vsite->get("shell")   eq 'on');
        my $apopChecked   ='CHECKED' if ($vsite->get("apop")    eq 'on');

        # Generate html.. 
	$cgiRow="<INPUT TYPE=CHECKBOX NAME=\"cgi\" VALUE=\"on\" $cgiChecked>";
	$ssiRow="<INPUT TYPE=CHECKBOX NAME=\"ssi\" VALUE=\"on\" $ssiChecked>";
	$sslRow="<INPUT TYPE=CHECKBOX NAME=\"ssl\" VALUE=\"on\" $sslChecked>";
        $caspRow="<INPUT TYPE=CHECKBOX NAME=\"casp\" VALUE=\"on\" $caspChecked>";
        $phpRow="<INPUT TYPE=CHECKBOX NAME=\"php\" VALUE=\"on\" $phpChecked>";
	$maxUsersRow="<INPUT TYPE=TEXT NAME=\"maxusers\" VALUE=\"$maxUsers\" SIZE=\"5\" onChange=\"checkDiskQuota( this, maxUsersNotValid );\">";
        $shellRow="<INPUT TYPE=CHECKBOX NAME=\"shell\" VALUE=\"on\" $shellChecked>";
        $apopRow="<INPUT TYPE=CHECKBOX NAME=\"apop\" VALUE=\"on\" $apopChecked>";
        
	# FPX is very funky.  It requires a 'webmaster' fixed account password
	# when enabling.  Once enabled this password must be changed from the FP client
	# or the service must be disabled and re-enabled.  Feh!
	$fpxRow="<INPUT TYPE=CHECKBOX NAME=\"fpx\" VALUE=\"on\" $fpxChecked>";
	unless ($vsite->get("fpx") eq 'on')
	{
		$fpxRow = <<EOF;
<TABLE WIDTH="246" BORDER="0" CELLPADDING="0" CELLSPACING="0">
  <TR>
    <TD WIDTH="25" ALIGN="LEFT" ROWSPAN="2">$fpxRow<FONT STYLE=$table_style>&nbsp;&nbsp;</FONT></TD>
    <TD WIDTH="150" ALIGN="RIGHT"><FONT STYLE=$table_style>[str.fpxOnPassword]  </FONT></TD>
    <TD WIDTH="75" ALIGN="LEFT"><INPUT TYPE="PASSWORD" NAME="fpxpass0" SIZE="8" MAXLENGTH="8"></TD>
  </TR>
  <TR>
    <TD WIDTH="150" ALIGN="RIGHT"><FONT STYLE=$table_style>[str.fpxPwAgain]  </FONT></TD>
    <TD WIDTH="75" ALIGN="LEFT"><INPUT TYPE="PASSWORD" NAME="fpxpass1" SIZE="8" MAXLENGTH="8"></TD>
  </TR>
</TABLE>
EOF

	# We've got to add a javascript test for a password definition when enabling
	$fpxJscriptTestPW = <<EOF;
if( f.fpx.checked ) {
		if( f.fpxpass0.value!=f.fpxpass1.value ) {
			f.fpxpass1.focus();
			top.showInfo( fpxPWnoMatch, 0, 'error' );
			return false;
		}
		if( f.fpxpass0.value.length==0 ) {
			f.fpxpass0.focus();
			top.showInfo( fpxNoPW, 0, 'error' );
			return false;
		}
	}
EOF
	}

	# Disallow FPX changes on a suspended site
	if(($vsite->get('suspend') eq 'on') && ($vsite->get('fpx') eq 'on')) {
	    $fpxRow = MSG_get_ui('suspended.unavailable.enabled');
	    $fpxRow .= '<INPUT TYPE="HIDDEN" SIZE="-1" NAME="fpx" VALUE="on">';
	} elsif (($vsite->get('suspend') eq 'on') && ($vsite->get('fpx') ne 'on')) {
	    $fpxRow = MSG_get_ui('suspended.unavailable.disabled');
	    $fpxJscriptTestPW = '';
	}

        # Network settings for the main group are edited in the RaQ admin
        # interface, not in the site settings page.  This is bad ui design.
	if($group eq $MainGroup)
	{
	    # MainGroup settings changed through RaQ admin interface
	    $ipRow="<A HREF=\"/nav/cList.php?root=root&group=server&goto=base_cpnetwork\" TARGET=\"_top\">$ipAddr</A><INPUT TYPE=HIDDEN NAME=\"ipaddr\" VALUE=\"$ipAddr\"><INPUT TYPE=HIDDEN NAME=\"oldipaddr\" VALUE=\"$ipAddr\">";
	    $domainRow="<A HREF=\"/nav/cList.php?root=root&group=server&goto=base_cpnetwork\" TARGET=\"_top\">$domainName</A><INPUT TYPE=HIDDEN NAME=\"domain\" VALUE=\"$domainName\"><INPUT TYPE=HIDDEN NAME=\"olddomain\" VALUE=\"$domainName\">";
	    $hostRow="<A HREF=\"/nav/cList.php?root=root&group=server&goto=base_cpnetwork\" TARGET=\"_top\">$hostName</A><INPUT TYPE=\"HIDDEN\" NAME=\"hostname\" SIZE=\"28\" VALUE=\"$hostName\"><INPUT TYPE=\"HIDDEN\" NAME=\"oldhostname\" VALUE=\"$hostName\">";
            $suspendRow=MSG_get_ui("disabled");
	} else
	{
	    $ipRow="<INPUT TYPE=\"TEXT\" NAME=\"ipaddr\" SIZE=\"15\" MAXSIZE=\"15\"VALUE=\"$ipAddr\" onChange=\"checkIPAddr( this, addrNotValid );\"><INPUT TYPE=\"HIDDEN\" NAME=\"oldipaddr\" VALUE=\"$ipAddr\">";
	    $hostRow="<INPUT TYPE=\"TEXT\" NAME=\"hostname\" SIZE=\"28\" VALUE=\"$hostName\" onChange=\"checkDomainName( this, hostNameNotValid );\"><INPUT TYPE=\"HIDDEN\" NAME=\"oldhostname\" VALUE=\"$hostName\">";
	    $domainRow="<INPUT TYPE=\"TEXT\" NAME=\"domain\" SIZE=\"28\" VALUE=\"$domainName\" onChange=\"checkDomainName( this, domainNameNotValid1 );\"><INPUT TYPE=\"HIDDEN\" NAME=\"olddomain\" VALUE=\"$domainName\">";
	    $suspendRow="<INPUT TYPE=CHECKBOX NAME=\"suspend\" VALUE=\"on\" $suspendChecked>";
	} 

	# auto-DNS checkbox is for the admin only
	$dnsRow=<<EOF;
        <TR>
                <TD WIDTH="26">
                <CENTER><A HREF="javascript: top.showInfo( dnsautoHelp );" onMouseOver="status='[str.help]'; top.showInfo( dnsautoHelp ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A></CENTER>
                </TD>
                <TD WIDTH="270"><FONT STYLE=[FONTT]>&nbsp;&nbsp;[str.dnsauto]</FONT>
                </TD>
                <TD WIDTH="254"><INPUT TYPE=CHECKBOX NAME="dnsauto" SIZE="5" VALUE="on" CHECKED></TD>
        </TR>
EOF
	$dnsRow =~ s/CHECKED//g unless ($vsite->get("dns_auto") eq 'on');

    } else # Site admin, do not allow editable fields, note the yes/no gifs
           # instead of checkboxes
    {
	$diskQuotaRow ="<FONT>" . $diskQuota . "</FONT>";
	$maxUsersRow  =$maxUsers;
	$ipRow        =$ipAddr;
	$hostRow      =$hostName;
	$domainRow    =$domainName;
	$webServerAliasRow=join(", ", @wsAlias);
	$emailServerAliasRow=join(", ", @esAlias);
        $webServerAliasRow ||= '<FONT COLOR=#FFFFFF>&nbsp;</FONT>';
        $emailServerAliasRow ||= '<FONT COLOR=#FFFFFF>&nbsp;</FONT>';

#	my $disabled_msg = MSG_get_ui("disabled");
	my $enabled_msg  = "<IMG SRC=\"/.cobalt/images/yes\">";
	my $disabled_msg = "<IMG SRC=\"/.cobalt/images/suspend_icon_small\">";

	if($vsite->get("cgi") eq 'on')
        { 
            $cgiRow=$enabled_msg;
        } else
	{
            $cgiRow=$disabled_msg;
        }

	if($vsite->get("ssi") eq 'on')
        { 
            $ssiRow=$enabled_msg;
        } else
	{ 
            $ssiRow=$disabled_msg;
        }

	if($vsite->get("ssl") eq 'on')
        { 
            $sslRow=$enabled_msg;
        } else
	{ 
            $sslRow=$disabled_msg;
        }
	
	if($vsite->get("fpx") eq 'on')
        { 
            $fpxRow=$enabled_msg;
        } else
	{ 
            $fpxRow=$disabled_msg;
        }

	if($vsite->get("casp") eq 'on')
        { 
            $caspRow=$enabled_msg;
        } else
	{ 
            $caspRow=$disabled_msg;
        }

	if($vsite->get("php") eq 'on')
        { 
            $phpRow=$enabled_msg;
        } else
	{ 
            $phpRow=$disabled_msg;
        }

	if($vsite->get("suspend") eq 'on') 
        { 
            $suspendRow=$enabled_msg;
        } else
	{ 
            $suspendRow=$disabled_msg;
        }

	if($vsite->get("shell") eq 'on') 
        { 
            $shellRow=$enabled_msg;
        } else
	{ 
            $shellRow=$disabled_msg;
        }

	if($vsite->get("apop") eq 'on') 
        { 
            $apopRow=$enabled_msg;
        } else
	{ 
            $apopRow=$disabled_msg;
        }
    }

    my $bwlimitRow=&Cobalt::BandWidth::bw_getip($ipAddr);
    if ($bwlimitRow) {
        $bwlimitRow .= '  ' . MSG_get_ui("kbps") . '.';
    }
    else {
        $bwlimitRow = MSG_get_ui("unlimited");
    }

    # figure out quota string
    my $quotamsg = MSG_get_ui("volSiteQuota",$volume);
    $diskQuotaRow .= "<FONT STYLE=$table_style> $quotamsg</FONT>";

    # Html headers..
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    
    # Read in the template
    if(($user eq $Adminuser) || ($user eq $Alteradmin))
    {
        open( TEMPLATE, "$Template_dir/siteMod.tmpl" )
    	    || die "Cannot read $Template_dir/siteMod.tmpl: $!\n";
    } else
    {
        open( TEMPLATE, "$Template_dir/showSite.tmpl" )
    	    || die "Cannot read $Template_dir/showSite.tmpl: $!\n";
    }

    while( <TEMPLATE> ) {
	if (/\[/o) {
	    s/\[USER_INFO\]/$info/;
	    s/\[BWLIMIT\]/$bwlimitRow/;
	    s/\[CGI\]/$cgiRow/;
	    s/\[SSI\]/$ssiRow/;
	    s/\[SSL\]/$sslRow/;
	    s/\[SHELL\]/$shellRow/;
	    s/\[FPX\]/$fpxRow/;
	    s/\[FPXJSCRIPT\]/$fpxJscriptTestPW/;
	    s/\[AUTODNS\]/$dnsRow/;
	    s/\[CASP\]/$caspRow/;
	    s/\[PHP\]/$phpRow/;
	    s/\[SUSPEND\]/$suspendRow/;
	    s/\[MAXUSERS\]/$maxUsersRow/;
	    s/\[WEBSERVERALIAS\]/$webServerAliasRow/;
	    s/\[EMAILSERVERALIAS\]/$emailServerAliasRow/;
	    s/\[APOP\]/$apopRow/;
	    s/\[DISKQUOTA\]/$diskQuotaRow/;
	    s/\[GROUP\]/$group/;
	    s/\[HOSTNAME\]/$hostRow/;
	    s/\[IPADDR\]/$ipRow/;
	    s/\[DOMAIN\]/$domainRow/;
      s/\[FQDN\]/$fqdn/;
	    s/\[SITEMODSCRIPTURL\]/$siteModScriptURL/;
            s/\[FONTH\]/$header_style/go;
            s/\[FONTT\]/$table_style/go;
            s/\[FONTB\]/$button_style/go;

	
	    s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;
	}
	print;
    }
    close TEMPLATE;
}
1;
