#!/usr/bin/perl

# use strict;

# author: Harris Vaegan-Lloyd
#         Cobalt Neworks
# date: 19-9-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteSSL;

use Cobalt::Product;
use Cobalt::Group;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Vacation;
use Cobalt::Vsite;
use Cobalt::Ssl;
use Cobalt::User;
use Cobalt::Util;
use Cobalt::Meta::vsite;

require Locale::gettext;
Locale::gettext::textdomain("ui");

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $siteManageRedirect='';
my $scriptUrl;
my $vsite = new Cobalt::Meta::vsite;
my ($q,$group,$locale,$button_style,$header_style,$table_style);

sub main {

$q=shift;

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

# the page that run this script
$page=$q->param( 'page' );

$group;
if($ENV{'PARAM_GROUP'}) { 
    $group=$ENV{'PARAM_GROUP'}; 
}
else { 
    $group=$q->param('group'); 
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "site-adm", $group)) {
	print "location: $forbiddenURL\n\n";    
	die "Unauthorised access by $user, not a member of $group.\n";
    }
}

if (!$group && @ARGV) {
    $group=$ARGV[0];
    $page='';
}

# get site info for fqdn
$vsite->retrieve($group);

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

$scriptUrl="/cgi-bin/.cobalt/turbo_ui/dispatch/siteSSL?group=$group";
$siteManageRedirect = "/nav/cList.php?root=sitemanageRoot&group=$group&hostname=" . $vsite->fqdn;

if( $page eq 'siteSSL' ) { 
    handleSiteSSLPage(\*STDOUT, $q); 
} else {
    genSiteSSLPage( \*STDOUT, $group );
}

}


#
# handle redirect
#
sub printRedirect
{
    my ($handle, $q, $url, $location) = @_;
    $location = $url . $location;    

    # handle msie differently. on the other hand, we might want netscape 
    # navigator to show the same behaviour as ie.
    if ($ENV{HTTP_USER_AGENT} =~ /MSIE/) {
	open (TEMPLATE, "$Template_dir/siteSSL_reload.tmpl")
	    || die "Cannot read $Template_dir/siteSSL_reload.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print $handle "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	while (<TEMPLATE>) {
	    s/\[SSLURLMSG\]/MSG_get_ui("ssl_redirect_msg", $location)/e;
	    s/\[str.([^\]]+)\]/MSG_get_ui("$1")/ge;
	    print $handle $_;
	}
	close(TEMPLATE);
    } else {
	print $handle $q->redirect(-location => $location,
				   -target => "_top");
	close(STDOUT); # fastcgi friendly
    }
}


#
# event handler 
#
sub handleSiteSSLPage {
    my ($handle, $q) = @_;
    my $err=0;
    my $ret;
    my $info='';
    
    my %subject;
    my $action;
    # read form input
    $subject{'C'} = $q->param('C');
    $subject{'ST'} = $q->param('ST');
    $subject{'L'} = $q->param('L');
    $subject{'O'} = $q->param('O');
    $subject{'OU'} = $q->param('OU');
    $subject{'CN'} = (vsite_get_bygroup($group))[1];
    $subject{'Email'} = $q->param('Email');
    
    my $certificate = $q->param('certificate');
    my $action = $q->param('certopt');
    my $group = $q->param('group');
    
    # Switch on action.
    # Action can be either -
    #		Generating a new self-signed certificate from scratch.
    #		Updating the certificate to one provided by a cert authority.
    #		Averting world war with open-source software.
    if ( $action eq 'gen_cert' ) {
		# Country code must be two uppercase letters.
		$subject{'C'} = uc($subject{'C'});
		if ( length( $subject{'C'} ) != 2 || ($subject{'C'} =~ /[^A-Z]/)) {
		    $err = 1;
		    $info .= Locale::gettext::dgettext('ui',"country_code_mustb2");
		}
		# Check each input to be usre it contains to commans or slashes.
		# This would fubar our parsing code when we get it out of a cert.
		foreach ( keys %subject ) {
		    if ( /[\,\/]/ ) {
			$err = 1;
			$info.=eval("\"".Locale::gettext::dgettext("ui", "ssl_identification_no_comma_slashes")."\"");
		    }
		}
		if (! $err ) {
		    $ret = vsite_ssl_set_identity( $group, 730,
						   $subject{'C'},
						   $subject{'ST'},
						   $subject{'L'},
						   $subject{'O'},
						   $subject{'OU'},
						   $subject{'CN'},
						   $ENV{'REMOTE_USER'}.'@'.$subject{'CN'} );
		    if($ret!~/^2/o) {
			$err=1;
			$info .= substr($ret,4).'<BR>';
		    }
		}
	} elsif ($action eq 'set_new_cert') {
		$ret = ssl_check_valid_cert($certificate);
		if($ret!~/^2/o) {
		    $err=1;
		    $info .= substr($ret,4).'<BR>';
		}
		if (! $err ) {
		    $ret = vsite_ssl_set_cert($group,$certificate);
		    if($ret!~/^2/o) {
				$err=1;
				$info.=substr($ret,4).'<BR>';
		    }
		}
	} elsif ($action eq 'del_cert') {
	    if ($certificate = vsite_ssl_get_cert($group)) {
		$ret = vsite_ssl_clear($group);
		if($ret!~/^2/o) {
		    $err=1;
		    $info .= substr($ret,4).'<BR>';
		}
	    } else {
		$err = 1;
		$info .= MSG_get_ui("ssl_no_cert_delete") . '<BR>';
	    }
	}

	# Do no do checking if we just deleted the cert.
	if ($action ne 'del_cert') {
	    $ret = vsite_check_cert($group);
	    if($ret!~/^2/o) {
		$err =1;
		$info.=substr($ret,4),'<BR>';
	    }

            # make sure the proper include is added for the home
            # group
            if ($group eq $MainGroup)
            {
                &admserv_set_ssl_include(1);
            }
	}
        elsif ($group eq $MainGroup)
        {
            # make sure the ssl line is removed from srm.conf
            &admserv_set_ssl_include(0);
        }

    my ($obj,$fqdn);
    if ( $err ) {
	genSiteSSLPage($handle, $group,$info);
    } else {
	# need to do a hard restart here otherwise the wrong cert is used when
	# accessing the site we just created or set the cert for
        Cobalt::Http::http_restart(9);
	sleep 2; # let the public server chill a few..
	if ($group eq $MainGroup) {
	      my $url = $ENV{HTTP_HOST}; 
	      #$url =~ s/\:\d+//; # reset to port 80
	      if (! defined $ENV{'HTTPS'} ) {
		  $url = 'https://' . $url;
		  printRedirect($handle, $q, $url, $siteManageRedirect);

		  # close all filehandles or ie gets confused
		  close $handle;
		  close STDERR;
		  close STDOUT;
		  close STDIN;
	    	  Cobalt::Admserv::admserv_restart(1);
		  exit 0;
	      } elsif (defined $ENV{'HTTPS'} && ($action eq 'del_cert')) {
		  $url = 'http://' . $url;
		  printRedirect($handle, $q, $url, $siteManageRedirect);

		  close $handle;
		  close STDERR;
		  close STDOUT;
		  close STDIN;
	    	  Cobalt::Admserv::admserv_restart(1);
		  exit 0;
	      }
	  }
	  genSiteSSLPage($handle, $group);
      }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the information shown to user
#
sub genSiteSSLPage {
    my ( $handle, $group, $info )=@_;

	my($subject, $issuer, @expires) = vsite_ssl_get_cert_info($group);

	my($cert, $req, $key) = vsite_ssl_get_cert($group);
        my $cert_setting = $cert ? 1 : 0;

	my $HideStart = '<!--';
	my $HideEnd = '-->';

	foreach ( keys %{$subject} ) {
		# If any of the values are different, it must not be a self-signed
		# certificate. Thus display the issuer by clearing out the hide
		# variables;
		if ( $$subject{$_} ne $$issuer{$_} ) {
			$HideStart = '';
			$HideEnd = '';
			last;
		}
	}

	my $template = "siteSSL.tmpl";

	if ( $group ne 'home' && ! &vsite_get_ssl($group) ) {
		$template = "siteNoSSL.tmpl";
	}

	my $home_cert_warning;
	$home_cert_warning = '[str.site.home.cert.warning]<BR>' if
		( $group eq $MainGroup );

	my ($hide_request0, $hide_request1) = ('<!--  ', '   -->');
	$hide_request0 = $hide_request1 = '' if ($req =~ /\w/);
	
	$req =~ s/\s$//g; # strip trailing space

	# Read in the template
	open (TEMPLATE, "$Template_dir/$template")
		|| die "Cannot read $Template_dir/$template: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print $handle "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	while( <TEMPLATE> ) {
		s/\[SCRIPTURL\]/$scriptUrl/o;
		s/\[USER_INFO\]/$info/;
		s/\[HIDE_START\]/$HideStart/;
		s/\[HIDE_END\]/$HideEnd/;
		s/\[HIDE_REQ0\]/$hide_request0/;
		s/\[HIDE_REQ1\]/$hide_request1/;
		s/\[GROUP\]/$group/;

		s/\[HOMEWARNING\]/$home_cert_warning/;

		s/\[OLDCERT\]/$cert_setting/;
		s/\[REQUEST\]/$req/;
		s/\[CERTIFICATE\]/$cert/;

		s/\[subject.([^\]]+)\]/$$subject{$1}/ge;
		s/\[issuer.([^\]]+)\]/$$issuer{$1}/ge;
		s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

		s/\[FONTH\]/$header_style/g;
		s/\[FONTT\]/$table_style/g;
		s/\[FONTB\]/$button_style/g;

		print $handle $_;
    	}
    	close TEMPLATE;	    
}

# add or remove the ssl include line from the admserv srm.conf file
sub admserv_set_ssl_include
{
    my $state = shift;

    my $conf_file = "$AdminConf/srm.conf";
    local *IN;
    local *OUT;

    FILE_lock_rw($conf_file, \*IN, \*OUT);

    # always reset or fastcgi will bite us
    my $found = 0;
    while (<IN>)
    {
        if (/^Include conf\/ssl.conf$/)
        {
            $found = 1;
            if ($state) { print OUT $_; }
        }
        else
        {
            print OUT $_;
        }
    }

    if ($state && !$found)
    {
        print OUT "Include conf/ssl.conf\n";
    }

    FILE_unlock_rw($conf_file, \*IN, \*OUT);
    chmod 0644, $conf_file;
}

1;
