#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 1.0
# date: 7-6-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteUserAdd;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Meta;
use Cobalt::Util;
use Cobalt::Vsite;

use DB_File;
use Fcntl qw(O_RDONLY O_RDWR O_CREAT);

# global variables
my ($q, $locale, $group);
my $siteUserAddScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserAdd";
my $siteUserListScriptURL;

sub main {

$q=shift;

# the page that run this script
my $page=$q->param( 'page' );

my $forbiddenURL="/.cobalt/error/forbidden.html";

my $info = $ENV{PARAM_INFO} ? $ENV{PARAM_INFO} : $q->param('info');

if($ENV{'PARAM_GROUP'}) { 
	# We're being called form the command line in this case.
	# No security checking needed.
    $group=$ENV{'PARAM_GROUP'}; 
}
else { 	
    # .htaccess already demands an site-adm group user.
    $group=$q->param('group'); 
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "site-adm", $group)) {
	print "location: $forbiddenURL\n\n";    
	die "Unauthorised access by $user, not a member of $group.\n";
    } 
}

if (!$group && @ARGV) {
    $group=$ARGV[0];
    $page='';
}

$siteUserListScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserList?group=$group";

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

if ($page eq 'siteuseradd') {
    handleSiteUserAddPage($q);
} else {

    # Enforce site user counts
    my @vsiteUsers = vsite_get_users($group);
    my $siteCurUsers = scalar(@vsiteUsers);

    @vo = Cobalt::Meta::query("type"  => "vsite",
			      "keys"  => ["maxusers"],
			      "where" => ["name", "=", "$group"]);
    my $siteMaxUsers = $vo[0];
    if ($siteCurUsers >= $siteMaxUsers) {
	$info='';
	$info.=MSG_get_ui("list_max_siteuser");
	print "location: $siteUserListScriptURL" . "&info=$info\n\n";
        return 0;
    }

    genSiteUserAddPage (\*STDOUT, $group, $info);
}

}

#
# event handler for add user page
#
sub handleSiteUserAddPage {
    my $q = shift;
    my $shell = ($q->param('shell') eq 'on') ? 'on' : 'off';
    my $apop = ($q->param('apop') eq 'on') ? 'on' : 'off';
    my $admin = ($q->param('siteadmin') eq 'on') ? 'on' : 'off';
    my $suspend = ($q->param('suspend') eq 'on') ? 'on' : 'off';
    my $fpx = ($q->param('fpx') eq 'on') ? 'on' : 'off';

    my $obj = Cobalt::Meta->new("type" => "users");
    $obj->put("name", $q->param('username'));
    $obj->put("fullname", $q->param('fullname'));
    $obj->put("altname", $q->param('altname'));
    $obj->put("password1", $q->param('password1'));
    $obj->put("password2", $q->param('password2'));
    $obj->put("quota", $q->param('diskquota'));
    $obj->put("shell", $shell);
    $obj->put("admin", $admin);
    $obj->put("aliases", $q->param('aliases'));
    $obj->put("group", $q->param('group'));
    $obj->put("fpx", $fpx);
    $obj->put("apop", $apop);
    $obj->put("suspend", $suspend);
    $obj->put("forward", '');
    $obj->put("vacation", 'off');

    my ($ret, $success)  = site_user_add($obj);

    my $fn_enc = $locale->getProperty("fullname_enc");
    if ($fn_enc) {
	Cobalt::User::user_fullname_enc($obj->getname(),$fn_enc);
    }

    my $info;
    if ($ret && !$success) {	# generate error message
	$info=$ret;
	print "location: $siteUserAddScriptURL\?group=$group&info=$info\n\n";
    } else {
	if ($ret) {
		$ret = MSG_get_ui('addUserSuccess') . '<BR>' . $ret;
	}
	print "location: $siteUserListScriptURL&info=$ret\n\n";
    }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the information shown to user
#
sub genSiteUserAddPage {
    my ($handle, $group, $info) = @_;

    my $vsiteobj=Cobalt::Meta->new("type" => "vsite");
    $vsiteobj->retrieve($group);

    my $diskQuota = $vsiteobj->get("user_quota");
    my $disabled = MSG_get_ui("disabled");
    my $suspended = MSG_get_ui("suspended.unavailable");

    my($fpxRow)='<INPUT TYPE=CHECKBOX NAME="fpx" VALUE="on" CHECKED>';
    if ($vsiteobj->get('suspend') eq 'on') {
	$fpxRow=$suspended;
    } elsif ($vsiteobj->get("fpx") eq 'on') {
	if ($vsiteobj->get("user_fpx") ne 'on') {
	    $fpxRow =~ s/CHECKED//g;
	}
    } else {
	$fpxRow=$disabled;
    }

    my($shellRow)='<INPUT TYPE="CHECKBOX" NAME="shell" CHECKED>';
    if ($vsiteobj->get("shell") eq 'on') {
	if ($vsiteobj->get("user_shell") ne 'on') {
	    $shellRow =~ s/CHECKED//;
	}
    } else {
        $shellRow=$disabled;
    }	


    my ($apopRow)='<INPUT TYPE="CHECKBOX" NAME="apop" CHECKED>';
    if ($vsiteobj->get("apop") eq 'on') {
	if ($vsiteobj->get("user_apop") ne 'on') {
	    $apopRow =~ s/CHECKED//;
	}
    } else {
        $apopRow=$disabled;
    }	

    my ($genNameFunction,$nameGenMode);
    if ($locale->getProperty('genUserName')) {
	$nameGenMode = $vsiteobj->get("user_namegen");
	if ($nameGenMode == 1) {
	    $genNameFunction='genFieldsLast(this.value);';
	} elsif ($nameGenMode == 2) {
	    $genNameFunction='genFieldsFirst(this.value);';
	} else {
	    $genNameFunction='genFieldsFirstInitLast(this.value);';
	}
    }

    my $genAliasFunction='';
    if ($locale->getProperty('genEmailAlias')) {
	$genAliasFunction='genEmailAlias(this.value);';
    }

    my ($begin_altname,$end_altname);
    if ($locale->getProperty('altname')) {
	$begin_altname = "-->";
	$end_altname = "<!--";
    }

    # Read in the template
    open (TEMPLATE, "$Template_dir/siteUserAdd.tmpl")
	|| die "Cannot read $Template_dir/siteUserAdd.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
	s/\[BEGIN_ALTNAME\]/$begin_altname/;
	s/\[END_ALTNAME\]/$end_altname/;
	s/\[DISKQUOTA\]/$diskQuota/;
	s/\[GENALIASFUNCTION\]/$genAliasFunction/;
	s/\[GENNAMEFUNCTION\]/$genNameFunction/;
	s/\[GROUP\]/$group/;
	s/\[MAXNAMELEN\]/$MaxNameLen/;
	s/\[SHELL\]/$shellRow/;
	s/\[SITEUSERADDSCRIPTURL\]/$siteUserAddScriptURL/o;
	s/\[SITEUSERLISTSCRIPTURL\]/$siteUserListScriptURL/;
	s/\[USER_INFO\]/$info/;
	s/\[FPX\]/$fpxRow/;
	s/\[APOP\]/$apopRow/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/ge;
	print $handle $_;
    }
    close TEMPLATE;	    
}
1;
