#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Microserver
# version: 2.0
# date: 7-6-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteUserDefaults;

use Cobalt::Product;
use Cobalt::Group;
use Cobalt::Locale;
use Cobalt::Vsite;
use Cobalt::User;
use Cobalt::Meta;
use Cobalt::Util;

use Fcntl qw(O_RDWR O_CREAT);
# use strict;
use Locale::gettext;

# global variables
my ($q,$locale, $group);
my $siteUserDefaultsScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserDefaults";
my $siteUserListScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserList";
my $forbiddenURL='/.cobalt/error/forbidden.html';

sub main {

$q=shift;

my $page=$q->param( 'page' );

if( $ENV{'PARAM_GROUP'} ) {
    $group=$ENV{'PARAM_GROUP'};
}
else {
    $group=$q->param('group');
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "site-adm", $group)) {
	print "location: $forbiddenURL\n\n";    
	die "Unauthorised access by $user, not a member of $group.\n";
    } 
}

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

if ($group && ( $page ne 'userdefaults' )) {
    genSiteUserDefaultsPage( \*STDOUT );
}
else { 
    handleSiteUserDefaults(); 
}

}


#
# event handler for site user defaults page
#
sub handleSiteUserDefaults {
    my $diskQuota=$q->param( 'diskquota' );

    my $obj=Cobalt::Meta->create("type" => "vsite");
    $obj->retrieve($group);

    my $shell=$q->param('shell');
    my $nameGenMode=$q->param('namegenmode');
    my $apop=$q->param('apop');
    my $fpx=$q->param('fpx');
    my $range=$q->param('range');
    
    ($shell eq 'on') ? ($shell = 'on') : ($shell = 'off');
    ($apop eq 'on') ? ($apop = 'on') : ($apop = 'off');
    ($fpx eq 'on') ? ($fpx = 'on') : ($fpx = 'off');
    
    my $info='';
    my $err=0;
    my $siteDiskQuota=$obj->get("quota");

    if ($range < 1) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "range_invalid")."\"") . '<BR>';
	$err=1;
    }
    # check disk quota
    if( $diskQuota!~/^\d+$/o || $diskQuota==0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"") . '<BR>';
	$err=1;
    }
    elsif( $diskQuota>$siteDiskQuota ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "userDiskQuotaTooBig")."\"") . '<BR>';
	$err=1;
    }
    else {
	$obj->put("user_quota", $diskQuota);
    }

    unless ($err) {
	$obj->put("user_shell"     => $shell,
		  "user_apop"      => $apop,
		  "user_namegen"   => $nameGenMode,
		  "user_fpx"       => $fpx,
		  "userlist_range" => $range);
	$obj->save;

	print "location: $siteUserListScriptURL?group=$group\n\n";
    } else {
	genSiteUserDefaultsPage( \*STDOUT, $info );
    }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the information shown to user
#
sub genSiteUserDefaultsPage {
    my ( $handle, $info )=@_;

    my $disabled=eval("\"".Locale::gettext::dgettext("ui", "disabled")."\"");
    # read defaults
    my $obj=Cobalt::Meta->create("type" => "vsite");
    $obj->retrieve($group);

    my $diskQuota=$obj->get("user_quota");

    my ($fpxRow)='<INPUT TYPE=CHECKBOX NAME="fpx" VALUE="on" CHECKED>';
    if (($obj->get("fpx") eq 'on') && ($obj->get("user_fpx") ne 'on')) {
	$fpxRow =~ s/CHECKED//;
    } elsif (($obj->get("fpx") ne 'on')) {
	$fpxRow = $disabled;
    }

    my ($shellRow)='<INPUT TYPE=CHECKBOX NAME="shell" VALUE="on" CHECKED>';
    if (($obj->get("shell") eq 'on') && ($obj->get("user_shell") ne 'on')) {
	$shellRow =~ s/CHECKED//;
    } elsif (($obj->get("shell") ne 'on')) {
	$shellRow = $disabled;
    }

    my ($apopRow)='<INPUT TYPE=CHECKBOX NAME="apop" VALUE="on" CHECKED>';
    if (($obj->get("apop") eq 'on') && ($obj->get("user_apop") ne 'on')) {
	$apopRow =~ s/CHECKED//;
    } elsif (($obj->get("apop") ne 'on')) {
	$apopRow = $disabled;
    }

    my ($rangeRow)='<INPUT TYPE=TEXT SIZE=5 NAME="range" VALUE="' .
	$obj->get("userlist_range") . '">';

    my ($sortRow)='<SELECT NAME="sort">' .
	'<OPTION VALUE="hostname">' . MSG_get_ui("sort_hostname") . '</OPTION>' .
	'<OPTION VALUE="fullname">' . MSG_get_ui("sort_fullname") . '</OPTION>' .
	'</SELECT>';

    my $genUserName=$locale->getProperty( 'genUserName' );
    my ($mode0Checked,$mode1Checked,$mode2Checked);
    my $hideGenStart='<!-- ';
    my $hideGenEnd=' -->';
    if ($genUserName) {
	$hideGenStart='';
	$hideGenEnd='';

	my $nameGenMode=$obj->get("user_namegen");

	# mode 0 is the default
	if ($nameGenMode==1) {
	    $mode1Checked='CHECKED';
	} elsif ($nameGenMode==2) {
	    $mode2Checked='CHECKED';
	} else {
	    $mode0Checked='CHECKED';
	}
    }

    if ($info) {
	$info = "'$info', 'error'";
    } else {
	$info = "'', 'status'";
    }

    # Read in the template
    open (TEMPLATE, "$Template_dir/siteUserDefaults.tmpl")
	|| die "Cannot read $Template_dir/siteUserDefaults.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
	s/\[DISKQUOTA\]/$diskQuota/;
	s/\[GROUP\]/$group/;
	s/\[HIDEGENEND\]/$hideGenEnd/;
	s/\[HIDEGENSTART\]/$hideGenStart/;
	if( $genUserName ) {
	    s/\[MODE0CHECKED\]/$mode0Checked/;
	    s/\[MODE1CHECKED\]/$mode1Checked/;
	    s/\[MODE2CHECKED\]/$mode2Checked/;
	}
	# s/\[SHELLCHECKED\]/$shellChecked/;
	s/\[SHELL\]/$shellRow/;
	s/\[RANGE\]/$rangeRow/;
	s/\[SORT\]/$sortRow/;
	s/\[SITEUSERDEFAULTSSCRIPTURL\]/$siteUserDefaultsScriptURL/o;
	s/\[SITEUSERLISTSCRIPTURL\]/$siteUserListScriptURL/o;
	s/\[USER_INFO\]/$info/;
	s/\[FPX\]/$fpxRow/;
	s/\[APOP\]/$apopRow/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print $handle $_;
    }
    close TEMPLATE;
}
1;
