#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 2.0
# date: 7-6-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteUserDel;

use Cobalt::Product;
use Cobalt::Email;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Vacation;
use Cobalt::Fpx;
use Cobalt::Http;
use Cobalt::Meta;
use Quota;
use FileHandle;
use Cobalt::Util qw( update_status );

use strict;

# global variables
use vars qw( $DEBUG );
my ($q, $userName, $group, $locale);
my $siteUserDelScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserDel';
my $siteUserListScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserList';
my $forbiddenURL="/.cobalt/error/forbidden.html";
my $log_fh;

sub main {

$DEBUG = 0;
$q=shift;
$userName=$q->param('name');
$group=$q->param('group');

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

# force login
my $loginName=$ENV{ REMOTE_USER };

if( !$loginName ) {
    # ask user to enter user name
    print "location: $siteUserDelScriptURL?username=$userName&group=$group\n\n";
} elsif (not user_authok($loginName, "site-adm", $group)) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorised access by $loginName, not a member of $group.\n";
}

my $uid = getpwnam($userName);
my ($blocks, $files) = (Quota::query(Quota::getqcarg('/home'), $uid))[0, 4];

if ($DEBUG) {
	print STDERR "$userName\'s disk usage is $blocks blocks and $files files\n";
}

Cobalt::Util::lock_ui(1);

# did some experimentation and came up with these numbers
# this should force every user deletion that will take approximately 15 sec.
# or more to fork off
if (($files > 1000 && $blocks > 327680) || $blocks > 409600) {
	# need to fork off actual deletion to the background
	
	$log_fh = new FileHandle(">/home/sites/$group/.userdelstat");
	my $old_fh = select $log_fh;
	$| = 1;
	select $old_fh;


	my $pid;
	
	$SIG{CHLD} = "IGNORE";
	$CGI::Fast::fcgi_request->Detach();

	if ($pid = fork) {
		# parent
		$CGI::Fast::fcgi_request->Attach();
		print STDERR "PID: $pid\n" if ($DEBUG);
		
		update_status($log_fh, [$pid, $userName, 'START']);
		
		$log_fh->close();
		print $q->redirect($siteUserListScriptURL . "?group=$group&");
		
		# parent needs to exit because of mod_fastcgi issues
		exit 0;
		
	} elsif ($pid == 0) {
		# child

		# reconnect to database just to be sure
		my $dbc = Cobalt::Meta::getdb;
		$dbc->reset;

		update_status($log_fh, [$$, $userName, 'START']);

		close STDIN;
		close STDOUT;
		close STDERR;

		handleSiteUserDelPage($q, 1);

		$log_fh->close();

		# exit because we are no longer connected to the server
		Cobalt::Util::lock_ui(0);
		exit 0;

	} elsif ($pid != 0) {
		# fork failed
		my $info = "foo";

		print "Location: $siteUserListScriptURL?group=$group&info=$info\n\n";
	}

} else {
	handleSiteUserDelPage($q, 0);
	Cobalt::Util::lock_ui(0);
}

}

#
# event handler for delete user page
#
sub handleSiteUserDelPage {
    my $cgi = shift;
    my $forked = shift;
    my $err = 0;
    my $info;

    # Remove Fpx user web
    # This must be run before user_del since fpx_get_web checks user existance
    if ($Features{'fpx'}) {
        if (fpx_get_web($userName)) {
            update_status($log_fh, [$$, $userName, 'FPX']) if ($forked);
	    fpx_set_web($userName, 0);
	}
	# return value is ignored, a failed fpx user web deletion is not 
	# critical to system functionality
    }

    update_status($log_fh, [$$, $userName, 'SYSTEM']) if ($forked);
    my $ret=user_del( $userName );
    if( $ret!~/^2/ ) {
	$info = substr( $ret, 4 ).'<BR>';
	update_status($log_fh, [$$, $userName, 'ERROR', $info]);
	# $err=0;
    }

    if( !$err ) {
	update_status($log_fh, [$$, $userName, 'EMAIL']) if ($forked);
	# remove email forward address if necessary
        if( alias_exist( $userName ) ) {
            $ret=alias_delete( $userName );
	    if( $ret!~/^2/ ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	# remove email aliases
	$ret=mail_virtuser_set_byuser( $userName );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# turn off vacation
	$ret=vacation_set_on( $userName, 0 );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
	# no vacation message
	$ret=vacation_set_message( $userName, '' );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    update_status($log_fh, [$$, $userName, 'DBASE']) if ($forked);
    # remove the user from the database
    my $uobj=Cobalt::Meta->create("type" => "users");
    $uobj->retrieve($userName);
    $uobj->delete;

     
    # remove the user from the defcon table if needed
    if (scalar(Cobalt::Meta::query("type" => "defcon",
                                   "keys" => ["name"],
                                   "where" => ["name", "=", "$userName"]))) {

        my $dobj=Cobalt::Meta->create("type" => "defcon");
        $dobj->retrieve($userName);
        $dobj->delete;
    }              

    if( $err ) {
	if ($forked) {
		update_status($log_fh, [$$, $userName, 'ERROR', $info]) 
        } else {
		print "location: $siteUserListScriptURL?group=$group&info=$info\n\n";
	}
    }
    else {
	if ($forked) {
		update_status($log_fh, [$$, $userName, 'DONE']);
	} else {
        	print "location: $siteUserListScriptURL?group=$group\n\n";
	}
    }

    # is this even necessary?
    Cobalt::Http::http_restart(1);
}

1;
