#!/usr/bin/perl
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::siteUserMod;

use Cobalt::Product;
use Cobalt::Group;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Vsite;
use Cobalt::Email;
use Cobalt::Fpx;
use Cobalt::Http;
use Cobalt::Meta;

# use strict;
use CGI;
use DB_File;
use Fcntl qw(O_RDONLY);
use Locale::gettext;

my ($q,$page,$group,$locale,$userName);
my $forbiddenURL="/.cobalt/error/forbidden.html";
my $siteUserListScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserList';
my $siteUserModScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/siteUserMod';


sub main {

$q=shift;
$page=$q->param( 'page' );

if( $ENV{ 'PARAM_GROUP' } ) { $group=$ENV{ 'PARAM_GROUP' }; }
else {
	$group=$q->param( 'group' );
	my $user = $ENV{REMOTE_USER};
	unless (user_authok($user, "site-adm", $group)) {
	    print "location: $forbiddenURL\n\n";    
	    die "Unauthorised access by $user, not a member of $group.\n";
	}
	# Make sure that you are changing a user in the
	# same group as the site-admin
	unless (user_authok($q->param('username'), "group", $group)) {
		print "location: $forbiddenURL\n\n";
		die "Unauthorised access by $user, not a member of $group.\n";
	}
	# As an extra check this script should NEVER be used to get at the
	# admin user.
	if ($q->param('username') eq $AdminUser) {
	    print "location: $forbiddenURL\n\n";    
	    die "Unauthorised attempt to modify the admin user using siteUserMod.cgi";
	}
}


$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

$userName=$q->param('username');

if( !$ENV{ REMOTE_USER } ) {
    print "location: $siteUserModScriptURL?username=$userName&page=$page\n\n";
}
else {
    if( $page eq 'usermod' ) { handleSiteUserModPage(); }
    else {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	genSiteUserModPage( \*STDOUT, $group, $userName );
    }
}

}


#
# event handler for modify user page
#
sub handleSiteUserModPage {
    my $err=0;
    my $info;

    # read form input
    my $userName=$q->param( 'username' );
    my $fullName=$q->param( 'fullname' );
    my $altname=$q->param( 'altname' );
    my $password1=$q->param( 'password1' );
    my $password2=$q->param( 'password2' );
    my $quota=$q->param( 'diskquota' );
    my $siteAdmin=$q->param( 'siteadmin' );
    my $group=$q->param( 'group' );
    my $suspend=$q->param('suspend');

    my $vobj=Cobalt::Meta->create("type" => "vsite");
    $vobj->retrieve($group);

    my $shellAccess= $vobj->get('shell') eq 'on' ? $q->param( 'shell' ) :
        undef;
    my $fpx= $vobj->get('fpx') eq 'on' ? $q->param( 'fpx' ) : undef;
    my $apop= $vobj->get('apop') eq 'on' ? $q->param('apop') : undef;

    $fpx = $fpx ? 'on' : 'off';
    $shellAccess = $shellAccess ? 'on' : 'off';
    $siteAdmin = $siteAdmin ? 'on' : 'off';
    $apop = $apop ? 'on' : 'off';
    $suspend = $suspend ? 'on' : 'off';

    my $uobj=Cobalt::Meta->create("type" => "users");
    $uobj->retrieve($userName);
    my $vobj=Cobalt::Meta->create("type" => "vsite");
    $vobj->retrieve($group);

    my $apopold=$uobj->get("apop");
    my $fpxold=$uobj->get("fpx");
    my $groupDiskQuota=$vobj->get("quota");
    my $usedSpace = (user_show($userName))[3]*$BytesPerBlock/1048576 - 1;

    # error check
    my @passwordInBytes=unpack( 'C*', $password1 );
    my $passwordBadChar=0;
    my $byte;
    foreach $byte ( @passwordInBytes ) {
	if( $byte>127 ) { $passwordBadChar=1; last; }
    }

    if( !$fullName ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "noFullName")."\"");
	$err=1;
    }
    elsif( $password1 ne $password2 ) {
	$info.='Passwords entered in the password fields does not match.';
	$err=1;
    }
    elsif( $passwordBadChar ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "passwordBadChar")."\"");
	$err=1;
    }
    elsif( $userName!~/^[a-z0-9][a-z0-9\.\-\_]*$/o ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "userNameNotValid")."\"");
	$err=1;
    }
    elsif( $fullName=~/\"/o ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "noQuotesInFullName")."\"");
	$err=1;
    }
    elsif( $quota!~/^\d+$/o || ( $userName ne $Adminuser && $quota==0 ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"");
	$err=1;
    }
    elsif( $quota>$groupDiskQuota ) {
	# for the message
	my $siteDiskQuota=$groupDiskQuota;
	$info.=eval("\"".Locale::gettext::dgettext("ui", "userDiskQuotaTooBig")."\"");
	$err=1;
    }
	elsif(($fpx eq 'on') and ($quota - $usedSpace < $Fpx_minquota*($fpxold eq 'on' ? 0 : 1))) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotBigEnough")."\"");
	$err=1;
	}
    elsif ($apop eq "on" && $apopold ne "on" && $password1 eq '') {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "apopPasswordNoChange")."\"");
	$err=1;
    }
    else {
	# everything is fine

	my $ret;
	my $shell=$Usershell;
	unless ($shellAccess eq 'on') { $shell=$Badshell; }

	# convert from megabyte to block
	my $diskQuota = $quota*1048576/$BytesPerBlock;

	if ($suspend eq 'off') {
	    $ret=user_enable ($userName);
	    if ($ret !~ /^2/o) {
		$info.=substr ($ret, 4).'<BR>';
		$err=1;
	    }
	}

	$ret=user_mod( $userName, $fullName, $password1, $diskQuota, $shell ); 
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# set site admin
	if ($siteAdmin eq 'on') {
	    $ret=vsite_admin_add( $group, $userName );
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}
	else {
	    $ret=vsite_admin_del( $group, $userName );
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 ).'<BR>';
		$err=1;
	    }
	}

	if ($fpx eq 'on') {
            my($ret)=fpx_set_web($userName,1);
            if ($ret !~ /^2/) {
                $info.=substr( $ret, 4 ).'<BR>';
                $err=1;
            }
        } 
	elsif (fpx_get_web($userName)) {
            my($ret)=fpx_set_web($userName,0);
            if ($ret !~ /^2/) {
                $info.=substr( $ret, 4 ).'<BR>';
                $err=1;
            }
        }
 
	if (($apop eq 'on') && ($apopold ne 'on' || $password1 ne '')) {
	    $ret=mail_apop_adduser ($userName, $password1);
	    if ($ret !~ /^2/o) {
		$info.=substr ($ret, 4).'<BR>';
		$err=1;
	    }
	} elsif ($apop ne 'on' && $apopold eq 'on') {
	    $ret=mail_apop_deluser ($userName);
	    if ($ret !~ /^2/o) {
		$info.=substr ($ret, 4).'<BR>';
		$err=1;
	    }
	}

	if ($suspend eq 'on') {
	    $ret=user_disable ($userName);
	    if ($ret !~ /^2/o) {
		$info.=substr ($ret, 4).'<BR>';
		$err=1;
	    }
	}
    }

    if( $err ) {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

	genSiteUserModPage(\*STDOUT, $group, $userName, $info);
    }
    else {
	$uobj->put("name"      => $userName,
		   "fullname"  => $fullName,
		   "altname"   => $altname,
		   "quota"     => $quota,
		   "shell"     => $shellAccess,
		   "admin"     => $siteAdmin,
		   "fpx"       => $fpx,
		   "apop"      => $apop,
		   "suspend"   => $suspend);
	$uobj->save;
	
	my $fn_enc = $locale->getProperty("fullname_enc");
	if ($fn_enc) {
	    my $ret=Cobalt::User::user_fullname_enc($userName,$fn_enc);
	}

	print "location: $siteUserListScriptURL\?group=$group\n\n";
    }

    Cobalt::Http::http_restart(1);
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the infomation shown to user
#
sub genSiteUserModPage {
    my ( $handle, $group, $userName, $info )=@_;

    my $uobj=Cobalt::Meta->create("type" => "users");
    $uobj->retrieve($userName);

    my $vobj=Cobalt::Meta->create("type" => "vsite");
    $vobj->retrieve($group);

    my $fullName = $uobj->get("fullname");
    my $diskQuota = $uobj->get("quota");
    my $shell = $uobj->get("shell");

    my $disabled = eval("\"".Locale::gettext::dgettext("ui", "disabled")."\"");
    my $suspended_enabled = eval("\"".Locale::gettext::dgettext("ui", "suspended.unavailable.enabled")."\"");
    my $suspended_disabled = eval("\"".Locale::gettext::dgettext("ui", "suspended.unavailable.disabled")."\"");

    # Shell account status
    my $shellRow='<INPUT TYPE=CHECKBOX NAME="shell" CHECKED>';
    my $shellChecked='';
    if ($shell ne 'on') {
	$shellRow =~ s/CHECKED//o;
    }
    if ($vobj->get("shell") ne 'on') {
	$shellRow=$disabled;
    }

    # FrontPage web status
    # If the user has an FPX web, always allow them to disable it.
    # If the site has an FPX root web, allow the admin to enable user web
    # "Disabled" if no existing user or site webs
    my $fpxRow='<INPUT TYPE=CHECKBOX NAME="fpx" VALUE="on" CHECKED>';
    if ($uobj->get("fpx") ne 'on') {
	$fpxRow =~ s/CHECKED//o;
    }
    if ($vobj->get('suspend') eq 'on') {
	if($uobj->get("fpx") eq 'on') {
            $fpxRow=$suspended_enabled.'<INPUT TYPE=HIDDEN SIZE=-1 NAME="fpx" VALUE="on">';
	} else {
            $fpxRow=$suspended_disabled;
	}
    } elsif ($vobj->get("fpx") ne 'on') {
	$fpxRow=$disabled;
    }

    # apop
    my $apopRow = '<INPUT TYPE=CHECKBOX NAME="apop" VALUE="on" CHECKED>';
    if ($uobj->get("apop") ne 'on') {
	$apopRow =~ s/CHECKED//o;
    }
    if ($vobj->get("apop") ne 'on') {
	$apopRow=$disabled;
    }

    # suspend
    my $suspendRow = '<INPUT TYPE=CHECKBOX NAME="suspend" VALUE="on" CHECKED>';
    if ($uobj->get("suspend") ne 'on') {
	$suspendRow =~ s/CHECKED//o;
    }

    my $adminRow = '<INPUT TYPE=CHECKBOX NAME="siteadmin" VALUE="on" CHECKED>';
    if ($uobj->get("admin") ne 'on') {
	$adminRow =~ s/CHECKED//o;
    }

    my ($altname,$begin_altname,$end_altname);
    if ($locale->getProperty('altname')) {
	my $alt = $uobj->get('altname');
	$altname = "$alt";
	$begin_altname = "-->";
	$end_altname = "<!--";
    }

    # Read in the template
    open( TEMPLATE, "$Template_dir/siteUserMod.tmpl" )
	|| die "Cannot read $Template_dir/siteUserMod.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[BEGIN_ALTNAME\]/$begin_altname/;
	s/\[END_ALTNAME\]/$end_altname/;
	s/\[ALTNAME\]/$altname/;
	s/\[DISKQUOTA\]/$diskQuota/;
	s/\[FULLNAME\]/$fullName/;
	s/\[GROUP\]/$group/;
	s/\[SHELL\]/$shellRow/;
	s/\[SITEADMIN\]/$adminRow/;
	s/\[SITEUSERLISTSCRIPTURL\]/$siteUserListScriptURL/o;
	s/\[SITEUSERMODSCRIPTURL\]/$siteUserModScriptURL/o;
	s/\[USER_INFO\]/$info/;
	s/\[USERNAME\]/$userName/g;
	s/\[FPX\]/$fpxRow/g;
	s/\[APOP\]/$apopRow/g;
	s/\[SUSPEND\]/$suspendRow/g;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print $handle $_;
    }
    close TEMPLATE;
}
1;
