#!/usr/bin/perl
#
# Cobalt Site Backup, Edition IV
# Will DeHaan, will@cobalt.com/cobalt@guck.net
# (C) 2000 Cobalt Networks, Inc., http://cobalt.com/, 2000
#
# site_backup.pl scheduling interface

package Cobalt::UI::sitebackupmenu;

use strict;
use Cobalt::Locale;
use Cobalt::Vsite;
use Cobalt::Product;
use Cobalt::Netutil;
use Locale::gettext;
use Cobalt::User;
use lib qw( /usr/sausalito/perl );
use Sauce::Util;
use Cobalt::UI qw( get_style_info );

use vars qw( $DEBUG );

my $iam = '/cgi-bin/.cobalt/turbo_ui/dispatch/sitebackupmenu';

my $cronFile = '/etc/cron.d/site-backups';

my $backup_dir = '/etc/cobalt/site-backups';
mkdir($backup_dir, 0700) unless (-d $backup_dir);

# Remember scheduled backups
my ( $cron_daily, $cron_weekly, $cron_monthly, $cron_never ) = 
	( "$backup_dir/daily", "$backup_dir/weekly", "$backup_dir/monthly", $backup_dir );

sub main {

$DEBUG = 0;

# Read & parse
my $q = shift; # CGI fields
my %fields;
my @web_data = ('change', 'protocol', 'frequency', 
                'smb_share', 'smb_username', 'smb_passwd', 
                'ftp_share', 'ftp_username', 'ftp_passwd', 
                'nfs_share');
my $key;
foreach $key (@web_data) { $fields{$key} = $q->param($key); }

###############
# who ami I? 
my $user = $ENV{'REMOTE_USER'};
my ($user_id, $user_dir) = (getpwnam($user))[2,7] if (($user =~ /^[a-z0-9][a-z0-9\.\-\_]+$/) && (length($user) < 32));
my $site = Cobalt::Vsite::vsite_getgroup($user);
# need to determine current site if we're admin|alteradmin
$site ||= $q->param('group') if (($user eq $Adminuser) || ($user eq $Alteradmin));
die "Security Violation: No site recognized for site backup by user $user\n" unless ($site);

my $forbiddenURL="/.cobalt/error/forbidden.html";
unless (user_authok($user, "site-adm", $site)) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

# Fetch site data
# array of (ip,hostname,group,webmaster email,subnet) or 0
my (@site_data) = Cobalt::Vsite::vsite_get_bygroup($site);

unless(-d "$Userdir/$site") { 
	die "Site directory can not be found at $Userdir/$site.  Aborting.\n";
}

###############
# test for & read scheduled backup
my ($scheduled, $schedule_script, $protocol, $remote_host, $user, $remote_user, $remote_path, $remote_password);
$schedule_script = "$cron_daily/$site" if (-r "$cron_daily/$site");
$schedule_script = "$cron_weekly/$site" if (-r "$cron_weekly/$site");
$schedule_script = "$cron_monthly/$site" if (-r "$cron_monthly/$site");
$schedule_script = "$cron_never/$site" if (-r "$cron_never/$site");
my $custom;
if(-r $schedule_script && !$q->param('udays')) {
	my $user_code = 0;
	open(SCHDL, $schedule_script) || die "Could not read schedule cron job $schedule_script as root: $!";
	while(<SCHDL>) {
		chop;
		if($user_code) {
			$custom .= $_;
		} elsif(/^# USER: (.+)$/) {
			$user = $1;
		} elsif (/^# PROTOCOL: (.+)$/) { # either FTP NFS or SMB
			$protocol = $1;
		} elsif (/^# HOST: (.+)$/) { # simple fqdn
			$remote_host = $1;
		} elsif (/^# REMOTE_USER: (.+)$/) { # simple remote username
			$remote_user = $1;
		} elsif (/^# REMOTE_PATH: (.+)$/) { # path
			$remote_path = $1;
		} elsif (/^# PASSWORD: (.+)$/) { # path
			$remote_password = $1;
		} elsif (/^# FIELD: ([a-z\_]+): (.*)$/) {
			$fields{$1} = $2 unless ($fields{'change'});
		} elsif (/below this line/) {
			$user_code = 1;
		}
	}
	close(SCHDL);
}

$fields{'protocol'} ||= $protocol;

###############
# reschedule if you're into it
if ($fields{'change'} eq '1' && !$q->param('udays')) { # test and save new cron job
	&verify_data(%fields);

	# delete our old scheduled backup
	unlink($schedule_script);

	print STDERR $fields{'frequency'}, "\n" if ($DEBUG);

	# remove scheduled backup from cron job
	# and delete the previous cron file unless there is an error in editing
	if ($fields{'frequency'} =~ /^[ni]/) {
		Sauce::Util::editfile($cronFile, \&edit_cron, 
				$schedule_script, $q, 0) && unlink("$cronFile~");
	}

	if($fields{'frequency'}) {
	
		# Render host, user and path
		my ($remote_password);
		if ($fields{'protocol'} eq 'FTP') {
			($remote_host, $remote_user, $remote_password) = 
			  ($fields{'ftp_share'}, $fields{'ftp_username'}, $fields{'ftp_passwd'});
			$remote_host =~ s/^([^\/]+)(\/.*)$/$1/; # split off path
			$remote_path = $2; 
		} elsif ($fields{'protocol'} eq 'SMB') {
			($remote_host, $remote_user, $remote_password) = 
			  ($fields{'smb_share'}, $fields{'smb_username'}, $fields{'smb_passwd'});
			$remote_host =~ s/^\\\\([^\\]+)\\(.*)$/$1/; # split off path
			$remote_path = $2; 
		} elsif ($fields{'protocol'} eq 'NFS') {
			($remote_host) = 
			  ($fields{'nfs_share'});
			$remote_host =~ s/^([^:]+)(:.*)$/$1/; # split off path
			$remote_path = $2; 
		}
		
		# if remote path is blank set it to /
		if (not $remote_path) {
			$remote_path = '/';
		}
	
		$schedule_script = $backup_dir . '/';
		if ($fields{'frequency'} !~ /^n|^i/) {
			$schedule_script .= $fields{'frequency'} . '/';
		}
		$schedule_script .= $site;
		my $backup_cmd = "/usr/local/sbin/site_backup.pl $schedule_script"; # make site_backup parse the script for args
	
		# Cleanly unlink an old schedule file, avoid symlink silliness even though we're in /etc
		if((-e $schedule_script) && ($schedule_script =~ /$site$/)) {
			unlink($schedule_script);
			&error('[str.backup.can.not.unschedule]') if (-e $schedule_script); # dis the symlink hacks
		}

		# Write the cron job
		open(SCHED, "> $schedule_script") || die "Could not schedule backup $schedule_script: $!";
		print SCHED <<EOF;
#!/bin/sh
# 
# Scheduled backup for site $site
# Do not edit!  Changes will be overwritten by the user interface
#
# USER: $ENV{'REMOTE_USER'}
# PROTOCOL: $fields{'protocol'}
# HOST: $remote_host
# REMOTE_USER: $remote_user
# REMOTE_PATH: $remote_path
# PASSWORD: $remote_password
$backup_cmd

# GUI reporting:
# FIELD: smb_share: $fields{'smb_share'}
# FIELD: smb_username: $fields{'smb_username'}
# FIELD: ftp_share: $fields{'ftp_share'}
# FIELD: ftp_username: $fields{'ftp_username'}
# FIELD: nfs_share: $fields{'nfs_share'}
# FIELD: frequency: $fields{'frequency'}

# Please add customizations below this line:
$custom
EOF
		close(SCHED);
	
		chmod(0700, $schedule_script);
	}
	
	# add job to crontab file
	if ($fields{'frequency'} !~ /^[ni]/) {
		Sauce::Util::editfile($cronFile, \&edit_cron, 
				$schedule_script, $q, 1) && unlink("$cronFile~");
	} elsif ($fields{'frequency'} =~ /^i/) {
		# handle immediate backups
		$CGI::Fast::fcgi_request->Detach();

		if (fork) {
			open STDIN, "/dev/null";
			open STDOUT, ">/dev/null";
			open STDERR, ">/dev/null";

			exec "/bin/sh $schedule_script 2>&1 1>/dev/null";
			exit 0;
		}
		
		$CGI::Fast::fcgi_request->Attach();
	}
}

###############
# render filename, note TGZ kinda like .tar.gz.  Any contents will work! 
# headerless, like it should be
my($filename, $date_stamp);
chomp($date_stamp = `date +%m-%d-%Y`);
$filename = $site_data[1].'-'.$date_stamp.'.tgz'; # Concat so perl doesn't confuse $'s

###############
# obtain local acct usage
# FIXME this could easily time out
my $userUsage = `/usr/bin/du -sk $Userdir/$site/$Webdir 2>&1`;
my( $archiveMin, $archiveMax ) = (int($userUsage*10/$BytesPerBlock)/20, int($userUsage*10/$BytesPerBlock)/10);
$archiveMax ||= '1.0';
my $user_error = $q->param('USERERROR');

###############
# Report backup dataset
my $paths = "$Userdir/$site/$Webdir/\*";

###############
# Render pull-down frequency menu
my $frequency = $fields{'frequency'};
my $frequency_pulldown = <<EOF;
	<SELECT NAME="frequency" onChange="toggle_daytime()">
	<OPTION VALUE="never">[str.backup.never]</OPTION>
	<OPTION VALUE="immediate">[str.backup.immediate]</OPTION>
	<OPTION VALUE="daily">[str.backup.daily]</OPTION>
	<OPTION VALUE="weekly">[str.backup.weekly]</OPTION>
	<OPTION VALUE="monthly">[str.backup.monthly]</OPTION>
	</SELECT>
EOF
$frequency_pulldown =~ s#(\"$frequency\")#$1 SELECTED#;

my @cron_entry = ();
if ($q->param('udays')) {
	@cron_entry = (	$q->param('minute'),
			$q->param('hour'),
			1,
			'*',
			0 );
} else {
	@cron_entry = read_cron($cronFile, $site);
}

my $days = calculate_days($frequency, @cron_entry);
my $times = generate_times(@cron_entry);

my $style = get_style_info();

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

open(TMPL, "$Template_dir/sitebackup.tmpl") || die "Could not read template $Template_dir/personalbackup.tmpl: $!";
print STDERR "$ENV{LANGUAGE}\n"  if ($DEBUG);
#$ENV{LANGUAGE} = 'en_US';
while(<TMPL>) {
	s/\[IAM\]/$iam/;
	s/\[MIN\]/$archiveMin/;
	s/\[MAX\]/$archiveMax/;
	s/\[USERUSAGE\]/$userUsage/;
	s/\[BACKUPMAX\]/$MaxWebBackup/;
	s/\[USERERROR\]/$user_error/;
	s/\[ARCHIVE\]/$filename/;
	s/\[PATHS\]/$paths/;
	s/\[SITE\]/$site/;
	s/\[CHANGE\]/1/;
	s/\[DAYS\]/$days/;
	s/\[TIMES\]/$times/;
	
	s/(ocol\" VALUE=\"$fields{'protocol'}\")>/$1 CHECKED>/;

	# my ($scheduled, $schedule_script, $protocol, $remote_host, $user, $remote_user, $remote_path, $remote_password);
	s/\[SMBPATH\]/$fields{'smb_share'}/;
	s/\[SMBUSERNAME\]/$fields{'smb_username'}/;
	s/\[SMBPASSWORD\]//; # $fields{'smb_passwd'}
	s/\[FTPPATH\]/$fields{'ftp_share'}/;
	s/\[FTPUSERNAME\]/$fields{'ftp_username'}/;
	s/\[FTPPASSWORD\]//; # $fields{'ftp_passwd'}
	s/\[NFSPATH\]/$fields{'nfs_share'}/;
	s/\[FREQUENCY\]/$frequency_pulldown/;

	s/\[FONTH\]/$style->{header}/g;
	s/\[FONTT\]/$style->{table}/g;
	s/\[FONTB\]/$style->{button}/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	print;
}
close(TMPL);

###############
# fin main()
}


########################## Local Subs

sub error {
        my $user_error = shift;
        $user_error =~ s/\s/\+/g;
        print "Location: /cgi-bin/.cobalt/turbo_ui/dispatch/sitebackupmenu?USERERROR=$user_error\n\n";
        exit 0;
}

sub verify_data {
	my %fields = shift;
	my ($remote_host, $remote_user, $remote_password, $remote_path);

        if ($fields{'protocol'} eq 'FTP') {
                ($remote_host, $remote_user, $remote_password) =
                  ($fields{'ftp_share'}, $fields{'ftp_username'}, $fields{'ftp_passwd'});
                $remote_host =~ s/^([^\/]+)(\/.*)$/$1/; # split off path
                $remote_path = $2;

		&error('[str.backup.missing.ftp.host]') unless ($remote_host);
		&error('[str.backup.invalid.ftp.host]') unless (Cobalt::Netutil::netutil_test_domainname($remote_host));

        } elsif ($fields{'protocol'} eq 'SMB') {
                ($remote_host, $remote_user, $remote_password) =
                  ($fields{'smb_share'}, $fields{'smb_username'}, $fields{'smb_passwd'});
                $remote_host =~ s/^\\\\([^\\]+)\\(.*)$/$1/; # split off path
                $remote_path = $2;
		
		&error('[str.backup.missing.smb.host]') unless ($remote_host);
		&error('[str.backup.invalid.smb.host]') unless (Cobalt::Netutil::netutil_test_domainname($remote_host));
        } elsif ($fields{'protocol'} eq 'NFS') {
                ($remote_host) =
                  ($fields{'nfs_share'});
                $remote_host =~ s/^([^:]+):(.*)$/$1/; # split off path
                $remote_path = $2;
		
		&error('[str.backup.missing.nfs.host]') unless ($remote_host);
		&error('[str.backup.invalid.nfs.host]') unless (Cobalt::Netutil::netutil_test_domainname($remote_host));
        }
	return 1;
}

sub calculate_days
{
	my $frequency = shift;
	my @cron_entry = @_;

	my $selected;
	my $days = "<SELECT NAME=\"days\">\n";

	if (!$frequency || $frequency =~ /^n|^i/) {
		$days .= "<OPTION VALUE=\"FOO\">[str.notApplicable]</OPTION>\n";
	} elsif ($frequency =~ /^d/) {
		$days .= "<OPTION VALUE=\"FOO\">[str.backup.daily]</OPTION>\n";
	} elsif ($frequency =~ /^w/) {
		# FIXME should probably account for locale here to determine
		# the first day of the week
		my @days = (	'[str.sunday]',
				'[str.monday]',
				'[str.tuesday]',
				'[str.wednesday]',
				'[str.thursday]',
				'[str.friday]',
				'[str.saturday]'
			);

		$days .= gen_options(0, 6, @days);

		# set one day selected so the cron entry isn't bad
		$selected = @cron_entry ? $cron_entry[4] : 0;
		$days =~ s/(VALUE=\"$selected\")/$1 SELECTED/;
	} else {
		# monthly
		$days .= gen_options(1, 31);
	
		# set one day selected so the cron entry isn't bad
		$selected = @cron_entry ? $cron_entry[2] : 1;
		$days =~ s/(VALUE=\"$selected\")/$1 SELECTED/;
	}

	$days .= "</SELECT>\n";

	return $days;
}

sub generate_times
{
	my ($hour, $min);

	my $table = "<TABLE CELLPADDING=0 CELLSPACING=0>\n<TR>\n<TD>\n";

	# start the hours drop-down
	my $select = "<SELECT NAME=\"hour\">\n";
	
	$select .= gen_options(0, 23);

	# set time
	if (@_) {
		$hour = $_[1];	
		$min = $_[0];
	} else {
		($hour, $min) = (0, 0);
	}
			
	$select =~ s/(VALUE=\"$hour\")>/$1 SELECTED>/;

	$select .= "</SELECT>\n";

	# add the hours drop down to the time field
	$table .= $select . "</TD><TD><B>:</B></TD>\n<TD>\n";

	# start the minutes drop-down
	$select = "<SELECT NAME=\"minute\">\n";

	$select .= gen_options(0, 59, -1);

	$select =~ s/(VALUE=\"$min\")>/$1 SELECTED>/;

	$select .= "</SELECT>\n";

	$table .= $select . "</TD>\n</TR>\n</TABLE>\n";

	return $table;
}

sub read_cron
{
	my $file = shift;
	my $site = shift;

	my ($entry, $found) = ('', 0);
	my @entry = ();

	open CRON, "$file";

	while ($entry = <CRON>) {
		if ($entry =~ /$site$/) {
			$found = 1;
			last;
		}
	}

	close CRON;
	
	if ($found) {
		@entry = split /\s+/, $entry;
	}

	return @entry[0, 1, 2, 3, 4];
}

sub gen_options
{
	my $min = shift;
	my $max = shift;
	my @display = @_;

	my $nums = 1;

	if (@display && $display[0] != -1) {
		$nums = 0;
	}
	
	my $options;

	for (my $i = $min, my $j = 0; $i < ($max + 1); $i++, $j++) {
		my $text = $i;

		# pad minutes for time
		if ($display[0] == -1 && $i < 10) {
			$text = "0$i";
		}

		$options .= "<OPTION VALUE=\"$i\">" 
			. ($nums ? $text : $display[$j]) . "</OPTION>\n";
	}

	return $options;
}

sub edit_cron
{
	my ($in, $out, $job, $cgi, $add) = @_;

	my $site = $cgi->param('group');
	
	my $entry = $cgi->param('minute');

	my $hour = $cgi->param('hour');

	$entry .= " $hour";

	my $freq = $cgi->param('frequency');
	
	if ($freq =~ /^d/) {
		$entry .= ' * * *';
	} elsif ($freq =~ /^w/) {
		$entry .= ' * * ' . $cgi->param('days');
	} elsif ($freq =~ /^m/) {
		$entry .= ' ' . $cgi->param('days') . ' * *';
	}

	
	$entry .= " root $job\n";

	my $fin = 0;

	while (<$in>) {
		if (!/$site$/) {
			print $out $_;
		} elsif ($add) {
			print $out $entry;
			$fin = 1;
		}
	}

	print $out $entry unless ($fin || !$add);

	return 1;
}
