#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 2.0
# date: 7-4-99
#
# Copyright (C) 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::snmp;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Services;
use Cobalt::User;

use Locale::gettext;

# global variables
my ($q, $page, $locale, $header_style, $button_style, $table_style);
my $snmpScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/snmp';
my $servicesScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/services';

sub main {

$q=shift;

$page=$q->param( 'page' );

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
  $button_style=$locale->getProperty("buttonStyle");
  $header_style=$locale->getProperty("headerStyle");
  $table_style=$locale->getProperty("tableStyle");
 }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

if( $page ne 'snmp' ) { genSNMPPage( \*STDOUT ); }
else {
    my $err=0;
    my $info;

    # read form input
    my $readCommunity=$q->param( 'readcommunity' );
    my $readWriteCommunity=$q->param( 'readwritecommunity' );

    if( $readCommunity && ( $readCommunity=~/\s/ ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "readCommunityNotValid")."\"");
	$err=1;
    }
    elsif( $readWriteCommunity && ( $readWriteCommunity=~/\s/ ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "readWriteCommunityNotValid")."\"");
	$err=1;
    }
    elsif( $readCommunity eq $readWriteCommunity ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "communityNotUnique")."\"");
	$err=1;
    }
    else {
	my $ret=Cobalt::Services::set_snmp_community( $readCommunity, $readWriteCommunity );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if( !$err ) { print "location: $servicesScriptURL\n\n"; }
    else { genSNMPPage( \*STDOUT, $info ); }
}

}


#
# argument 0 is the file handle the page goes to
# argument 1 is the infomation shown to user
#
sub genSNMPPage {
    my ( $handle, $info )=@_;

    my ( $readCommunity, $readWriteCommunity )=Cobalt::Services::get_snmp_community();

    # Read in the template
    open( TEMPLATE, "$Template_dir/snmp.tmpl" )
          || die "Cannot read $Template_dir/snmp.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    while( <TEMPLATE> ) {
        s/\[READCOMMUNITY\]/$readCommunity/;
        s/\[SCRIPTURL\]/$snmpScriptURL/o;
        s/\[SERVICESSCRIPTURL\]/$servicesScriptURL/o;
        s/\[READWRITECOMMUNITY\]/$readWriteCommunity/;
        s/\[USER_INFO\]/$info/;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

        print $handle $_;
    }
    close TEMPLATE;
}

1;
