#!/usr/bin/perl
#
# Copyright 1998-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::userBulkAdd;

require Cobalt::Locale;
require Cobalt::User;
require Cobalt::Vsite;
require Cobalt::Util;
use Cobalt::Product;

use WebFS::FileCopy;

use Fcntl qw/O_RDONLY O_RDWR O_CREAT/;
use DB_File;
use Locale::gettext;
use Jcode;

# global variables
my ($q, $page, $locale, $group);
my $scriptURL;
my $scriptStatusURL;
my $siteListScriptURL;
my $exportURL;

# hide ourselves away in our own private little directory
my $statFile = "$Lockdir/bulkstat";

my $tmpdir = "$DB_dir_root/tmp";
mkdir($tmpdir,0700) unless (-d $tmpdir);

sub main {

  $q=shift;
  $page=$q->param(page);
  $group=$q->param(group);
  $info=$q->param(info);
  $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
  if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

  my $forbiddenURL="/.cobalt/error/forbidden.html";
  my $user = $ENV{REMOTE_USER};
  unless (Cobalt::User::user_authok($user, "site-adm", $group)) {
      print "location: $forbiddenURL\n\n";
      die "Unauthorized access by $user, not an admin.\n";
  }

  $siteListScriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/userBulkAdd\?group=$group";
  $scriptURL="/cgi-bin/.cobalt/turbo_ui/dispatch/userBulkAdd\?group=$group";
  $scriptStatusURL="/cgi-bin/.cobalt/turbo_ui/dispatch/userBulkAdd\?group=$group&page=bulkstatus";
  $exportURL="/cgi-bin/.cobalt/turbo_ui/dispatch/userlist\?group=$group";

  if( $page eq 'userimport' ) { &handleBulkAddPage(); }
  else {
      if ( $page eq 'bulkstatus' ) {
          &genBulkStatusPage();
      } else {
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

          &genBulkPage(\*STDOUT, $info);
      } 
  }
}


sub handleBulkAddPage {
    my $location=$q->param( 'location' );
    my $targetFile="$tmpdir/lst.lst";

    my $error=0;
    my $info='';

    if( $location eq 'url' ) {
	# get package file from an URL
	my $url=$q->param( 'url' );

	if( !$url ) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "noPackageURL")."\"");
	    $error=1;
	}
	else {
	    # get the file
	    my $httpResponse=( get_urls( $url ) )[ 0 ];

	    if( $httpResponse->is_success ) {
		# save the downloaded file
		open DOWNLOADED, ">$targetFile";
		print DOWNLOADED $httpResponse->content;
		close DOWNLOADED;
	    }
	    else {
		$info.=eval("\"".Locale::gettext::dgettext("ui", "packageDownloadFailed")."\"");
		$error=1;
	    }
	}
    }
    elsif( $location eq 'upload' ) {
	# upload package file from client machine
	my $file=$q->param( 'file' );

	if( !$file ) {
	    $info.=eval("\"".Locale::gettext::dgettext("ui", "noPackage")."\"");
	    $error=1;
	}
	#elsif( $file!~/\.pkg/o ) {
	#    $info.=eval("\"".Locale::gettext::dgettext("ui", "packageExtensionNotValid")."\"");
	#    $error=1;
	#}
	else {
	    # remove file for security reason
	    unlink $targetFile if (-e $targetFile);
	    system('/bin/mv', $q->tmpFileName($file), $targetFile);
	}
    }

    if (open(BULK, $targetFile)) {
	$error = 1 unless <BULK> =~ /\t/;
	close(BULK);
    } else {
	$error = 1;
    }

    unless ($error) {
      # start installation if everything is fine
      $group=$q->param(group);

      open STATFILE, "> $statFile";
      print STATFILE "0 0 0";
      close STATFILE;
      unlink "$statFile.rep" if (-e "$statFile.rep");

      # now fork and format everything.
      my $pid;
      FORK: {
          if ($pid = fork) {
              &genBulkStatusPage(); 
              exit 0;
          } elsif (defined $pid) {
              last;
          } elsif ($! =~ /No more process/) {
              sleep 5;
              redo FORK;
          } else {
              return "Can't fork: $!\n";
          }
      }

      # child process
      my $num_users=0;
      my $num_succ=0;
      my $num_fail=0;
      my $vsiteobj=new Cobalt::Meta::vsite;
      $vsiteobj->retrieve($group);
      my $siteMaxUsers = $vsiteobj->maxusers;
      my $user_quota   = $vsiteobj->user_quota;
      my $user_shell   = ($vsiteobj->user_shell) ? "on" : "off";
      my $user_fpx     = ($vsiteobj->user_fpx)   ? "on" : "off";
      my $user_apop    = ($vsiteobj->user_apop)  ? "on" : "off";
      $vsiteobj->DESTROY;

      if (-e $targetFile) {
          # Lock the user interface
          Cobalt::Util::lock_ui(1);

	  open BULK, "$targetFile";
	  my @user, @lines;
	  my $error=0;
	  
	  # Enforce site user counts
	  my @vsiteUsers = Cobalt::Vsite::vsite_get_users($group);
	  my $siteCurUsers = scalar(@vsiteUsers);
	  my $fn_enc = $locale->getProperty("fullname_enc");
	  
	  my $dbc = Cobalt::Meta::getdb;
	  $dbc->reset;

	  my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
	  my $importAsCharset = $locale->getProperty("userImportAsCharset");
      
	  while (<BULK>) {
	      # since macs use \r instead of \n perl will read a bunch of lines
	      # ended by \r as one line, so need to split them up ourselves
	      # here or only one user will get added
	      if ($importAsCharset eq "euc-jp") {
		  $_ = Jcode->new($_)->euc;
	      }
	      @lines = split /[\r\n]/, $_;
	      
	      for my $foo (@lines) {
			@user = split /[\t]/m, $foo;
	      		$user[2] =~ s/[\n\r]//g;
	      		next if ($user[0] !~ /[\S]/);
	      		my $obj = new Cobalt::Meta(type       =>  "users",
						 group      =>  $group,
						 name       =>  $user[0],
						 fullname   =>  $user[1],
						 altname    =>  $user[4],
						 password1  =>  $user[2],
						 password2  =>  $user[2],
						 aliases    =>  $user[3],
						 quota      =>  $user_quota,
						 shell      =>  $user_shell,
						 fpx        =>  $user_fpx,
						 apop       =>  $user_apop,
						 admin      =>  "off",
						 suspend    =>  "off",
						 vacation   =>  "off",
						 forward    =>  "");
		      
		      my $ret;
		      if ($siteCurUsers+$num_succ >= $siteMaxUsers) {
			  $ret=Cobalt::Util::MSG_get_ui("list_max_bulkuser");
		      } else {
			  $ret = Cobalt::User::site_user_add($obj);
			  Cobalt::User::user_fullname_enc($obj->getname, $fn_enc) if ($fn_enc);
		      }
		      $num_users++;
		      $obj->DESTROY;
	
		      my $info;
		      if ($ret) { # generate error message
			  open REPFILE, ">>$statFile.rep"; 
			  print REPFILE "<TR><TD COLSPAN=\"2\" WIDTH=\"100\">$user[0]</TD><TD> $ret</TD></TR>";
			  close REPFILE;
			  $num_fail++;
		      } else {
			  $num_succ++;
		      }
		      
		      open STATFILE, "> $statFile";
		      print STATFILE "$num_users $num_succ $num_fail";
		      close STATFILE;
		}
	  }
	  close BULK;
	  undef($importAsCharset);
      }

      open STATFILE, "> $statFile";
      print STATFILE "DONE $num_fail $num_users $num_succ";
      close STATFILE;
    } else {
      my $ret=Cobalt::Util::MSG_get_ui("corrupt_bulkuser");
      $info="'$ret', 1000, 'error'";
      Cobalt::Util::lock_ui(0);
      print "location: $siteListScriptURL&info=$info\n\n";
    }
  
    Cobalt::Util::lock_ui(0);
    print "location: $siteListScriptURL&info=$info\n\n";
    exit 0; 
}


#
# argument 0 is the file handle the page goes to
# argument 1 is the information to show to user
#
sub genBulkPage {
    my($handle, $info)=@_;

    if (!$info) { $info=''; }
    # Read in the template
    if (-e "$Template_dir/userBulkAdd.tmpl") {
	open TEMPLATE, "$Template_dir/userBulkAdd.tmpl";

	while (<TEMPLATE>) {
	    s/\[SCRIPTURL\]/$scriptURL/;
	    s/\[EXPORTURL\]/$exportURL/;
	    s/\[GROUP\]/$group/;
	    s/\[SITEUSERLISTSCRIPTURL\]/$siteListScriptURL/;
	    s/\[USER_INFO\]/$info/;
            s/\[FONTH\]/$header_style/g;
            s/\[FONTT\]/$table_style/g;
            s/\[FONTB\]/$button_style/g;
	    
	    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	    print $handle $_;
	}

	close TEMPLATE;
    }
}


sub genBulkStatusPage {

    # Read in the status
    my (@users_added,$ntot,$stot,$ftot);
    if (-e $statFile) {
	open STAT, "$statFile";
	while (<STAT>) {
	    @users_added = split /\s+/m, $_;
	}
	close STAT;
    }

    if ($users_added[0] =~ /^DONE/) {
      my $num_fail=$users_added[1];
      my $num_tot=$users_added[2];
      my $num_succ=$users_added[3];

      if ($num_fail == 0) { 
        my $msg=Locale::gettext::dgettext("ui", "bulkSuccess");
        $info="'<CENTER>$num_succ $msg</CENTER>', 1000, 'status'";
        print "location: $siteListScriptURL&info=$info\n\n";
        unlink "$statFile" if (-e $statFile);
        unlink "$statFile.rep" if (-e "$statFile.rep");
        return 0;
      }

      if (-e "$Template_dir/bulkAddDone.tmpl") {
	  open TEMPLATE, "$Template_dir/bulkAddDone.tmpl";
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


	  while (<TEMPLATE>) {
	      if ($_ =~ /\[REPORT\]/) {
		  if (-e "$statFile.rep") {
		      open REPFILE, "$statFile.rep";
		      while (<REPFILE>) {
			  print $_;
		      }
		      close REPFILE;
		  }
	      } else {
		  s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;
		  s/\[SITEUSERLISTSCRIPTURL\]/$siteListScriptURL/o;
		  s/\[FAILED\]/$num_fail/o;
		  s/\[TOT\]/$num_tot/o;
		  s/\[USER_INFO\]/$info/o;
                  s/\[FONTH\]/$header_style/go;
                  s/\[FONTT\]/$table_style/go;
                  s/\[FONTB\]/$button_style/go;

		  print STDOUT $_;
	      }          
	  }
	  close TEMPLATE;
      }
      #unlink "$statFile" if (-e $statFile);
      #unlink "$statFile.rep" if (-e "$statFile.rep");
      return 0;
    }

    if (-e "$Template_dir/bulkAddStatus.tmpl") {
	open TEMPLATE, "$Template_dir/bulkAddStatus.tmpl";

	($users_added[0] != 1) ? $ntot = 's' : $ntot = '';
	($users_added[1] != 1) ? $stot = 's' : $stot = '';
	($users_added[2] != 1) ? $ftot = 's' : $ftot = '';
	
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


	while (<TEMPLATE>) {
	    s/\[SCRIPTURL\]/$scriptStatusURL/o;
            s/\[FONTH\]/$header_style/g;
            s/\[FONTT\]/$table_style/g;
            s/\[FONTB\]/$button_style/g;
	    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	    s/\[SUCCESS\]/$users_added[1]/;
	    s/\[FAIL\]/$users_added[2]/;
	    s/\[NUMUSERS\]/$users_added[0]/;
	    s/\[NS\]/$ntot/;
	    s/\[SS\]/$stot/;
	    s/\[FS\]/$ftot/;

	    print $_;
	}

	close TEMPLATE;
    }
}


1;
