#!/usr/bin/perl

#
# works only within personal profile
# split personality admin|alteradmin vs. non-admin
#
# Copyright (C) 1999-2000, Cobalt Networks, Inc
# All rights reserved

package Cobalt::UI::userMod;

use strict;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Email;
use Cobalt::Meta;

use Locale::gettext;

use vars qw( $DEBUG );
my ($q,$locale,$page,$button_style,$header_style,$table_style);
my $userModScriptDir="$AdminCGIs_dir/userMod/";
my $userModScriptURL='/cgi-bin/.cobalt/turbo_ui/dispatch/userMod';

sub main {

# global variables
$DEBUG = 0;
$q=shift;

$page=$q->param('page');
my $info = ($q->param('err') eq 'yes') ? ("$ENV{'USERERROR'}") : ("");

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if ($locale) { $ENV{LANGUAGE}=$locale->getLocale;
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }


if( $page eq 'usermod' ) { handleUserModPage(); }
else { genUserModPage($info); }

}

#
# event handler for modify user page
#
sub handleUserModPage {
    my $userName=$ENV{ REMOTE_USER };

    my $err=0;
    my $info;

    # read form input
    my $fullName=$q->param( 'fullname' );
    my $altName=$q->param( 'altname' );
    my $password1=$q->param( 'password1' );

    # error check
    my @passwordInBytes=unpack( 'C*', $password1 );
    my $passwordBadChar=0;
    my $byte;
    foreach $byte ( @passwordInBytes ) {
	if( $byte>127 ) { $passwordBadChar=1; last; }
    }

    if( !$fullName ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "noFullName")."\"");
	$err=1;
    }
    elsif( $passwordBadChar ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "passwordBadChar")."\"");
	$err=1;
    }
    elsif( $fullName=~/\'/o || $fullName=~/\"/o ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "noQuotesInFullName")."\"");
	$err=1;
    }
    else {
	# everything is fine

	my $ret=user_mod( $userName, $fullName, $password1 );
        print STDERR "Password is: $password1" if ($DEBUG);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	my @change = ("fullname","$fullName");
	if ($locale->getProperty("altname")) {
	    push @change, ("altname","$altName");
	}
	$ret=Cobalt::Meta::update("type" => "users",
				  "change" => \@change,
				  "where" => ["name","=","$userName"]);

	if ((my $fn_enc=$locale->getProperty("fullname_enc"))) {
	    $ret=Cobalt::User::user_fullname_enc($userName,$fn_enc);
	}

	# update the APOP data base if they changed their password
	my $apop = mail_apop_isuser($userName);
	if ($password1 ne '' && $apop) {
	    $ret=mail_apop_adduser ($userName, $password1);
	    if( $ret!~/^2/o ) {
		$info.=substr( $ret, 4 );
		$err=1;
	    }
	}

    }

    if($err) {
	$ENV{"USERERROR"}="$info";
	print "location: /cgi-bin/.cobalt/turbo_ui/dispatch/userMod?err=yes\n\n";
    } else {
	print "location: /cgi-bin/.cobalt/turbo_ui/dispatch/userMod\n\n";
    }
}

#
# argument 0 is the infomation shown to user
#
sub genUserModPage {
    my ($info)=@_;

    my $userName=$ENV{ REMOTE_USER };

    my $userObj = Cobalt::Meta->new("type" => "users");
    $userObj->retrieve("$userName");

    my $full=$userObj->get("fullname");
    my $fullName="$full";

    my ($altname,$begin_altname,$end_altname);
    if ($locale->getProperty("altname")) {
	my $alt = $userObj->get("altname");
	$altname = "$alt";
	$begin_altname = "-->";
	$end_altname = "<!--";
    }

    # Read in the template
    open( TEMPLATE, "$Template_dir/userMod.tmpl" )
	|| die "Cannot read $Template_dir/userMod.tmpl: $!\n";

    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);


    while( <TEMPLATE> ) {
	s/\[ALTNAME\]/$altname/;
	s/\[ALTNAME_BEGIN\]/$begin_altname/;
	s/\[ALTNAME_END\]/$end_altname/;
	s/\[FULLNAME\]/$fullName/;
	s/\[USER_INFO\]/$info/;
	s/\[USERMODSCRIPTURL\]/$userModScriptURL/go;
	s/\[USERNAME\]/$userName/g;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;


	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;

	print;
    }
    close TEMPLATE;
}
1;
