#!/usr/bin/perl

# Site HTTP daemon control and configuration
#
# Copyright 2000 Cobalt Networks, Inc
# Timothy Stonis <tim@cobaltnet.com>
# Version 1.0

package Cobalt::UI::webparam;

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::User;
use Cobalt::Meta;
use Cobalt::Util;
use Cobalt::Http;

use DB_File; 		# UI database reporting
use Fcntl qw(O_RDWR O_CREAT O_RDONLY);
use Locale::gettext;

## Globals
my($serviceScriptPath)="/cgi-bin/.cobalt/turbo_ui/dispatch/siteList";
my($template)="$Template_dir/webParam.tmpl";
my($webparam_script)="/cgi-bin/.cobalt/turbo_ui/dispatch/webparam";
my $forbiddenURL="/.cobalt/error/forbidden.html";

my ($group,$q,$page,$locale, $siteObj, $maxUsers, $quota, $ftpquota, $ftpusers, $anon, $ipaddr, $button_style, $header_style, $table_style);

sub main {

$q=shift;
$group = $q->param('group');
$page=$q->param('page');
my $user = $ENV{REMOTE_USER};

unless (user_authok($user, "admin")) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorized access by $user, not an admin.\n";
}

$locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale();
                $button_style=$locale->getProperty("buttonStyle");
                $header_style=$locale->getProperty("headerStyle");
                $table_style=$locale->getProperty("tableStyle"); }

my $ret;
if ($page eq 'webconfig') {
    update_config();
} else {
    generate_config();	# Create the form
}

}
## fin main()


## Subs
sub update_config
{

    # Get the old state 
    my $minspare = Cobalt::Http::get_http_param("MinSpareServers");
    my $maxspare = Cobalt::Http::get_http_param("MaxSpareServers");
    my $maxclient = Cobalt::Http::get_http_param("MaxClients");
    my $hostlookup = Cobalt::Http::get_http_param("HostnameLookups");

    $newmaxspare=$q->param('MAXSPARE');
    $newminspare=$q->param('MINSPARE');
    $newclient=$q->param('MAXCLIENT');
    $newhostname=$q->param('hostnamelookup');
    if ($newhostname eq '') { $newhostname = "off" };

    

    # Do some checking
    if ( ($newmaxspare <= 0) || ($newminspare <= 0) || ($newclient <=0)) {
      generate_config(MSG_get_ui("http_greater_than_zero"));
      return;
    } 
    if ( $newminspare > $newmaxspare ) {
      generate_config(MSG_get_ui("http_max_less_min"));
      return; }

    if ( $newmaxspare > $newclient ) {
      generate_config(MSG_get_ui("http_client_more_max"));
      return; }

    my $numserv = &maximum_procs();
 
    if ( $newminspare > $numserv ) {
      generate_config(MSG_get_ui("http_over_recommend"));
      return; }
    if ( $newmaxspare > $numserv ) {
      generate_config(MSG_get_ui("http_over_recommend"));
      return; }
 
    if ( $newclient > ($numserv*2) ) {
      generate_config(MSG_get_ui("http_over_recommend"));
      return; }

    Cobalt::Http::set_http_param("MinSpareServers",$newminspare);
    Cobalt::Http::set_http_param("MaxSpareServers",$newmaxspare);
    Cobalt::Http::set_http_param("MaxClients",$newclient);
    Cobalt::Http::set_http_param("HostnameLookups",$newhostname);

    Cobalt::Http::http_restart(1, 1); # The second argument indicates hard-restart

    generate_config();

    return;
}

sub maximum_procs
{ 
    my $totmem;
    open( MEM, "/proc/meminfo" );
    while (<MEM>) {
      if (/^MemTotal:\s+(\d+)/) {
        $totmem=$1
      }
    }
    close(MEM);
    if (!$totmem) { $totmem = 65536; }
 
    # Adrian and Tim's special formula
    return( int((($totmem/1024)-30)/1.5) );
} 

sub generate_config
{ 
    my($info)=@_;
    my $err=$info if ($info); 

    # Get the params
    my $minspare = Cobalt::Http::get_http_param("MinSpareServers");
    my $maxspare = Cobalt::Http::get_http_param("MaxSpareServers");
    my $maxclient = Cobalt::Http::get_http_param("MaxClients");
    my $hostlookup = Cobalt::Http::get_http_param("HostnameLookups");
    if ($hostlookup eq "on")  { $hostlookup = "CHECKED"; } else {
      $hostlookup = ""; }

    my $numserv = &maximum_procs();
    my $numserv2 = 2*$numserv;
 
    # display page
    my $locale = Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
    my $charset = $locale->getProperty("http_Charset");
    my $charsetText;
    if ($charset) {
	$charsetText = "; charset=$charset";
    }
    print "Content-type:  text/html$charsetText\nCache-Control: no-cache\n\n";
    undef($charset, $charsetText);

    open(TMPL,"$template") || die "Can not open template $editTemplate for reading: $!";
    while(<TMPL>)
    {
	s/\[SHOWINFO\]/$info/g;
	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/ge;
	s/\[MAXSPARE\]/$maxspare/g;
	s/\[MINSPARE\]/$minspare/g;
	s/\[MAXCLIENT\]/$maxclient/g;
	s/\[HOSTCHECKED\]/$hostlookup/g;
	s/\[LIMIT\]/$numserv/g;
	s/\[2XLIMIT\]/$numserv2/g;
	s/\[IAM\]/$webparam_script/g;
        s/\[FONTH\]/$header_style/g;
        s/\[FONTT\]/$table_style/g;
        s/\[FONTB\]/$button_style/g;
	
	print;
    }
    close(TMPL);

} 

1;
